function PaymentService(
	ApplicationContext, PaymentRepository, Query, PaymentPayAccount, OperatorRepository, IntegrationService, PrintTEFVoucher,
	GetConsumerLimit, ZHPromise, AccountSaleCode, ChargePersonalCredit, CartPricesRepository, PrinterService, QRCodeSaleRepository,
	ScreenService, SavePayment, RemovePayment, ParamsParameterRepository, WiBXConvertValue, WiBXRequestPayment, WiBXCancelPayment,
	WiBXPaymentStatus, GetStatusAdiantamento, GetAdvancedPayment, CallCrmBunus, ValidaTipoConsReceb, ValidaVendaMesa, PerifericosService, GetTipoProd,
	BuscaSaldoCartaoTodo, UpdatePayment, UndoPayment){
	var self = this;

	/* Tipos de pagamento que realiza transação eletrônica por integração
		1 -> Credito
		2 -> Debito
		F/G -> Voucher
		H -> Mercado Pago
	*/
	this.PAYMENT_INTEGRATION = ['1', '2', 'F', 'G', 'H', 'D', 'E'];

	this.PAYMENT_CREDIT_DEBIT = ['9', 'A'];

	this.isOnSale = false;

	this.initializePayment = function (accountData, params, accountDetails, ITEMVENDA, NRLUGARMESA, ITVENDADES, payments, PRODSENHAPED) {
		return self.formatPaymentData(accountData, params, accountDetails, ITEMVENDA, NRLUGARMESA, ITVENDADES, payments, PRODSENHAPED).then(function (paramsPayment) {
			IntegrationService.waitingIntegration = Array();
			return PaymentRepository.save(paramsPayment);
		});
	};

    this.convertWiBX = function(value){
        return AccountSaleCode.findOne().then(function (saleCode){
            var query = Query.build()
                .where('value').equals(value)
                .where('saleCode').equals(saleCode);

            return WiBXConvertValue.download(query);
        });
    };

    this.paymentWiBX = function(value){
        return AccountSaleCode.findOne().then(function (saleCode){
            var query = Query.build()
                .where('value').equals(value)
                .where('saleCode').equals(saleCode);

            return WiBXRequestPayment.download(query);
        });
    };

    this.cancelWiBX = function(){
        return AccountSaleCode.findOne().then(function (saleCode){
            var query = Query.build()
                .where('saleCode').equals(saleCode);

            return WiBXCancelPayment.download(query);
        });
    };

    this.statusWiBX = function(){
        return AccountSaleCode.findOne().then(function (saleCode){
            var query = Query.build()
                .where('saleCode').equals(saleCode);

            return WiBXPaymentStatus.download(query);
        });
    };

    this.formatPaymentData = function(accountData, params, accountDetails, ITEMVENDA, NRLUGARMESA, ITVENDADES, payments, PRODSENHAPED){
        return new Promise(function(resolve){
            return OperatorRepository.findOne().then(function(operatorData){
                var CDCLIENTE      = !!accountData.CDCLIENTE ? accountData.CDCLIENTE : '';
                var NMRAZSOCCLIE   = !!accountData.NMRAZSOCCLIE ? accountData.NMRAZSOCCLIE : '';
                var CDCONSUMIDOR   = !!accountData.CDCONSUMIDOR ? accountData.CDCONSUMIDOR : '';
                var NMCONSUMIDOR   = !!accountData.NMCONSUMIDOR ? accountData.NMCONSUMIDOR : '';
                var DSEMAILCONS    = !!accountData.DSEMAILCONS ? accountData.DSEMAILCONS : '';
                var CDVENDEDOR     = null;
                var NMFANVEN       = null;
                var NRMESA         = !!accountData.NRMESA ? accountData.NRMESA : '';
                var NRPESMESAVEN   = !!accountData.NRPESMESAVEN ? accountData.NRPESMESAVEN : '';
                var NRVENDAREST    = !!accountData.NRVENDAREST ? accountData.NRVENDAREST : '';
                var NRCOMANDA      = !!accountData.NRCOMANDA ? accountData.NRCOMANDA : '';
                var CREDITOPESSOAL = !!accountData.CREDITOPESSOAL ? accountData.CREDITOPESSOAL : '';
                var CDFAMILISALD   = !!accountData.CDFAMILISALD ? accountData.CDFAMILISALD : '';
                var VRRECARGA      = !!accountData.VRRECARGA ? accountData.VRRECARGA : '';
                var numeroProdutos = !!accountDetails.numeroProdutos ? accountDetails.numeroProdutos : 1;
                var TOTALVENDA     = accountDetails.vlrtotal;
                var VRTXSEVENDA    = accountDetails.vlrservico;
                var VRCOUVERT      = accountDetails.vlrcouvert;
                var TOTALSUBSIDY   = accountDetails.totalSubsidy;
                var REALSUBSIDY    = accountDetails.realSubsidy;
                var VALORPAGO      = accountDetails.vlrpago;
                var TOTAL          = parseFloat((accountDetails.vlrprodutos - accountDetails.vlrdesconto).toFixed(2));
                var IDTPVENDACONS  = null;
                var CDSUPERVISORs  = !!accountDetails.CDSUPERVISORs ? accountDetails.CDSUPERVISORs : '';
                var logServico     = !!accountDetails.logServico ? accountDetails.logServico : '';
                var CDSUPERVISORd  = !!accountDetails.CDSUPERVISORd ? accountDetails.CDSUPERVISORd : '';
                var logDesconto    = !!accountDetails.logDesconto ? accountDetails.logDesconto : '';
				var NRCPFFIDUDS    = accountDetails.NRCPFFIDUDS||null;
				var VRDESCCASHUDS   = accountDetails.VRDESCCASHUDS||null;
				var VRDESCFID   	= accountDetails.VRDESCFID||null;
				var NRCPFINTEPROMO  = accountDetails.NRCPFINTEPROMO||null;
				var IDPEDINTEPROMO  = accountDetails.IDPEDINTEPROMO||null;

                if(!_.isEmpty(payments)) {
                    var bindedGetCustomReceObject = _.bind(self.getCustomReceObject, this, operatorData);
                    payments = _.map(payments, bindedGetCustomReceObject);
                }

                var result = {
                    TIPORECE: payments == null ? Array() : payments,
                    ITEMVENDA: ITEMVENDA,
                    ITVENDADES: ITVENDADES,
                    PRODSENHAPED: PRODSENHAPED,
                    DATASALE: {
                        // valores da venda
                        TOTALVENDA: TOTALVENDA,
                        FALTANTE: TOTALVENDA,
                        VALORPAGO: 0,
                        ANTECIPADOS: VALORPAGO,
                        TROCO: 0,
                        REPIQUE: 0,
                        TOTAL: TOTAL,
                        TOTALSUBSIDY: TOTALSUBSIDY,
                        REALSUBSIDY: REALSUBSIDY,
                        // taxa de serviço
                        VRTXSEVENDA: VRTXSEVENDA,
                        VRTXSEORIG: VRTXSEVENDA,
                        // couvert
                        VRCOUVERT: VRCOUVERT,
                        // desconto
                        VRDESCONTO: 0,
                        PCTDESCONTO: 0,
                        TIPODESCONTO: 'P',
                        VRDESCONTOTAX: 0,
                        PCTDESCONTOTAX: 0,
                        TIPODESCONTOTAX: 'P',
                        fidelityDetails: null,
                        FIDELITYDISCOUNT: accountDetails.fidelityDiscount,
                        cashbackUsed4ALL: accountDetails.cashbackUsed4ALL,
                        FIDELITYVALUE: accountDetails.fidelityValue,
                        CPF4ALL: null,
                        CODPROMO: null
                    },
                    CDCLIENTE: CDCLIENTE,
                    NMRAZSOCCLIE: NMRAZSOCCLIE,
                    CDCONSUMIDOR: CDCONSUMIDOR,
                    NMCONSUMIDOR: NMCONSUMIDOR,
                    CDVENDEDOR: CDVENDEDOR,
                    NMFANVEN: NMFANVEN,
                    NRMESA: NRMESA,
                    NRPESMESAVEN: NRPESMESAVEN,
                    NRVENDAREST: NRVENDAREST,
                    NRCOMANDA: NRCOMANDA,
                    CREDITOPESSOAL: CREDITOPESSOAL,
                    CDFAMILISALD: CDFAMILISALD,
                    VRRECARGA: VRRECARGA,
                    chave: params.chave,
                    NRLUGARMESA: NRLUGARMESA,
                    numeroProdutos: numeroProdutos,
                    servico: {
                        CDSUPERVISOR: CDSUPERVISORs,
                        logServico: logServico
                    },
                    desconto: {
                        CDSUPERVISOR: CDSUPERVISORd,
                        logDesconto: logDesconto
                    },
                    DELIVERY: false,
                    DSEMAILCONS: DSEMAILCONS,
					NRCPFFIDUDS:NRCPFFIDUDS,
					VRDESCCASHUDS:VRDESCCASHUDS,
					VRDESCFID:VRDESCFID,
					NRCPFINTEPROMO:NRCPFINTEPROMO,
					IDPEDINTEPROMO:IDPEDINTEPROMO
                };

                self.calcTotalSale(result, true);

                // Caso parte da conta já tiver sido paga com cartão, ajusta o valor total e faltante.
                result.DATASALE.TOTALVENDA = parseFloat((result.DATASALE.TOTALVENDA + result.DATASALE.VALORPAGO).toFixed(2));
                result.DATASALE.FALTANTE = parseFloat((result.DATASALE.FALTANTE + result.DATASALE.VALORPAGO).toFixed(2));

                // Valor utilizado caso seja necessário o recálculo do total, levando em consideração adiantamentos.
                result.DATASALE.ANTECIPADOS = parseFloat((result.DATASALE.ANTECIPADOS + result.DATASALE.VALORPAGO).toFixed(2));

                // Verifica parametros e se houve alteração da taxa de serviço para mostrar o label do repique na tela de recebimento.
                if (operatorData.IDTPCONTRREPIQ !== 'N' && !CREDITOPESSOAL) {
                    ParamsParameterRepository.findOne().then(function (params) {
                        // Se houve alteração no valor da taxa de serviço o repique é desabilitado.
                        if (operatorData.modoHabilitado !== 'B' && operatorData.IDCOMISVENDA !== 'N') {
                            result.showRepique = (accountDetails.vlrservoriginal == accountDetails.vlrservico) ? true : false;
                        } else {
                            result.showRepique = true;
                        }
                    }.bind(this));
                } else {
                    result.showRepique = false;
                }

                if (CDCLIENTE != '' && CDCONSUMIDOR != ''){
                    self.getConsumerLimit(CDCLIENTE, CDCONSUMIDOR, 'all').then(function(limits){
                        result.limitDebito = limits[0].debito;
                        result.limitCredito = limits[0].credito;
                        result.IDTPVENDACONS = limits[0].IDTPVENDACONS;
                        resolve(result);
                    });
                }
                else {
                    resolve(result);
                }
            }.bind(this));
        });
    };

	this.getCustomReceObject = function(operatorData, payment) {
		var date = new Date(payment.DTHRINCMOV);
		date = moment(date).format('L') + ' ' + moment(date).format('LTS');

		return {
			'VRMOVIVEND': parseFloat(payment.VRMOV),
			'TRANSACTION': {
				'data': {
					'NRCOMANDA': payment.NRCOMANDA||null,
					'NRVENDAREST': payment.NRVENDAREST||null,
					'IDSTATUSMOV': payment.IDSTATUSMOV||null,
					'NRSEQMOVMOB': payment.NRSEQMOVMOB||null,
					'AUTHKEY': null,
					'CDLOJATEF': operatorData.CDLOJATEF,
					'CDTERTEF': operatorData.CDTERTEF,
					'DSENDIPSITEF': operatorData.DSENDIPSITEF,
					'CDBANCARTCR': payment.DSBANDEIRA,
					'CDNSUHOSTTEF': payment.CDNSUTEFMOB,
					'IDTIPORECE': payment.IDTIPORECE,
					'IDTPTEF': payment.IDTPTEF,
					'NRCARTBANCO': payment.NRCARTBANCO,
					'NRCONTROLTEF': payment.NRADMCODE,
					'PAYMENTCONFIRMATION': false,
					'REMOVEALLINTEGRATIONS': false,
					'STLPRIVIA': payment.TXPRIMVIATEF,
					'STLSEGVIA': payment.TXSEGVIATEF,
					'NRLUGARMESA': payment.NRLUGARMESA,
					'TRANSACTIONDATE': date.split(' ')[0].replace(/\//g, ''),
					'VRMOVIVEND': parseFloat(payment.VRMOV),
					'TRANSACTIONCODE': '',
					'TRANSACTIONID':''
				},
				'status': true
			},
			'CDTIPORECE': payment.CDTIPORECE,
			'DSBUTTON': payment.DSBUTTON,
			'IDDESABTEF': "N",
			'IDTIPORECE': payment.IDTIPORECE,
			'IDTPTEF': payment.IDTPTEF,
			'NRBUTTON': "",
			'DTHRINCOMVEN': date,
			'IDADIANTAMENTO': payment.IDADIANTAMENTO
		};
	};

	this.promiseFormatPaymentData = function(paymentData){
		var defer = ZHPromise.defer();
		defer.resolve(paymentData);
		return defer.promise;
	};

	this.getPaymentValue = function () {
		return PaymentRepository.findOne().then(function (payment) {
			return payment.DATASALE;
		});
	};

	this.findAllPayment = function () {
		return PaymentRepository.findOne().then(function (payment) {
			return payment.TIPORECE;
		});
	};

    this.handlePayment = function (currentRow) {
        // verifica se caixa e pagamento selecionado faz integração
        return self.checkIfMustCallIntegration(currentRow.tiporece).then(function (mustCallIntegration) {
            if (mustCallIntegration) {
                return ApplicationContext.UtilitiesService.logVendaTef('Pagamento por meio da integração iniciado, utilizando IDTIPORECE: ' + currentRow.tiporece.IDTIPORECE + '.').then(function() {
                    window.logVendaTef = ApplicationContext.UtilitiesService.logVendaTef;
                    currentRow.eletronicTransacion.data.CDNSUHOSTTEF = !_.isEmpty(currentRow.eletronicTransacion.data.CDNSUHOSTTEF) ? currentRow.eletronicTransacion.data.CDNSUHOSTTEF : null;
                    currentRow.eletronicTransacion.data.IDSTATUSMOV = 'P'; // pendente
                    currentRow.eletronicTransacion.data.IDTPTEF = currentRow.eletronicTransacion.data.IDTPTEF||operatorData.IDTPTEF;
                    if (!Util.isDesktop()){
                        return self.savePayment(currentRow).then(function(paymentData){
                            // chama integração
                            return IntegrationService.integrationPayment(currentRow).then(function (integrationResult) {
                                if (!integrationResult.error) {
                                    return ApplicationContext.UtilitiesService.logVendaTef('Pagamento realizado com sucesso. Dados da transação: ' + currentRow.eletronicTransacion.data + '.').then(function() {
                                        integrationResult.data.NRSEQMOVMOB = paymentData[0].NRSEQMOVMOB;
                                        integrationResult.data.eletronicTransacion.data.NRSEQMOVMOB = paymentData[0].NRSEQMOVMOB;
                                        integrationResult.data.eletronicTransacion.data.IDSTATUSMOV = null;
                                        return self.getDataUpdatePayment().then(function(retorno){
                                            return self.updatePayment(retorno.paymentData, integrationResult.data).then(function(){
                                                return self.handlePrintPayment(integrationResult.data.eletronicTransacion.data).then(function(){
                                                    if(!!currentRow.adiantamento) return self.setPaymentSaleAdvance(integrationResult.data);
                                                    else return self.setPaymentSale(integrationResult.data);
                                                });
                                            });
                                        });
                                    });
                                } else {
                                    return self.getDataUndoPayment().then(function(data){
                                        return self.undoPayment(data.NRSEQMOVMOB,data.paymentData).then(function (){
                                            return ApplicationContext.UtilitiesService.logVendaTef('Erro no pagamento. Mensagem: ' + integrationResult.message2 + '.').then(function() {
                                                return integrationResult;
                                            });
                                        });
                                    });
                                }
                            });
                        });
                    }
                    else {
                        return {
                            "error" : true,
                            "message" : "Não foi possível chamar a integração. Sua instância não existe.",
                            "data":{}
                        };
                    }
                });
            } else {
                // pagamento sem integração
                if(!!currentRow.adiantamento) return self.setPaymentSaleAdvance(currentRow);
                else return self.setPaymentSale(currentRow);
            }
        }.bind(this));
    };

	this.handlePrintPayment = function(dataPrinter) {
		return new Promise(function(resolve) {

		    const printPriSegVia = IntegrationService.getPrintPriSegVia();

            if (printPriSegVia.indexOf(dataPrinter.IDTPTEF) != -1) {
				var tefObject = {
					TEFVOUCHER: [{
						STLPRIVIA: dataPrinter.STLPRIVIA,
						STLSEGVIA: ''
					}]
				};

				self.handlePrintReceipt(tefObject, false);

				ScreenService.confirmMessage(
					'Deseja imprimir a via do cliente?', 'question',
					function(){
						var tefPrintVoucher = tefObject.TEFVOUCHER[0];
						tefPrintVoucher.STLPRIVIA = '';
						tefPrintVoucher.STLSEGVIA = dataPrinter.STLSEGVIA;
						self.handlePrintReceipt(tefObject);
						resolve();
					}.bind(this),
					function(){
						resolve();
					}.bind(this)
				);
			} else {
			    resolve();
			}

		});
	};

	this.savePayment = function(currentPayment) {
		return new Promise(function(resolve) {
			return PaymentRepository.findOne().then(function(paymentData) {
				var query = Query.build()
					.where('paymentData').equals(paymentData)
					.where('currentPayment').equals(currentPayment);

				return SavePayment.download(query).then(function(paymentData){
					resolve(paymentData);
				});
			}.bind(this));
		});
	};

    this.checkIfMustCallIntegration = function (tiporece) {
        return OperatorRepository.findOne().then(function (operatorData) {
            return tiporece.IDDESABTEF != 'S' && _.includes(self.PAYMENT_INTEGRATION, tiporece.IDTIPORECE) && operatorData.IDUTILTEF == 'T';
        }.bind(this));
    };

	this.setPaymentSale = function (currentRow) {
		return OperatorRepository.findOne().then(function(operatorData) {
			return PaymentRepository.findOne().then(function (payment) {
				// seta recebimento
				self.formatPriceChart(payment.TIPORECE, currentRow);
				// calcula valor pago no total da venda
				self.calcTotalSale(payment, false);
				// salva modificações do recebimento
				return PaymentRepository.save(payment).then(function () {
					return self.statusPaymentReturn(false, '', payment.DATASALE);
				}.bind(this));
			}.bind(this));
		}.bind(this));
	};

	this.setPaymentSaleAdvance = function(currentRow) {
		var amountAdvanced = 0;
		return OperatorRepository.findOne().then(function(operatorData) {
			return PaymentRepository.findOne().then(function (payment) {
				return self.getAdvancedPayment(operatorData.chave, payment.NRVENDAREST, payment.NRCOMANDA).then(function(advancedPaymentData) {

					if(!_.isEmpty(advancedPaymentData)){
						_.forEach(advancedPaymentData, function(actualAdvancedData) {
							amountAdvanced += parseFloat(actualAdvancedData.VRMOV);
						});
					}

					// seta recebimento
					self.formatPriceChart(payment.TIPORECE, currentRow);
					// calcula valor pago no total da venda
					self.calcTotalSale(payment, false);
					// salva modificações do recebimento
					return PaymentRepository.save(payment).then(function () {
						return self.statusPaymentReturn(false, '', payment.DATASALE);
					}.bind(this));
				}.bind(this));
			}.bind(this));
		}.bind(this));
	};

	this.formatPriceChart = function (tiporece, currentRow) {
		var button = currentRow.tiporece;
		var onPaymentType = _.find(tiporece, function (editing) {
			return editing.CDTIPORECE === button.CDTIPORECE;
		});

		// pagamentos que realizam integração nunca se sobrescrevem
		if (_.includes(self.PAYMENT_INTEGRATION, button.IDTIPORECE) || _.isEmpty(onPaymentType)) {
			tiporece.push(self.defaultPaymentType(button, currentRow));
		}
		else if (_.includes(self.PAYMENT_CREDIT_DEBIT, button.IDTIPORECE)) {
			onPaymentType.VRMOVIVEND = currentRow.VRMOVIVEND;
			onPaymentType.DTHRINCOMVEN = self.dateTime();
		}
		else {
			onPaymentType.VRMOVIVEND += currentRow.VRMOVIVEND;
			onPaymentType.DTHRINCOMVEN = self.dateTime();
		}
	};

	this.defaultPaymentType = function (button, currentRow) {
		cartaoTodo = button.cartaoTodo||null;
		if(currentRow.tiporece && !cartaoTodo){
			if(currentRow.tiporece.cartaoTodo){
				cartaoTodo = currentRow.tiporece.cartaoTodo;
			}
		}
		return {
			CDTIPORECE: button.CDTIPORECE,
			IDTIPORECE: button.IDTIPORECE,
			DSBUTTON: button.DSBUTTON,
			VRMOVIVEND: currentRow.VRMOVIVEND,
			REPIQUE: currentRow.REPIQUE > 0 ? currentRow.REPIQUE : 0,
			TRANSACTION: currentRow.eletronicTransacion,
            TROCO: currentRow.hasChange || false,
			DTHRINCOMVEN: self.dateTime(),
			cartaoTodo: cartaoTodo
		};
	};

	this.dateTime = function () {
		var dateTime = new Date();

		return dateTime.getDateBr() +
			' ' + ApplicationContext.UtilitiesService.padLeft(dateTime.getHours(), 2, '0') +
			':' + ApplicationContext.UtilitiesService.padLeft(dateTime.getMinutes(), 2, '0') +
			':' + ApplicationContext.UtilitiesService.padLeft(dateTime.getSeconds(), 2, '0');
	};

	this.calcTotalSale = function (payment, flag){ // Flag controla se está executando a função na entrada da tela.
        var DATASALE = payment.DATASALE;
        var amountPaid = 0;
        var repique = 0;

        payment.TIPORECE.forEach(function (tiporece){
            amountPaid += tiporece.VRMOVIVEND;
            repique += tiporece.REPIQUE;
        });

        DATASALE.VALORPAGO = amountPaid;
        if (!flag && DATASALE.TOTALVENDA < amountPaid){
            DATASALE.FALTANTE = 0;
            DATASALE.REPIQUE = repique;
            DATASALE.TROCO = parseFloat((amountPaid - DATASALE.TOTALVENDA).toFixed(2));
        } else {
            DATASALE.FALTANTE = parseFloat((DATASALE.TOTALVENDA - amountPaid).toFixed(2));
            DATASALE.TROCO = 0;
            DATASALE.REPIQUE = 0;
        }
    };

	this.handleRemovePayment = function (tiporece) {
		return new Promise(function (resolve) {
			self.updateSetReversal(tiporece.TRANSACTION.data, function (dataTransaction) {
				var arrTiporece = Array();

				if (!tiporece.TRANSACTION.status) {
					arrTiporece.push({
						'CDTIPORECE': tiporece.CDTIPORECE,
						'DTHRINCOMVEN': tiporece.DTHRINCOMVEN
					});

					resolve(self.removePaymentSale(arrTiporece));
				} else {
					self.findIntegrations().then(function (integrations) {
						if (!dataTransaction.REMOVEALLINTEGRATIONS) {
							arrTiporece.push({
								'CDTIPORECE': tiporece.CDTIPORECE,
								'CDNSUHOSTTEF': dataTransaction.CDNSUHOSTTEF,
								'DTHRINCOMVEN': tiporece.DTHRINCOMVEN
							});
						} else {
							arrTiporece = _.map(integrations.data, function (integration) {
								return {
									'CDTIPORECE': integration.CDTIPORECE
								};
							});
						}

						resolve(self.handleCancelIntegration(dataTransaction, arrTiporece, integrations.data));
					});
				}
			});
		});
	};

	this.updateSetReversal = function (dataTransaction, callback) {
		PaymentRepository.findAll().then(function (payments) {
			payments = payments[0].TIPORECE;
			payments = _.filter(payments, function (payment) {
				return payment.TRANSACTION.data.CDNSUHOSTTEF === dataTransaction.CDNSUHOSTTEF;
			});

			payments = _.map(payments, function (payment) {
				return payment.TRANSACTION.data;
			});

			callback(payments[0]);
		});
	};

	this.handleCancelIntegration = function (dataTransaction, arrTiporece, integrations) {
		const removePaymentFlow = IntegrationService.getRemovePaymentFlow();
		/*if(
			!dataTransaction.CDNSUHOSTTEF && dataTransaction.NRSEQMOVMOB && dataTransaction.IDSTATUSMOV=='P'
		){
			return self.handleIntegrationResult({error:false,data:null}, arrTiporece);
		} else*/
		if (!dataTransaction.REMOVEALLINTEGRATIONS && (removePaymentFlow.indexOf(dataTransaction.IDTPTEF) != -1)) {
			integrations = _.filter(integrations, function (integration) {
				return integration.CDNSUHOSTTEF === dataTransaction.CDNSUHOSTTEF;
			}.bind(this));

			return IntegrationService.reversalIntegration(self.removePaymentSale, integrations).then(function(reversalIntegrationResult){
				if(!reversalIntegrationResult.error) {
					return self.removePayment(Array(dataTransaction)).then(function(){
						return self.handleIntegrationResult(reversalIntegrationResult, arrTiporece);
					}.bind(this));
				} else {
					return self.handleIntegrationResult(reversalIntegrationResult, arrTiporece);
				}
			}.bind(this));
		} else {
			return IntegrationService.cancelIntegration(dataTransaction).then(function (cancelIntegrationResult) {
				return self.handleIntegrationResult(cancelIntegrationResult, arrTiporece);
			}.bind(this));
		}
	};

	this.handleIntegrationResult = function (integrationResult, arrTiporece) {
		if (!integrationResult.error) {
			self.handleRefoundTEFVoucher(integrationResult.data);
			return self.removePaymentSale(arrTiporece);
		} else {
			return integrationResult;
		}
	};

    this.removePaymentSale = function (arrTiporece) {
        return PaymentRepository.findOne().then(function (payment) {
            // remove recebimento
            payment.TIPORECE = _.filter(payment.TIPORECE, function (tiporece) {
                var noMatch = true;
                var toMatch = {
                    'CDTIPORECE': tiporece.CDTIPORECE,
                    'DTHRINCOMVEN': tiporece.DTHRINCOMVEN,
                    'CDNSUHOSTTEF': tiporece.TRANSACTION.data.CDNSUHOSTTEF
                };

                _.forEach(arrTiporece, function (toExclude) {
                    if (_.isMatch(toMatch, toExclude)) {
                        noMatch = false;
                    }
                });
            if(!noMatch && tiporece.TRANSACTION.data.NRSEQMOVMOB){
                tiporece.TRANSACTION.data.NRCOMANDA = payment.NRCOMANDA;
                tiporece.TRANSACTION.data.NRVENDAREST = payment.NRVENDAREST;
                tiporece.TRANSACTION.data.CDFILIAL = operatorData.CDFILIAL;
                tiporece.TRANSACTION.data.CDCAIXA = operatorData.CDCAIXA;
                self.undoPayment(tiporece.TRANSACTION.data.NRSEQMOVMOB,tiporece.TRANSACTION.data);
            }
                return noMatch;
            });
            // recalcula valor
            self.calcTotalSale(payment, operatorData.chave, false);
            // salva alterações
            return PaymentRepository.save(payment).then(function () {
                return self.statusPaymentReturn(false, '', null);
            });
        });
    };

	this.handleRefoundTEFVoucher = function (arrRefoundIntegration) {
		if (!_.isEmpty(arrRefoundIntegration)) {
			var arrRefoundTEFVoucher = Array();
			if (!_.isArray(arrRefoundIntegration) && _.isObject(arrRefoundIntegration))
				arrRefoundIntegration = [arrRefoundIntegration];

			arrRefoundIntegration.forEach(function (refoundIntegration) {
				if (_.get(refoundIntegration, 'STLPRIVIA')) {
					arrRefoundTEFVoucher.push(refoundIntegration);
				}
			});

			if (!_.isEmpty(arrRefoundTEFVoucher)) {
				self.printTEFVoucher(arrRefoundTEFVoucher);
			}
		}
	};

	this.printTEFVoucher = function (arrTEFVoucher) {
		OperatorRepository.findOne().then(function (operatorData) {
			var query = Query.build();
			query = query.where('DATA').equals({
				'chave': operatorData.chave,
				'arrTEFVoucher': arrTEFVoucher
			});

			PrintTEFVoucher.download(query).then(function (printTEFVoucherResult) {
				if (!_.isEmpty(printTEFVoucherResult[0].data)) {
					printTEFVoucherResult = { TEFVOUCHER: printTEFVoucherResult[0].data };
					self.handlePrintReceipt(printTEFVoucherResult);
				}
			});
		});
	};

	this.payAccount = function (adiantamento) {
		return self.findIntegrations().then(function (integrations) {
			if (integrations.error) {
				return self.sendPayment(adiantamento);
			} else {
				integrations = integrations.data;
				var mustCompleteIntegrationResult = self.mustCompleteIntegration(integrations);

				if (mustCompleteIntegrationResult.length > 0) {
					return IntegrationService.completeIntegration(integrations).then(function (completeIntegration) {
						if (!completeIntegration.error) {
							return self.sendPayment(adiantamento);
						} else {
							return completeIntegration;
						}
					}.bind(this));
				} else {
					return self.sendPayment(adiantamento);
				}
			}
		}.bind(this));
	};

	this.mustCompleteIntegration = function (integrations) {
		// valida se as integrações realizadas necessitam de confirmação
		return integrations.filter(function (integration) {
			return integration.PAYMENTCONFIRMATION;
		});
	};

	this.sendPayment = function (adiantamento) {
		ScreenService.changeLoadingMessage("Aguarde, salvando venda no banco de dados...");
		return AccountSaleCode.findOne().then(function (saleCodeObj) {
			return PaymentRepository.findOne().then(function (payment) {
				self.preparePayment(payment);

				var paymentMenuWidget = ApplicationContext.templateManager.container.getWidget("paymentPopup");

				self.addDigitalWalletsToTiporece(payment, paymentMenuWidget.currentRow.digitalWallets, paymentMenuWidget.currentRow.selectedDigitalWallet);
				if(existe(paymentMenuWidget,"currentRow.tiporece.IDTIPORECE")){
				self.checkIDTIPORECE(payment, paymentMenuWidget.currentRow.tiporece.IDTIPORECE);
				}
				self.adcAdiantamentoTiporece(payment, adiantamento);

                payment.EMAIL = !_.isEmpty(payment.DSEMAILCONS) ? payment.DSEMAILCONS : '';

				var query = Query.build()
					.where('DATA').equals(payment)
					.where('saleCode').equals(saleCodeObj.saleCode);
				setTimeout(function(){ ScreenService.changeLoadingMessage("Aguarde, salvando venda no banco de dados..."); },10);
				return PaymentPayAccount.download(query).then(function (data) {
					var dataPay = data.payAccount;
					var dataAdvance = data.advancePayAccount;
					ScreenService.changeLoadingMessage("Aguarde...");
					if(!adiantamento) return self.statusPaymentReturn(dataPay.error, dataPay.error ? dataPay.message : '', dataPay);
					else return self.statusPaymentReturn(dataAdvance.error, dataAdvance.error ? dataAdvance.message : '', dataAdvance);
				});
			}.bind(this));
		}.bind(this));
	};

	this.adcAdiantamentoTiporece = function (payment, adiantamento) {
		_.forEach(payment.TIPORECE, function (tiporece) {
			tiporece.ADIANTAMENTO = (!adiantamento) ? false : true;
		});
	};

	this.addDigitalWalletsToTiporece = function (payment, digitalWallets, selectedDigitalWallet) {
		if((!_.isEmpty(digitalWallets) && !_.isEmpty(selectedDigitalWallet))){
			_.forEach(payment.TIPORECE, function (tiporece) {
				tiporece.digitalWallets = digitalWallets;
				tiporece.selectedDigitalWallet = selectedDigitalWallet;
			});
		}
	};

	this.checkIDTIPORECE = function(payment, currentRowIDTIPORECE) {
		_.forEach(payment.TIPORECE, function(tiporece) {
			if(_.isEmpty(tiporece.IDTIPORECE)) tiporece.IDTIPORECE = currentRowIDTIPORECE;
		});
	};

	this.chargePersonalCredit = function (paymentDetails) {
		return self.findIntegrations().then(function (integrations) {
			if (integrations.error) {
				return self.personalCreditConfirmTransaction(paymentDetails);
			} else {
				integrations = integrations.data;
				var mustCompleteIntegrationResult = self.mustCompleteIntegration(integrations);

				if (mustCompleteIntegrationResult.length > 0) {
					return IntegrationService.completeIntegration(integrations).then(function (completeIntegration) {
						if (!completeIntegration.error) {
							return self.personalCreditConfirmTransaction(paymentDetails);
						} else {
							return completeIntegration;
						}
					}.bind(this));
				} else {
					return self.personalCreditConfirmTransaction(paymentDetails);
				}
			}
		}.bind(this));
	};

	this.personalCreditConfirmTransaction = function (paymentDetails) {
		return AccountSaleCode.findOne().then(function (saleCodeObj) {
			var query = Query.build()
				.where('DATA').equals(paymentDetails)
				.where('saleCode').equals(saleCodeObj.saleCode);
			return ChargePersonalCredit.download(query);
		});
	};

	this.preparePayment = function (payment) {
		payment.TIPORECE.forEach(function (tiporece) {
			Util.extend(tiporece, tiporece.TRANSACTION.data);
			_.unset(tiporece, 'TRANSACTION');
		});
	};

	this.handleCancelForSale = function (integrations) {
		// para PAYMENTCONFIRMATION = true é chamado o cancelamento
		return self.mustRedirectReversal(integrations) ?
			IntegrationService.cancelIntegration(integrations[0]) :
			IntegrationService.reversalIntegration(self.removePaymentSale, integrations).then(function(reversalIntegrationResult){
				if(!reversalIntegrationResult.error) {
					self.handleRefoundTEFVoucher(reversalIntegrationResult.data);

					return self.removePayment(integrations).then(function(){
						return reversalIntegrationResult;
					}.bind(this));
				} else {
					if(reversalIntegrationResult.data.length > 1) {
						var reversedPayments = Array();
						var paymentsToRemove = Array();
						reversalIntegrationResult.data.pop();

						reversalIntegrationResult.data.forEach(function(reversedPayment){
							paymentsToRemove.push(reversedPayment.toRemove);

							reversedPayment = {
								'CDNSUHOSTTEF': reversedPayment.toRemove.CDNSUHOSTTEF,
								'NRCONTROLTEF': reversedPayment.REVERSEDNRCONTROLTEF
							};
							reversedPayments.push(reversedPayment);
						});

						return self.removePayment(reversedPayments).then(function(){
							return self.removePaymentSale(paymentsToRemove).then(function(){
								self.handlePrintReceipt({TEFVOUCHER: reversalIntegrationResult.data});
								return reversalIntegrationResult;
							});
						}.bind(this));
					} else {
						return reversalIntegrationResult;
					}
				}
			}.bind(this)
		);
	};

	this.mustRedirectReversal = function (integrations) {
		// pega PAYMENTCONFIRMATION da primeira posição pois será o mesmo para qualquer recebimento
		return integrations[0].PAYMENTCONFIRMATION && this.isOnSale;
	};

	this.findIntegrations = function () {
		var tiporeceTransactions = Array();

		return self.findAllPayment().then(function (arrTiporece) {
			arrTiporece.forEach(function (tiporece) {
				if (tiporece.TRANSACTION.status && tiporece.IDADIANTAMENTO != 'S'){
					tiporece.TRANSACTION.data.CDTIPORECE = tiporece.CDTIPORECE;
					tiporece.TRANSACTION.data.DTHRINCOMVEN = tiporece.DTHRINCOMVEN;
					tiporece.TRANSACTION.data.VRMOVIVEND = tiporece.VRMOVIVEND;
					tiporece.TRANSACTION.data.IDTIPORECE = tiporece.IDTIPORECE;
					tiporeceTransactions.push(tiporece.TRANSACTION.data);
				}
			});
			return self.statusPaymentReturn(tiporeceTransactions.length === 0, '', tiporeceTransactions);
		});
	};

	this.clearPayment = function () {
		PaymentRepository.clearAll();
	};

	this.statusPaymentReturn = function (error, message, data) {
		return {
			error: error,
			message: message,
			data: data
		};
	};

	this.setIsOnSale = function (ISONSALE) {
		this.isOnSale = ISONSALE;
		if (ISONSALE) {
			this.updateSaleCode();
		}
	};

	this.updateSaleCode = function () {
		var saleCodeObj = [{
			'saleCode': new Date().getTime()
		}];
		AccountSaleCode.clearAll().then(function () {
			AccountSaleCode.save(saleCodeObj);
		}.bind(this));
	};

	this.getConsumerLimit = function (CDCLIENTE, CDCONSUMIDOR, type) {
		var query = Query.build()
			.where('CDCLIENTE').equals(CDCLIENTE)
			.where('CDCONSUMIDOR').equals(CDCONSUMIDOR)
			.where('type').equals(type);
		return GetConsumerLimit.download(query);
	};

    this.getTipoProd = function (ITEMVENDA, CDCLIENTE, CDCONSUMIDOR) {
        var query = Query.build()
            .where('ITEMVENDA').equals(ITEMVENDA)
            .where('CDCLIENTE').equals(CDCLIENTE)
            .where('CDCONSUMIDOR').equals(CDCONSUMIDOR);
        return GetTipoProd.download(query);
    };

	this.handleOpenDiscount = function () {
		return self.findAllPayment().then(function (TIPORECE) {
			return self.statusPaymentReturn(TIPORECE.length > 0, '', null);
		});
	};

	this.handleApplyDiscount = function (currentRow) {
		return PaymentRepository.findOne().then(function (payment) {
			//payment.VRDESCCASHUDS=payment.VRDESCCASHUDS||0;
			self.calculateDiscount(currentRow, payment.DATASALE, payment.numeroProdutos, payment);
			var DATASALE = payment.DATASALE;
			// verifica se valor após desconto aplicado é maior que zero e se todos os itens terão valor final >= 0.01
			if (DATASALE.PCTDESCONTO < 100 &&
				(parseFloat(((DATASALE.TOTAL) - (DATASALE.VRDESCONTO) - DATASALE.FIDELITYVALUE).toFixed(2)) - (0.01 * payment.numeroProdutos)) >= 0) {
				// salva dados da venda com desconto aplicado
				payment.desconto.CDSUPERVISOR = currentRow.CDSUPERVISORd;
				payment.desconto.logDesconto = currentRow.TIPODESCONTO;
				payment.desconto.motivoDesconto = currentRow.MOTIVODESCONTO;
				payment.desconto.CDGRPOCORDESC = !_.isEmpty(currentRow.CDOCORR) ? currentRow.CDOCORR[0] : null;
				payment.descontoPromocoesIntPayment = null;
				payment.descontoPromocoesIntWallet  = null;
				payment.descontoPromocoesIntOffers  = payment.descontoPromocoesIntOffers||null;
				return PaymentRepository.save(payment).then(function () {
					return self.statusPaymentReturn(false, '', null);
				}.bind(this));
			} else {
				return self.statusPaymentReturn(true, '', null);
			}
		});
	};

	this.handleApplyTaxDiscount = function (currentRow) {
		return PaymentRepository.findOne().then(function (payment) {
			var taxReference = payment.DATASALE.VRTXSEORIG;
			self.calculateTaxDiscount(currentRow, payment.DATASALE);
			var DATASALE = payment.DATASALE;
			// verifica se valor após desconto aplicado é maior que zero e se todos os itens terão valor final >= 0.01
			if (DATASALE.PCTDESCONTOTAX <= 100 && DATASALE.VRTXSEVENDA <= taxReference){
				// salva dados da venda com desconto aplicado
				payment.desconto.CDSUPERVISOR = currentRow.CDSUPERVISORd;
				payment.desconto.logDesconto = currentRow.TIPODESCONTO;
				payment.desconto.motivoDesconto = currentRow.MOTIVODESCONTO;
				payment.desconto.CDGRPOCORDESC = !_.isEmpty(currentRow.CDOCORR) ? currentRow.CDOCORR[0] : null;
				return PaymentRepository.save(payment).then(function () {
					return self.statusPaymentReturn(false, '', null);
				}.bind(this));
			} else {
				return self.statusPaymentReturn(true, '', null);
			}
		});
	};

	this.calculateDiscount = function (currentRow, DATASALE, numeroProdutos, payment){
		var VRDESCONTO = currentRow.VRDESCONTO;
		var TIPODESCONTO = currentRow.TIPODESCONTO;
		var TOTALVENDA = 0;

		if (TIPODESCONTO === 'P') {
			DATASALE.PCTDESCONTO = VRDESCONTO;
			DATASALE.VRDESCONTO = ApplicationContext.UtilitiesService.truncValue(((DATASALE.TOTAL) - DATASALE.ANTECIPADOS) * (VRDESCONTO / 100));
		} else {
			if(VRDESCONTO>DATASALE.TOTAL){
				VRDESCONTO=DATASALE.TOTAL-(0.01 * numeroProdutos);
			}
			DATASALE.PCTDESCONTO = ApplicationContext.UtilitiesService.truncValue((VRDESCONTO / ((DATASALE.TOTAL)- DATASALE.ANTECIPADOS)) * 100);
			DATASALE.VRDESCONTO = VRDESCONTO;
		}
		if (DATASALE.VRDESCONTO >= DATASALE.TOTAL || DATASALE.PCTDESCONTO >= 100){
			if (DATASALE.TOTAL > 0.01){
				DATASALE.VRDESCONTO = DATASALE.TOTAL - (0.01 * numeroProdutos);
				DATASALE.PCTDESCONTO = (DATASALE.VRDESCONTO * 100) / (DATASALE.TOTAL);
				if (DATASALE.PCTDESCONTO >= 100){
					DATASALE.PCTDESCONTO = 99.99;
				}
				DATASALE.VRDESCONTO = ApplicationContext.UtilitiesService.truncValue(DATASALE.VRDESCONTO);
				DATASALE.PCTDESCONTO = ApplicationContext.UtilitiesService.truncValue(DATASALE.PCTDESCONTO);
			}
		}
		DATASALE.TIPODESCONTO = TIPODESCONTO;

		// desconto aplicado no valor bruto dos itens
		TOTALVENDA = parseFloat(((DATASALE.TOTAL) - DATASALE.ANTECIPADOS - DATASALE.VRDESCONTO - DATASALE.FIDELITYDISCOUNT).toFixed(2));
		// aplica valores calculados nos dados da venda
		DATASALE.TOTALVENDA = Math.round((TOTALVENDA + DATASALE.VRCOUVERT + DATASALE.VRTXSEVENDA) * 100) / 100;
		DATASALE.FALTANTE = Math.round((TOTALVENDA + DATASALE.VRCOUVERT + DATASALE.VRTXSEVENDA) * 100) / 100;
	};

	this.calculateTaxDiscount = function (currentRow, DATASALE) {
		var VRDESCONTO = currentRow.VRDESCONTO;
		var TIPODESCONTO = currentRow.TIPODESCONTOTAX;
		var TOTALVENDA = 0;

		if (TIPODESCONTO === 'P') {
			DATASALE.PCTDESCONTOTAX = VRDESCONTO;
			DATASALE.VRDESCONTOTAX = DATASALE.VRTXSEORIG * (VRDESCONTO / 100);
		} else {
			DATASALE.PCTDESCONTOTAX = ApplicationContext.UtilitiesService.truncValue((VRDESCONTO / DATASALE.VRTXSEORIG) * 100);
			DATASALE.VRDESCONTOTAX = VRDESCONTO;
		}
        DATASALE.TIPODESCONTOTAX = TIPODESCONTO;

		DATASALE.VRTXSEVENDA = parseFloat((DATASALE.VRTXSEORIG - DATASALE.VRDESCONTOTAX).toFixed(2));

		// desconto aplicado no valor bruto dos itens
		TOTALVENDA = parseFloat((DATASALE.TOTAL - DATASALE.ANTECIPADOS - DATASALE.VRDESCONTO - DATASALE.FIDELITYDISCOUNT).toFixed(2));

		// aplica valores calculados nos dados da venda
		DATASALE.TOTALVENDA = Math.round((TOTALVENDA + DATASALE.VRTXSEVENDA + DATASALE.VRCOUVERT) * 100) / 100;
		DATASALE.FALTANTE = Math.round((TOTALVENDA + DATASALE.VRTXSEVENDA + DATASALE.VRCOUVERT) * 100) / 100;
	};

	this.updateCartPrices = function (chave, products, CDCLIENTE, CDCONSUMIDOR, CPF4ALL){
		var query = Query.build()
			.where('chave').equals(chave)
			.where('products').equals(products)
			.where('CDCLIENTE').equals(CDCLIENTE)
			.where('CDCONSUMIDOR').equals(CDCONSUMIDOR)
			.where('CPF4ALL').equals(CPF4ALL);
		return CartPricesRepository.download(query);
	};

    this.handlePrintReceipt = function(dadosImpressao, delayPrint) {
    	if(_.isUndefined(delayPrint)) {
    		delayPrint = true;
    	}

    	return OperatorRepository.findOne().then(function(operatorData){
			if (!_.isEmpty(dadosImpressao)){
				if(!!dadosImpressao.IDUTILIMPFRONT){
					var param = {};
					param.impressora = dadosImpressao.dadosImpressao;
					param.IDUTILIMPFRONT = dadosImpressao.IDUTILIMPFRONT;
					param.comandos = dadosImpressao.comandos;
					return PerifericosService.print(param).then(function () {});
				} else if (_.get(dadosImpressao, 'TEXTOCUPOM1VIA')){
					var noSpaceQrCode = true;
					var TEXTOCUPOM1VIA = dadosImpressao.TEXTOCUPOM1VIA.slice(0, dadosImpressao.TEXTOCUPOM1VIA.length - 38);
					var TEXTORODAPE = (noSpaceQrCode) ? dadosImpressao.TEXTORODAPE.slice(37, dadosImpressao.TEXTORODAPE.length) : dadosImpressao.TEXTORODAPE;
					var text = '';
					text += ' ' + '\n \r';
					text += (noSpaceQrCode) ? TEXTOCUPOM1VIA : dadosImpressao.TEXTOCUPOM1VIA;

					PrinterService.printerCommand(PrinterService.TEXT_COMMAND, text);
					PrinterService.printerCommand(PrinterService.BARCODE_COMMAND, dadosImpressao.TEXTOCODIGOBARRAS);
					PrinterService.printerCommand(PrinterService.QRCODE_COMMAND, dadosImpressao.TEXTOQRCODE);
					PrinterService.printerCommand(PrinterService.TEXT_COMMAND, TEXTORODAPE);

					if (dadosImpressao.tipoImpressaoSenha === "1"){
						PrinterService.printerCommand(PrinterService.TEXT_COMMAND, dadosImpressao.TEXTOSENHARODAPE);
					}
					else {
						PrinterService.printerCommand(PrinterService.BARCODE_COMMAND, self.deixarApenasDigitos(dadosImpressao.TEXTOSENHARODAPE));
					}

					PrinterService.printerCommand(PrinterService.TEXT_COMMAND, dadosImpressao.TEXTOPRODCAN);

					// Impressão do rodapé da API de Painel de senhas do Madero.
					if (!_.isEmpty(dadosImpressao.TEXTOPAINELSENHA)) {
						PrinterService.printerCommand(PrinterService.TEXT_COMMAND, dadosImpressao.TEXTOPAINELSENHA.inicio);
						PrinterService.printerCommand(PrinterService.QRCODE_COMMAND, dadosImpressao.TEXTOPAINELSENHA.qrCode);
						PrinterService.printerCommand(PrinterService.TEXT_COMMAND, dadosImpressao.TEXTOPAINELSENHA.final);
					}

					if (!_.isEmpty(dadosImpressao.TEXTOCUPOM2VIA)){
						PrinterService.printerCommand(PrinterService.TEXT_COMMAND, dadosImpressao.TEXTOCUPOM2VIA);
						PrinterService.printerCommand(PrinterService.BARCODE_COMMAND, dadosImpressao.TEXTOCODIGOBARRAS);
						PrinterService.printerCommand(PrinterService.QRCODE_COMMAND, dadosImpressao.TEXTOQRCODE);
						PrinterService.printerCommand(PrinterService.TEXT_COMMAND, dadosImpressao.TEXTORODAPE);
					}
					var space = (operatorData.IDTPEMISSAOFOS  === 'SAT' || !_.isEmpty(dadosImpressao.TEXTOPAINELSENHA)) ? 2 : 0;
					space = operatorData.IDHABCAIXAVENDA === 'FOS' ? space + 1 : space;
					self.printerSpaceCommand(space);

					if (!_.isEmpty(dadosImpressao.TEXTOMSGRODAPE)) {
						PrinterService.printerCommand(PrinterService.TEXT_COMMAND, dadosImpressao.TEXTOMSGRODAPE);
					}

					if (_.get(dadosImpressao, 'TEFVOUCHER')) {
						dadosImpressao.TEFVOUCHER.forEach(function (tefVoucher) {
							if (!_.isEmpty(tefVoucher.STLPRIVIA)) {
								PrinterService.printerCommand(PrinterService.TEXT_COMMAND, tefVoucher.STLPRIVIA);
							 	self.printerSpaceCommand(2);
	
							 	if(delayPrint)
							 		PrinterService.printerCommand(PrinterService.DELAY_COMMAND, '3000');
		                    }
						});
						dadosImpressao.TEFVOUCHER.forEach(function(tefVoucher){
							if (!_.isEmpty(tefVoucher.STLSEGVIA)){
								PrinterService.printerCommand(PrinterService.TEXT_COMMAND, tefVoucher.STLSEGVIA);
								self.printerSpaceCommand(2);
							}
						});
					}
					return new Promise(function (resolve) {
						// inicializa a impressão dos cupons
						// mesmo retornando seu resultado, não se trata a resposta da impressão (hoje)
						if(!_.isEmpty(PrinterService.TEXT_COMMAND) && dadosImpressao.TEXTOCUPOM1VIA != null) {
							PrinterService.printerInit().then(function (result) {
								if (result.error){
									ScreenService.alertNotification(result.message);
								}
								resolve();
							});
						} else {
							resolve();
						}
					});
				} else {
					dadosImpressao.TEFVOUCHER.forEach(function(tefVoucher){
						if (!_.isEmpty(tefVoucher.STLPRIVIA)){
							PrinterService.printerCommand(PrinterService.TEXT_COMMAND, tefVoucher.STLPRIVIA);
							self.printerSpaceCommand(2);
						}
					});

					return new Promise(function (resolve) {
						// inicializa a impressão dos cupons
						// mesmo retornando seu resultado, não se trata a resposta da impressão (hoje)
						PrinterService.printerInit().then(function (result) {
							if (result.error){
								ScreenService.alertNotification(result.message);
							}
							resolve();
						});
					});
				}
			}
		});
	};

	this.printerSpaceCommand = function (max) {
		for (var i = 0; i < max; i++) {
			PrinterService.printerSpaceCommand();
		}
	};

	this.qrCodeSale = function (chave, qrCode, cpf) {
		return AccountSaleCode.findOne().then(function (saleCodeObj) {
			var query = Query.build()
				.where('chave').equals(chave)
				.where('QRCODE').equals(qrCode)
				.where('CPF').equals(cpf)
				.where('saleCode').equals(saleCodeObj.saleCode);
			return QRCodeSaleRepository.download(query);
		});
	};

	this.handleAccountAddition = function (CDSUPERVISOR) {
		return PaymentRepository.findOne().then(function (payment) {
			var DATASALE = payment.DATASALE;
			// ajusta nova taxa de serviço
			var NEWVRTXSEVENDA = (DATASALE.FALTANTE < DATASALE.VRTXSEVENDA) ?
				parseFloat((DATASALE.VRTXSEVENDA - DATASALE.FALTANTE).toFixed(2)) : 0;

			DATASALE.TOTALVENDA = parseFloat((DATASALE.TOTALVENDA -
				parseFloat((DATASALE.VRTXSEVENDA - NEWVRTXSEVENDA).toFixed(2))).toFixed(2));
			DATASALE.VRTXSEVENDA = NEWVRTXSEVENDA;
			self.calcTotalSale(payment, false);

			// adiciona supervisor para log
			payment.servico.CDSUPERVISOR = CDSUPERVISOR;
			payment.servico.logServico = !NEWVRTXSEVENDA ? 'RET_TAX' : 'ALT_TAX';

			return PaymentRepository.save(payment);
		}.bind(this));
	};

    this.removePayment = function(payments){
    	if(!_.isArray(payments)) {
    		payments = Array(payments);
    	}

    	var query = Query.build()
			.where('DATA').equals(payments);

		return RemovePayment.download(query);
    };

    this.getAdvancedPayment = function(chave, NRVENDAREST, NRCOMANDA){
    	var query = Query.build()
			.where('chave').equals(chave)
			.where('NRVENDAREST').equals(NRVENDAREST)
			.where('NRCOMANDA').equals(NRCOMANDA);

		return GetAdvancedPayment.download(query);
    };

    this.callCrmBunus = function (telefoneConsumidor, TOTALVENDA) {
		var query = Query.build()
			.where('telefoneConsumidor').equals(telefoneConsumidor)
			.where('TOTALVENDA').equals(TOTALVENDA);

		return CallCrmBunus.download(query);
	};

	this.getValidaTipoConsReceb = function(TIPORECE, CDCLIENTE, CDCONSUMIDOR){
		var query = Query.build()
						.where('TIPORECE').equals(TIPORECE)
						.where('CDCLIENTE').equals(CDCLIENTE)
						.where('CDCONSUMIDOR').equals(CDCONSUMIDOR);
		return ValidaTipoConsReceb.download(query);
	};

	this.buscaSaldoCartaoTodo = function(chave,card_number){
		var query = Query.build()
						.where('chave').equals(chave)
						.where('card_number').equals(card_number);
		return BuscaSaldoCartaoTodo.download(query);
	};

	this.getValidaVendaMesa = function(){
		return AccountSaleCode.findOne().then(function (saleCodeObj) {
			return PaymentRepository.findOne().then(function (payment) {
				self.preparePayment(payment);
				var query = Query.build()
					.where('DATA').equals(payment)
					.where('saleCode').equals(saleCodeObj.saleCode);
				return ValidaVendaMesa.download(query).then(function (data) {
					return data;
				});
			});
		});
	};

	this.deixarApenasDigitos = function (string) {
		return string.replace(/\D/g, '');
	};

	this.getDataUpdatePayment = function(){
        return PaymentRepository.findOne().then(function(paymentData){
            return {paymentData:paymentData};
        });
    };

    this.updatePayment = function(paymentData, integrationResult){
        var query = Query.build()
            .where('paymentData').equals(paymentData)
            .where('integrationResult').equals(integrationResult);

        return UpdatePayment.download(query);
    };

    this.undoPayment = function(NRSEQMOVMOB,paymentData){
        var query = Query.build()
            .where('paymentData').equals(paymentData)
            .where('NRSEQMOVMOB').equals(NRSEQMOVMOB);

        return UndoPayment.download(query);
    };

	this.getDataUndoPayment = function(){
        return SavePayment.findOne().then(function(NRSEQMOVMOB){
            return PaymentRepository.findOne().then(function(paymentData){
				return {paymentData:paymentData,NRSEQMOVMOB:NRSEQMOVMOB.NRSEQMOVMOB};
            });
        });
    };

}

Configuration(function (ContextRegister) {
	ContextRegister.register('PaymentService', PaymentService);
});
