<?php

namespace Odhen\API\Service;

use Odhen\API\Util\Exception;

class Venda {

    protected $entityManager;
    protected $notaFiscalService;
    protected $estoque;
    protected $getParametro;
    protected $util;
    protected $sat;
    protected $databaseUtil;
    protected $precoService;
    protected $comandaService;
    protected $vendaValidacao;
    protected $vendaNormalizacao;
    protected $consumidorService;
    protected $extratocons;
    protected $impressaoDelivery;
    private $logVendaPath;
	protected $PromocoesIntServiceAPI;
	protected $fidelidadeUDSAPI;
    protected $cartaoTodo;

    public function __construct(
        \Doctrine\ORM\EntityManager $entityManager,
        \Odhen\API\Service\NotaFiscal $notaFiscal,
        \Odhen\API\Service\Estoque $estoque,
        \Odhen\API\Util\GetParametro $getParametro,
        \Odhen\API\Util\Util $util,
        \Odhen\API\Service\SAT $sat,
        \Odhen\API\Util\Database $databaseUtil,
        \Odhen\API\Service\Preco $precoService,
        \Odhen\API\Service\Comanda $comandaService,
        \Odhen\API\Service\VendaValidacao $vendaValidacao,
        \Odhen\API\Service\VendaNormalizacao $vendaNormalizacao,
        \Odhen\API\Service\Consumidor $consumidorService,
        \Odhen\API\Service\Extratocons $extratocons,
        \Odhen\API\Service\ImpressaoDelivery $impressaoDelivery,
        \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager,
        \Odhen\API\Service\PromocoesInt $PromocoesIntServiceAPI,
		\Odhen\API\Service\FidelidadeUDS $fidelidadeUDSAPI,
        \Odhen\API\Service\CartaoTodo $cartaoTodo
        ){

        $this->entityManager = $entityManager;
        $this->notaFiscalService = $notaFiscal;
        $this->estoque = $estoque;
        $this->getParametro = $getParametro;
        $this->util = $util;
        $this->sat = $sat;
        $this->databaseUtil = $databaseUtil;
        $this->precoService = $precoService;
        $this->comandaService = $comandaService;
        $this->vendaValidacao = $vendaValidacao;
        $this->vendaNormalizacao = $vendaNormalizacao;
        $this->consumidorService = $consumidorService;
        $this->extratocons   = $extratocons;
        $this->impressaoDelivery = $impressaoDelivery;
        $this->instanceManager = $instanceManager;
        $this->habilitaLog = $this->instanceManager->getParameter('HABILITA_LOG_VENDA');
        $this->logVendaPath = null;
		$this->PromocoesIntServiceAPI = $PromocoesIntServiceAPI;
		$this->fidelidadeUDSAPI = $fidelidadeUDSAPI;
		$this->cartaoTodo = $cartaoTodo;
    }

    const APROVADA = 'O';
    const ENTRADA = 'E';
    const SINCRONIZA_ESTOQUE = 'S';
    const ITEM_APROVADO = 'A';
    const ITEM_CANCELADO = '6';
    const ITEM_ABASTECIMENTO = 'A';
    const TIPO_NFCE = 'N';
    const TIPO_SAT = 'S';
    const OBS_ADICIONA_PRODUTO = 'A';
    const PRODUTO_TIPO_TAXA_SERVICO = 'S';
    const PRODUTO_TIPO_TAXA_ENTREGA = 'E';
    const PRODUTO_TIPO_TAXA_EXCEDENTE = 'X';
    const PRODUTO_TIPO_NORMAL = 'N';
    

    public function venda(
        $NRORG, $CDFILIAL, $CDLOJA, $CDCAIXA,
        $CDVENDEDOR, $CDOPERADOR, $DTABERCAIX, $DTVENDA,
        $TOTALVENDA, $ITEMVENDA, $TIPORECE, $NMCONSVEND,
        $NRINSCRCONS, $DSENDECONSVENDA, $NRSEQVENDA, $VRTROCOVEND, $EMAIL,
        $VRDESCVENDA, $CDCLIENTE, $CDCONSUMIDOR, $imprimeCupom,
        $simulateSaleValidation, $IDORIGEMVENDA, $CDSENHAPED, $NRVENDAREST = null,
        $NRCOMANDA = null, $fidelityDetails = null, $FIDELITYDISCOUNT = 0, $cashbackUsed4ALL = 0, $FIDELITYVALUE = 0, $CODPROMO = null, $motivoDesconto = null, $CDGRPOCORDESC = null, $DSOBSFINVEN = null, $ITVENDADES = null, $REPIQUE = 0, $saleCode = null, $INFORMAEMAIL, $DSEMAILCONS, $IDENVCUPOMEMAIL,
        $NRCPFFIDUDS, $VRDESCCASHUDS, $arrPromocoesIntUtilizadas = array(), $supervisorDesc = null){

        $this->getLogHeader($CDFILIAL, $CDLOJA, $CDCAIXA, $CDOPERADOR, $DTVENDA, $TOTALVENDA);

        try {
            $this->util->geraLog(' -beginTransaction', $CDFILIAL, $CDCAIXA, $DTVENDA);
            $this->entityManager->getConnection()->beginTransaction();
            $respostaValidacao = $this->vendaValidacao->validaVenda($CDFILIAL, $CDCAIXA);
            $this->util->geraLog(' -Validação Venda (1): ' . ($respostaValidacao['error'] ? 'com' : 'sem') . ' erro.', $CDFILIAL, $CDCAIXA, $DTVENDA);
            if (!$respostaValidacao['error']){
                $respostaNormalizacao = $this->vendaNormalizacao->normalizaVenda($CDFILIAL, $CDCAIXA, $ITEMVENDA, $TIPORECE, $VRTROCOVEND);
                $IDTPEMISVEND = $respostaNormalizacao['IDTPEMISVEND'];
                $ITEMVENDA = $respostaNormalizacao['ITEMVENDA'];
                $TIPORECE = $respostaNormalizacao['TIPORECE'];
                $VRTROCOVEND = $respostaNormalizacao['VRTROCOVEND'];

                self::adicionaObsCobrada($ITEMVENDA);
                $TOTALPRODUTOS = self::calculaTotalProdutos($ITEMVENDA, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR);
                $validacaoVenda = self::montaPrecosProdutos($TOTALPRODUTOS, $ITEMVENDA, $VRDESCVENDA, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR);
                if (!$validacaoVenda['error']){
                    $ITEMVENDA = $validacaoVenda['produtosComPreco'];
                    if (!empty($REPIQUE)) {
                        $dadosTaxa = $this->buscaDadosTaxaServico($CDFILIAL, $CDLOJA);
                        $produtoTaxaServico = self::adicionaProdutoTaxaServico($dadosTaxa, null, null, $REPIQUE, 0, $TOTALPRODUTOS);
                        if (!empty($produtoTaxaServico['TAXAORIG'])) {
                            array_push($ITEMVENDA, $produtoTaxaServico['TAXAORIG']);
                            if (!empty($produtoTaxaServico['TAXAEXCE'])){
                                array_push($ITEMVENDA, $produtoTaxaServico['TAXAEXCE']);
                            }
                        }
                    }

                    $VRDESCVENDA = $validacaoVenda['totalDescontos'];

                    $respostaValidacaoValor = $this->vendaValidacao->validaValor($ITEMVENDA, $TOTALVENDA, $VRDESCVENDA, $CDFILIAL, $CDCAIXA);
                    $this->util->geraLog(' -Validação Venda (2): ' . ($respostaValidacaoValor['error'] ? 'com' : 'sem') . ' erro.', $CDFILIAL, $CDCAIXA, $DTVENDA);
                    if (!$respostaValidacaoValor['error']){
                        $TOTALVENDALIQ = $respostaValidacaoValor['TOTALVENDA'];
                        $respostaValidacaoDebitoConsumidor = $this->vendaValidacao->validaVendaDebitoConsumidor($CDFILIAL, $CDLOJA, $CDOPERADOR, $CDCLIENTE, $CDCONSUMIDOR, $TIPORECE, $TOTALVENDA, $respostaValidacaoValor['subsidy']);
                        $this->util->geraLog(' -Validação Débito Consumidor: ' . ($respostaValidacaoDebitoConsumidor['error'] ? 'com' : 'sem') . ' erro.', $CDFILIAL, $CDCAIXA, $DTVENDA);
                        if (!$respostaValidacaoDebitoConsumidor['error']){
                            $respostaValidacaoCreditoPessoal = $this->vendaValidacao->validaVendacreditoPessoal($TIPORECE, $ITEMVENDA, $CDCLIENTE, $CDCONSUMIDOR, $CDFILIAL, $CDCAIXA);
                            $this->util->geraLog(' -Validação Crédito Pessoal: ' . ($respostaValidacaoCreditoPessoal['error'] ? 'com' : 'sem') . ' erro.', $CDFILIAL, $CDCAIXA, $DTVENDA);
                            if (!$respostaValidacaoCreditoPessoal['error']){
                                $FAMSALDOPROD = $respostaValidacaoCreditoPessoal['FAMSALDOPROD'];
                                $CDSENHAPED = !empty($CDSENHAPED) ? $CDSENHAPED : $this->util->generateCDSENHAPED($CDFILIAL, $NRORG);
                                $NRMESA = null;

                                $result = $this->trataVenda(
                                    $NRORG,
                                    $CDFILIAL,
                                    $CDLOJA,
                                    $CDCAIXA,
                                    $CDVENDEDOR,
                                    $CDOPERADOR,
                                    $DTABERCAIX,
                                    $DTVENDA,
                                    $IDTPEMISVEND,
                                    $TOTALVENDALIQ,
                                    $TOTALVENDA,
                                    $ITEMVENDA,
                                    $TIPORECE,
                                    $NMCONSVEND,
                                    $NRINSCRCONS,
                                    $DSENDECONSVENDA,
                                    $CDSENHAPED,
                                    $NRSEQVENDA,
                                    $VRTROCOVEND,
                                    $EMAIL,
                                    $VRDESCVENDA,
                                    $CDCLIENTE,
                                    $CDCONSUMIDOR,
                                    $NRVENDAREST,
                                    $NRCOMANDA,
                                    $NRMESA,
                                    $IDORIGEMVENDA,
                                    $imprimeCupom,
                                    $simulateSaleValidation,
                                    null, // VRTXSEVENDA
                                    1, // NRPESMESAVENDA
                                    null, // DSCOMANDA
                                    $FAMSALDOPROD,
                                    null, // DTHRMESAFECHVEN
                                    $fidelityDetails,
                                    $FIDELITYDISCOUNT,
                                    $cashbackUsed4ALL,
                                    $FIDELITYVALUE,
                                    $CODPROMO,
                                    $motivoDesconto,
                                    $CDGRPOCORDESC,
                                    $respostaValidacaoValor['subsidy'],
                                    $DSOBSFINVEN,
                                    $ITVENDADES,
                                    false, // DELIVERY
                                    0, // VRACRCOMANDA
                                    $REPIQUE,
                                    $saleCode,
                                    $IDENVCUPOMEMAIL,
                                    $INFORMAEMAIL,
                                    $DSEMAILCONS,
									$NRCPFFIDUDS,
									$VRDESCCASHUDS,
									$arrPromocoesIntUtilizadas,
                                    $supervisorDesc
                                );

                                $this->util->geraLog(' -Validação Trata Venda: ' . ($result['error'] ? 'com' : 'sem') . ' erro.', $CDFILIAL, $CDCAIXA, $DTVENDA);

                                if(!$result['error']) {
                                    $this->util->geraLog('  VENDA REALIZADA COM SUCESSO. NRSEQVENDA: ' . $result["NRSEQVENDA"], $CDFILIAL, $CDCAIXA, $DTVENDA);
                                } else {
                                    $this->util->geraLog('Erro: ' . $result['message'], $CDFILIAL, $CDCAIXA, $DTVENDA);
                                }
                            } else {
                                $this->util->geraLog('Erro: ' . $respostaValidacaoCreditoPessoal['message'], $CDFILIAL, $CDCAIXA, $DTVENDA);
                                $result = $respostaValidacaoCreditoPessoal;
                            }
                        } else {
                            $this->util->geraLog('Erro: ' . $respostaValidacaoDebitoConsumidor['message'], $CDFILIAL, $CDCAIXA, $DTVENDA);

                            $result = $respostaValidacaoDebitoConsumidor;
                        }
                    } else {
                        $this->util->geraLog('Erro: ' . $respostaValidacaoValor['message'], $CDFILIAL, $CDCAIXA, $DTVENDA);
                        $result = $respostaValidacaoValor;
                    }
                } else {
                    $this->util->geraLog('Erro: ' . $validacaoVenda['message'], $CDFILIAL, $CDCAIXA, $DTVENDA);
                    $result = $validacaoVenda;
                }
            } else {
                $this->util->geraLog('Erro: ' . $respostaValidacao['message'], $CDFILIAL, $CDCAIXA, $DTVENDA);
                $result = $respostaValidacao;
            }
            $this->util->geraLog(' -commit', $CDFILIAL, $CDCAIXA, $DTVENDA);
            $this->entityManager->getConnection()->commit();
            // Valida se a venda foi inserida no banco.
            if(isset($result['NRSEQVENDA'])){
                $this->validaInsercaoVenda($CDFILIAL, $CDCAIXA, $DTVENDA, $result['NRSEQVENDA']);
            }

        } catch (\Exception $e) {
            $this->util->geraLog($e->getMessage(), $CDFILIAL, $CDCAIXA, $DTVENDA);
            Exception::logException($e, Exception::LOG_TYPES['EXCEPTION_LOG']);
            if ($this->entityManager->getConnection()->isTransactionActive()) {
                $this->entityManager->getConnection()->rollBack();
            }
            $result = array(
                'error' => true,
                'message' => $e->getMessage()
            );
        }
        return $result;
    }

    public function vendaMesa(
        $NRORG, $CDFILIAL, $CDLOJA, $CDCAIXA,
        $CDVENDEDOR, $CDOPERADOR, $DTABERCAIX, $DTVENDA,
        $TOTALPRODUTOS, $TIPORECE, $NMCONSVEND, $NRINSCRCONS, $DSENDECONSVENDA, $CDSENHAPED,
        $VRTROCOVEND, $EMAIL, $VRDESCVENDA, $CDCLIENTE,
        $CDCONSUMIDOR, $imprimeCupom, $simulateSaleValidation, $dadosMesa,
        $arrayPosicoes, $IDORIGEMVENDA, $VRTXSEVENDA,
        $FIDELITYDISCOUNT = 0, $cashbackUsed4ALL = 0, $FIDELITYVALUE = 0,
        $motivoDesconto = null, $CDGRPOCORDESC = null, $DSOBSFINVEN = null,
        $DELIVERY = false, $VRACRCOMANDA = 0, $REPIQUE = 0, $saleCode = null, $INFORMAEMAIL, $DSEMAILCONS, $IDENVCUPOMEMAIL,
        $arrPromocoesIntUtilizadas = array(), $supervisorDesc = null){
        $this->getLogHeader($CDFILIAL, $CDLOJA, $CDCAIXA, $CDOPERADOR, $DTVENDA, $TOTALPRODUTOS);

        try {
            $this->util->geraLog(' -beginTransaction', $CDFILIAL, $CDCAIXA, $DTVENDA);
            $this->entityManager->getConnection()->beginTransaction();

            $respostaValidacao = $this->vendaValidacao->validaVenda($CDFILIAL, $CDCAIXA);
            $this->util->geraLog(' -Validação Venda (1): ' . ($respostaValidacao['error'] ? 'com' : 'sem') . ' erro.', $CDFILIAL, $CDCAIXA, $DTVENDA);

            if ($respostaValidacao['error'] == false) {
                $respostaNormalizacao = $this->vendaNormalizacao->normalizaVenda($CDFILIAL, $CDCAIXA, null, $TIPORECE, $VRTROCOVEND);
                $IDTPEMISVEND = $respostaNormalizacao['IDTPEMISVEND'];
                $TIPORECE = $respostaNormalizacao['TIPORECE'];
                $VRTROCOVEND = $respostaNormalizacao['VRTROCOVEND'];
				$VRDESCCASHUDS = empty($dadosMesa[0]['VRDESCCASHUDS'])?0:floatval($dadosMesa[0]['VRDESCCASHUDS']);
                $validacaoVenda = self::validaVendaComanda(
                    $CDFILIAL, $dadosMesa, $arrayPosicoes, $CDLOJA,
                    $VRDESCVENDA, $VRTXSEVENDA, $CDCLIENTE, $DELIVERY,
                    $VRACRCOMANDA, $TOTALPRODUTOS, $REPIQUE, $VRDESCCASHUDS
                );
                $this->util->geraLog(' -Validação Venda (2): ' . ($validacaoVenda['error'] ? 'com' : 'sem') . ' erro.', $CDFILIAL, $CDCAIXA, $DTVENDA);

                if ($validacaoVenda['error'] == false) {
                    $ITEMVENDA = $validacaoVenda['ITEMVENDA'];
                    $ITVENDADES = $validacaoVenda['ITVENDADES'];
                    $TOTALVENDALIQ = $validacaoVenda['TOTALVENDALIQ'];
                    $TOTALVENDABRT = $validacaoVenda['TOTALVENDABRT'];

                    if(!$DELIVERY) {
                        $NRCPFFIDUDS = $dadosMesa[0]['NRCPFFIDUDS'];
                    } else {
                        $NRCPFFIDUDS = null;
                    }

                    $VRDESCVENDA = floatval(bcadd(strval($VRDESCVENDA), strval($validacaoVenda['PRODUCTDISCOUNTS']), 2));
                    $NRPESMESAVENDA = count($arrayPosicoes)==0 ? ($dadosMesa[0]['NRPESMESAVEN']) : count($arrayPosicoes);
                    $NRVENDAREST = $dadosMesa[0]['NRVENDAREST'];
                    $NRCOMANDA = $dadosMesa[0]['NRCOMANDA'];
                    $NRMESA = $dadosMesa[0]['NRMESA'];
                    $DSCOMANDA = $dadosMesa[0]['DSCOMANDA'];
                    $DTHRMESAFECHVEN = $dadosMesa[0]['DTHRMESAFECH'];
                    $NRSEQVENDA = null;

                    // Valida débito consumidor.
                    $respostaValidacaoDebitoConsumidor = $this->vendaValidacao->validaVendaDebitoConsumidor($CDFILIAL, $CDLOJA, $CDOPERADOR, $CDCLIENTE, $CDCONSUMIDOR, $TIPORECE, $TOTALVENDALIQ, $validacaoVenda['SUBSIDY']);
                    $this->util->geraLog(' -Validação Débito Consumidor: ' . ($respostaValidacaoDebitoConsumidor['error'] ? 'com' : 'sem') . ' erro.', $CDFILIAL, $CDCAIXA, $DTVENDA);
                    if ($respostaValidacaoDebitoConsumidor['error']){
                        $this->util->geraLog('Erro: ' . $respostaValidacaoDebitoConsumidor['message'], $CDFILIAL, $CDCAIXA, $DTVENDA);
                        throw new \Exception($respostaValidacaoDebitoConsumidor['message']);
                    }
                    // Valida crédito pessoal.
                    $respostaValidacaoCreditoPessoal = $this->vendaValidacao->validaVendacreditoPessoal($TIPORECE, $ITEMVENDA, $CDCLIENTE, $CDCONSUMIDOR, $CDFILIAL, $CDCAIXA);
                    $this->util->geraLog(' -Validação Crédito Pessoal: ' . ($respostaValidacaoCreditoPessoal['error'] ? 'com' : 'sem') . ' erro.', $CDFILIAL, $CDCAIXA, $DTVENDA);
                    if ($respostaValidacaoCreditoPessoal['error']){
                        $this->util->geraLog('Erro: ' . $respostaValidacaoCreditoPessoal['message'], $CDFILIAL, $CDCAIXA, $DTVENDA);
                        throw new \Exception($respostaValidacaoCreditoPessoal['message']);
                    }
                    // Busca dados do Fidelidade 4ALL e UDS.
                    $tableFidelity = $this->handleFidelity($CDFILIAL, $dadosMesa[0]['NRVENDAREST'], $dadosMesa[0]['NRCOMANDA'], $TOTALVENDABRT, $ITEMVENDA);
                    if (!empty($tableFidelity)){
                        $fidelityDetails = $tableFidelity['fidelityDetails'];
                        $CODPROMO = $tableFidelity['CODPROMO'];
                    } else {
                        $fidelityDetails = null;
                        $CODPROMO = null;
                    }

                    $result = $this->trataVenda(
                        $NRORG,
                        $CDFILIAL,
                        $CDLOJA,
                        $CDCAIXA,
                        $CDVENDEDOR,
                        $CDOPERADOR,
                        $DTABERCAIX,
                        $DTVENDA,
                        $IDTPEMISVEND,
                        $TOTALVENDALIQ,
                        $TOTALVENDABRT,
                        $ITEMVENDA,
                        $TIPORECE,
                        $NMCONSVEND,
                        $NRINSCRCONS,
                        $DSENDECONSVENDA,
                        $CDSENHAPED,
                        $NRSEQVENDA,
                        $VRTROCOVEND,
                        $EMAIL,
                        $VRDESCVENDA,
                        $CDCLIENTE,
                        $CDCONSUMIDOR,
                        $NRVENDAREST,
                        $NRCOMANDA,
                        $NRMESA,
                        $IDORIGEMVENDA,
                        $imprimeCupom,
                        $simulateSaleValidation,
                        $VRTXSEVENDA,
                        $NRPESMESAVENDA,
                        $DSCOMANDA,
                        $respostaValidacaoCreditoPessoal['FAMSALDOPROD'],
                        $DTHRMESAFECHVEN,
                        $fidelityDetails,
                        $FIDELITYDISCOUNT,
                        $cashbackUsed4ALL,
                        $FIDELITYVALUE,
                        $CODPROMO,
                        $motivoDesconto,
                        $CDGRPOCORDESC,
                        $validacaoVenda['SUBSIDY'],
                        $DSOBSFINVEN,
                        $ITVENDADES,
                        $DELIVERY,
                        $VRACRCOMANDA,
                        $REPIQUE,
                        $saleCode,
                        $IDENVCUPOMEMAIL,
                        $INFORMAEMAIL,
                        $DSEMAILCONS,
                       	$NRCPFFIDUDS,
						$VRDESCCASHUDS,
                       	$arrPromocoesIntUtilizadas,
                        $supervisorDesc
                    );

                    $this->util->geraLog(' -Validação Trata Venda: ' . ($result['error'] ? 'com' : 'sem') . ' erro.', $CDFILIAL, $CDCAIXA, $DTVENDA);

                    if (!$result['error']){
                        $this->util->geraLog('  VENDA REALIZADA COM SUCESSO. NRSEQVENDA: ' . $result["NRSEQVENDA"], $CDFILIAL, $CDCAIXA, $DTVENDA);
                        $params = array(
                            'NRPESMESAVEN' => -(count($arrayPosicoes)),
                            'CDFILIAL'     => $CDFILIAL,
                            'CDLOJA'       => $CDLOJA,
                            'NRVENDAREST'  => $NRVENDAREST
                        );
                        $this->entityManager->getConnection()->executeQuery('UPDATE_NRPESMESAVEN', $params);
                        
                        if($DELIVERY){
                            if($TOTALVENDABRT > 0){
                                $orders = $this->comandaService->liberaComandaDelivery($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $CDLOJA, $result['NRSEQVENDA'], $CDCAIXA);
                                $params = array(
                                    'CDFILIAL' => $CDFILIAL,
                                    'CDLOJA'   => $CDLOJA,
                                    'CDCAIXA'  => $CDCAIXA
                                );
                                $imprimeEntregaAut = $this->entityManager->getConnection()->fetchAssoc("IMPRIMEDLV_AUTOMATICO", $params);

                                $resultDlv['error'] = false;
                                if($imprimeEntregaAut['IDIMPAUTENTREG'] == 'S'){
                                    $resultDlv = $this->impressaoDelivery->imprimeDelivery($orders);
                                }
                                $result['errorDlv'] = $resultDlv['error'];
                                if($resultDlv['error']){
                                    $result['messageDlv'] = $resultDlv['message'];
                                }
                            }
                        } else{
                            $this->comandaService->liberaComanda($CDFILIAL, $dadosMesa, $arrayPosicoes, $CDOPERADOR, $CDLOJA, $IDORIGEMVENDA);
                        }
                    } else {
                        $this->util->geraLog('Erro: ' . $result['message'], $CDFILIAL, $CDCAIXA, $DTVENDA);
                    }
                } else {
                    $this->util->geraLog('Erro: ' . $validacaoVenda['message'], $CDFILIAL, $CDCAIXA, $DTVENDA);
                    $result = $validacaoVenda;
                }
            } else {
                $this->util->geraLog('Erro: ' . $respostaValidacao['message'], $CDFILIAL, $CDCAIXA, $DTVENDA);
                $result = $respostaValidacao;
            }

            $this->util->geraLog(' -commit', $CDFILIAL, $CDCAIXA, $DTVENDA);
            $this->entityManager->getConnection()->commit();

            // Valida se a venda foi inserida no banco.
            if(isset($result['NRSEQVENDA'])){
                $this->validaInsercaoVenda($CDFILIAL, $CDCAIXA, $DTVENDA, $result['NRSEQVENDA']);
            }

        } catch (\Exception $e) {
            $this->util->geraLog($e->getMessage(), $CDFILIAL, $CDCAIXA, $DTVENDA);
            Exception::logException($e, Exception::LOG_TYPES['EXCEPTION_LOG']);
            if ($this->entityManager->getConnection()->isTransactionActive()) {
                $this->entityManager->getConnection()->rollBack();
            }
            $result = array(
                'error' => true,
                'message' => $e->getMessage()
            );
        }
        return $result;
    }

    protected function trataVenda($NRORG,
        $CDFILIAL, $CDLOJA, $CDCAIXA,
        $CDVENDEDOR, $CDOPERADOR, $DTABERCAIX, $DTVENDA,
        $IDTPEMISVEND, $TOTALVENDALIQ, $TOTALVENDABRT, $ITEMVENDA, $TIPORECE,
        $NMCONSVEND, $NRINSCRCONS, $DSENDECONSVENDA, $CDSENHAPED, $NRSEQVENDA,
        $VRTROCOVEND, $EMAIL, $VRDESCVENDA, $CDCLIENTE,
        $CDCONSUMIDOR, $NRVENDAREST, $NRCOMANDA, $NRMESA,
        $IDORIGEMVENDA, $imprimeCupom, $simulateSaleValidation, $VRTXSEVENDA,
        $NRPESMESAVENDA, $DSCOMANDA, $FAMSALDOPROD, $DTHRMESAFECHVEN,
        $fidelityDetails, $FIDELITYDISCOUNT, $cashbackUsed4ALL, $FIDELITYVALUE, $CODPROMO,
        $motivoDesconto, $CDGRPOCORDESC, $SUBSIDY, $DSOBSFINVEN, $ITVENDADES = null,
        $DELIVERY = false, $VRACRCOMANDA = 0, $REPIQUE = 0, $saleCode, $IDENVCUPOMEMAIL, $INFORMAEMAIL, $DSEMAILCONS,
        $NRCPFFIDUDS, $VRDESCCASHUDS, $arrPromocoesIntUtilizadas, $CDSUPERVISOR){

        $VRTXSEVENDA += $VRACRCOMANDA;

        $impostos = $this->calculaImpostoParaImpressao($NRORG, $CDFILIAL, $CDCAIXA, $ITEMVENDA, $TOTALVENDABRT, $VRDESCVENDA, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR, $VRTXSEVENDA);
        $VRTOTTRIBIBPT = $impostos['VRTOTTRIBIBPT'];
        $impostoEstadual = $impostos['impostoEstadual'];

        $respostaVenda = $this->realizaVenda(
            $NRORG,
            $CDFILIAL,
            $CDLOJA,
            $CDCAIXA,
            $CDVENDEDOR,
            $CDOPERADOR,
            $DTABERCAIX,
            $DTVENDA,
            $IDTPEMISVEND,
            $TOTALVENDABRT,
            $ITEMVENDA,
            $TIPORECE,
            $NMCONSVEND,
            $NRINSCRCONS,
            $DSENDECONSVENDA,
            $CDSENHAPED,
            $NRSEQVENDA,
            $VRTROCOVEND,
            $EMAIL,
            $VRDESCVENDA,
            $VRTOTTRIBIBPT,
            $CDCLIENTE,
            $CDCONSUMIDOR,
            $NRVENDAREST,
            $NRCOMANDA,
            $NRMESA,
            $IDORIGEMVENDA,
            $VRTXSEVENDA,
            $NRPESMESAVENDA,
            $DSCOMANDA,
            $FAMSALDOPROD,
            $TOTALVENDALIQ,
            $DTHRMESAFECHVEN,
            $fidelityDetails,
            $FIDELITYDISCOUNT,
            $cashbackUsed4ALL,
            $FIDELITYVALUE,
            $CODPROMO,
            $motivoDesconto,
            $CDGRPOCORDESC,
            $SUBSIDY,
            $DSOBSFINVEN,
            $ITVENDADES,
            $DELIVERY,
            $REPIQUE,
            $NRCPFFIDUDS,
			$VRDESCCASHUDS,
			$arrPromocoesIntUtilizadas,
            $CDSUPERVISOR
        );

        // VERIFICAÇÃO DE DADOS NAS TABELAS
        $this->validaIntegridadeTabelas($CDFILIAL, $CDCAIXA, $respostaVenda['NRSEQVENDA'], $DTVENDA);
        // FIM DA VERIFICAÇÃO DOS DADOS

        $result = array(
            'error' => false,
            'message' => '',
            'messageCurl' => '',
            'NRSEQVENDA' => $respostaVenda['NRSEQVENDA'],
            'mensagemImpressao' => '',
            'dadosImpressao' => array(),
            'resultMail' => null,
            'errPainelSenha' => ''
        );
        if (!$simulateSaleValidation) {
            $valorVenda = $this->calculaValorDaVenda($ITEMVENDA);
            try {
                $IDEMPRESAMEI = $this->entityManager->getConnection()->fetchAssoc("BUSCA_FILESERVER")['IDEMPRESAMEI'];
            } catch (\Exception $e){
                $IDEMPRESAMEI = 'N';
            }

            if ($IDEMPRESAMEI !== 'S'){
                if ($valorVenda['total'] <= 0 && $valorVenda['totalCancelamento'] > 0) {
                    $this->cancelaVenda($CDFILIAL, $CDCAIXA, $NRORG, $respostaVenda['NRSEQVENDA'], $CDOPERADOR, 'R', null);
                } else if ($IDTPEMISVEND === self::TIPO_SAT) {
				$FIDELITYVALUE = $FIDELITYVALUE + $VRDESCCASHUDS;
                    $respostaTransmissao = $this->sat->vendaSAT($CDFILIAL, $CDCAIXA, $respostaVenda['NRSEQVENDA'], $NRORG, $NRINSCRCONS, $NMCONSVEND, $CDSENHAPED, $CDOPERADOR, $VRTOTTRIBIBPT, $impostoEstadual, $imprimeCupom, $FIDELITYVALUE, $NRMESA, $INFORMAEMAIL, $DSEMAILCONS, $IDENVCUPOMEMAIL, $saleCode, $NRCPFFIDUDS);
                    $this->util->geraLog(' -Transmissão SAT: ' . ($respostaTransmissao['error'] ? 'com' : 'sem') . ' erro.', $CDFILIAL, $CDCAIXA, $DTVENDA);
                    if (!$respostaTransmissao['error']) {
                        $result['dadosImpressao'] = $respostaTransmissao['dadosImpressao'];
                        $result['mensagemImpressao'] = $respostaTransmissao['mensagemImpressao'];
                        $result['errPainelSenha'] = $respostaTransmissao['errPainelSenha'];
                    } else {
                        $result = $respostaTransmissao;
                    }
                } else if ($IDTPEMISVEND === self::TIPO_NFCE) {
                    $this->util->geraLog(' -Inicio da Transmissão NFCE.', $CDFILIAL, $CDCAIXA, $DTVENDA);
				$FIDELITYVALUE = $FIDELITYVALUE + $VRDESCCASHUDS;
                    $respostaTransmissao = $this->notaFiscalService->transmitirNFCe($NRORG, $CDCAIXA, $CDFILIAL, $DTABERCAIX, $CDLOJA, $CDOPERADOR, $IDTPEMISVEND, $respostaVenda['NRSEQVENDA'], $VRTOTTRIBIBPT, $TOTALVENDALIQ, $imprimeCupom, $FIDELITYVALUE, $NRMESA, $DTVENDA, $saleCode, $INFORMAEMAIL, $DSEMAILCONS, $IDENVCUPOMEMAIL, $NRCPFFIDUDS);
                    $this->util->geraLog(' -Transmissão NFCE: ' . ($respostaTransmissao['error'] ? 'com' : 'sem') . ' erro.', $CDFILIAL, $CDCAIXA, $DTVENDA);
                    if (!$respostaTransmissao['error']) {
                        $result['dadosImpressao'] = $respostaTransmissao['dadosImpressao'];
                        $result['mensagemImpressao'] = $respostaTransmissao['mensagemImpressao'];
                        $result['errPainelSenha'] = $respostaTransmissao['errPainelSenha'];
                        $result['IDSTATUSNFCE'] = $respostaTransmissao['IDSTATUSNFCE'];
                        $result['mensagemNfce'] = $respostaTransmissao['mensagemNfce'];
                        // Parametro para impressão de nota fiscal no SAAS.
                        $result['paramsImpressora'] = $respostaTransmissao['paramsImpressora'];
                    } else {
                        $this->util->geraLog('Erro: ' . $respostaTransmissao['message'], $CDFILIAL, $CDCAIXA, $DTVENDA);
                        $result = $respostaTransmissao;
                    }
                }
            }
            else {
                $params = array(
                    'IDVENDAGERENCIAL' => 'S',
                    'NRORG' => $NRORG,
                    'NRSEQVENDA' => $respostaVenda['NRSEQVENDA'],
                    'CDFILIAL' => $CDFILIAL,
                    'CDCAIXA' => $CDCAIXA
                );
                $this->entityManager->getConnection()->executeQuery("UPDATE_VENDA_MEI", $params);
            }

            // Insere na EXTRATOCONS Online caso tenha vendido crédito pessoal.
            try {
                if (!$respostaTransmissao['error'] && !empty($respostaVenda['EXTRATOCONSPARAMS'])){
                    foreach($respostaVenda['EXTRATOCONSPARAMS'] as $params){
                        $this->extratocons->insereExtratocons($params);
                    }
                    $this->util->geraLog(' -Saldo do consumidor inserido na EXTRATOCONS Online.', $CDFILIAL, $CDCAIXA, $DTVENDA);
                }
            } catch(\Exception $e){
                $result['messageCurl'] = 'Não houve retorno do EXTRATOCONS ONLINE. Saldo do consumidor pode não ter sido atualizado.';
                $this->util->geraLog(' -Não houve retorno do EXTRATOCONS ONLINE.', $CDFILIAL, $CDCAIXA, $DTVENDA);
            }

            // Transmite os dados para a 4ALL e UDS.
            try {
                if (!empty($respostaVenda['4ALLDATA']) && !$result['error']){
                    $this->extratocons->transmiteVenda(json_encode($respostaVenda['4ALLDATA']),'4ALL');
                }
            } catch (\Exception $e){
                $result['messageCurl'] = 'Erro ao transmitir a venda para o programa de fidelidade:<br>' . $e->getMessage() . '<br>';
            }
            try {
                if (!empty($respostaVenda['UDSDATA']) && !$result['error']){
                    $cpfcli = $respostaVenda['UDSDATA']['cpfCli'];
                    $resultFidelidade = $this->fidelidadeUDSAPI->transmiteVendaUDS(json_encode($respostaVenda['UDSDATA']));
                    $this->util->geraLog(' -Transmite Venda para API fidelidadde UDS.', $CDFILIAL, $CDCAIXA, $DTVENDA);
                    $result['IDENVIOFIDUDS']='N';
                    if(isset($resultFidelidade['success'])){
                        if($resultFidelidade['success']){
                            $result['IDENVIOFIDUDS']='V';
                            $params = array(
                                'IDENVIOFIDUDS' => $result['IDENVIOFIDUDS'],
                                'NRORG' => $NRORG,
                                'NRSEQVENDA' => $respostaVenda['NRSEQVENDA'],
                                'CDFILIAL' => $CDFILIAL,
                                'CDCAIXA' => $CDCAIXA
                            );
                            $this->entityManager->getConnection()->executeQuery("UPDATE_VENDA_IDENVIOFIDUDS", $params);
                        }
                    }
                }
            } catch (\Exception $e){
                $result['messageCurl'] = 'Erro ao transmitir a venda para o programa de fidelidade:<br>' . $e->getMessage() . '<br>';
                $this->util->geraLog('Erro ao transmitir a venda para o programa de fidelidade:<br>' . $e->getMessage(), $CDFILIAL, $CDCAIXA, $DTVENDA);
            }

			//transmite para Integração de Promoções
            try {
                if (!empty($respostaVenda['PROMOINTDATA']) && !$result['error']){
                    $this->util->geraLog(' -Transmite PROMOVENDA para API Integração de Promoções.', $CDFILIAL, $CDCAIXA, $DTVENDA);
					
					$retSendOrderStatus = $this->PromocoesIntServiceAPI->chamadaAPIPromocoesInt('sendOrderStatus',$respostaVenda['PROMOINTDATA']);

                    if(isset($retSendOrderStatus['success'])){
                        if(!$retSendOrderStatus['success']){
                            if(isset($retSendOrderStatus['message'])){
								if(!empty($retSendOrderStatus['message'])){
									throw new \Exception('Erro na API Integração de Promoções: '.$retSendOrderStatus['message']);
								} else {
									throw new \Exception('Erro ao transmitir a venda para o programa a API Integração de Promoções');
								}
							}
                        }
                    } else {
						throw new \Exception('Erro ao transmitir a venda para o programa a API Integração de Promoções');
					}
                }
            } catch (\Exception $e){
                $result['messageCurl'] = $result['messageCurl'].' -- Erro ao transmitir a venda para a API Integração de Promoções:<br>' . $e->getMessage() . '<br>';
                $this->util->geraLog('Erro ao transmitir a venda para o programa a API Integração de Promoções:<br>' . $e->getMessage(), $CDFILIAL, $CDCAIXA, $DTVENDA);
				throw new \Exception($e->getMessage());
            }

            if (!empty($result['mensagemImpressao'])){
                $result['mensagemImpressao'] = 'Ocorreu um problema na impressão da nota fiscal. ' . $result['mensagemImpressao'] . '<br>';
            }
        }
        if ($result['error']) {
            $respostaCancelamento = $this->cancelaVenda($CDFILIAL, $CDCAIXA, $NRORG, $respostaVenda['NRSEQVENDA'], $CDOPERADOR, 'R', null, true);
            if ($respostaCancelamento['error']) {
                $result['message'] = $result['message'] . ' - ' . $respostaCancelamento['message'];
            }
        } else if (isset($EMAIL)) {
            $resultMail = $this->util->sendEmailVenda($EMAIL, $respostaTransmissao, $DTVENDA, $IDTPEMISVEND);
            $result['resultMail'] = $resultMail;
        }

        return $result;
    }

    public function validaVendaComanda($CDFILIAL, $dadosMesa, $arrayPosicoes, $CDLOJA, $VRDESCVENDA, $VRTXSEVENDA, $CDCLIENTE, $DELIVERY, $VRACRCOMANDA, $TOTALPRODUTOS, $REPIQUE = 0, $VRDESCCASHUDS) {
		$VRDESCVENDA = $VRDESCVENDA /*+ $VRDESCCASHUDS*/;
        $TOTALVENDA = self::buscaTotalComanda($CDFILIAL, $dadosMesa, $arrayPosicoes);
        $ITVENDADES = self::montaItemsDesistencia($CDFILIAL, $dadosMesa[0]['NRVENDAREST']);
        $ITEMVENDA = self::montaItemVenda($CDFILIAL, $dadosMesa, $arrayPosicoes, $CDLOJA, $VRDESCVENDA, $TOTALVENDA, $VRTXSEVENDA, $CDCLIENTE, $DELIVERY, $VRACRCOMANDA, $REPIQUE);
        $valorVenda = $this->calculaValorDaVenda($ITEMVENDA);
        $totalProdutos = $valorVenda['total'];
        $totalCancelamento = $valorVenda['totalCancelamento'];
        $productDiscounts = $valorVenda['discounts'];

        $VALORVENDAFRONT = round($TOTALPRODUTOS - $VRDESCVENDA, 2);

        if ($DELIVERY) {
            // adiciona taxa de entrega ao valor da venda
            $totalProdutos += $VRACRCOMANDA;
        }

        if (count($ITEMVENDA) == 0) {
            $validacaoVenda = array(
                'error' => true,
                'message' => 'Não foram encontrados produtos para esta venda.'
            );
        } else if ($totalProdutos <= 0 && $totalCancelamento <= 0) {
            $validacaoVenda = array(
                'error' => true,
                'message' => 'O total da venda deve ser maior que 0.'
            );
        } else if (round($totalProdutos, 2) != $VALORVENDAFRONT){
            $VALORVENDAFRONT = str_replace('.', ',', number_format($VALORVENDAFRONT, 2));
            $totalProdutos = str_replace('.', ',', number_format($totalProdutos, 2));
            $validacaoVenda = array(
                'error' => true,
                'message' => 'Valor total da venda ('. $VALORVENDAFRONT .') difere da soma dos preços dos produtos (' . $totalProdutos . ').'
            );
        } else {
            $validacaoVenda = array(
                'error' => false,
                'ITEMVENDA' => $ITEMVENDA,
                'ITVENDADES' => $ITVENDADES,
                'TOTALVENDALIQ' => $totalProdutos,
                'TOTALVENDABRT' => $TOTALVENDA,
                'SUBSIDY' => $valorVenda['subsidy'],
                'PRODUCTDISCOUNTS' => $productDiscounts
            );
        }
        return $validacaoVenda;
    }

    public function buscaTotalComanda($CDFILIAL, $dadosMesa, $arrayPosicoes) {
        $total = 0;
        foreach($dadosMesa as $mesaAtual) {
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'NRVENDAREST' => $mesaAtual['NRVENDAREST'],
                'NRCOMANDA' => $mesaAtual['NRCOMANDA']
            );
            $totalPorPosicao = $this->entityManager->getConnection()->fetchAll("CALCULA_TOTAL_COMANDA", $params);
            $totalPorPosicaoFiltrado = self::filtraItensPorPosicoes($totalPorPosicao, $arrayPosicoes);
            $total += self::totalizaMesa($totalPorPosicaoFiltrado);
        }
        return $total;
    }

    private function totalizaMesa($totalPorPosicao) {
        $total = 0;
        foreach ($totalPorPosicao as $totalAtual) {
            $total += $totalAtual['TOTAL'];
        }
        return $total;
    }

    private function buscaDadosTaxaServico($CDFILIAL, $CDLOJA) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA
        );
        return $this->entityManager->getConnection()->fetchAssoc("BUSCA_DADOS_TAXA_SERVICO", $params);
    }

    private function montaItemVenda($CDFILIAL, $dadosMesa, $arrayPosicoes, $CDLOJA, $VRDESCVENDA, $TOTALVENDA, $VRTXSEVENDA = null, $CDCLIENTE, $DELIVERY, $VRACRCOMANDA = null, $REPIQUE = 0){
        $produtos = self::buscaItensPedidos($CDFILIAL, $CDLOJA, $dadosMesa, $arrayPosicoes);
        if( empty($produtos) && !$DELIVERY ){
            throw new \Exception('Não foi realizado nenhum pedido para esta mesa/comanda.', 1);
        }
        $observacoes = self::buscaItensPedidosObs($CDFILIAL, $dadosMesa);
        $dadosTaxa = $this->buscaDadosTaxaServico($CDFILIAL, $CDLOJA);
        $itensTratados = array();

        $PRODTX = 0; // calcula total dos produtos que entram na taxa de serviço
        $PRODTXENTREGA = 0; // calcula total dos produtos que entram na taxa de entrega
        foreach ($produtos as $produto) {
            if ($produto['IDSTPRCOMVEN'] != '6'){
                $vrProduto = self::calculaTotalItem($produto);

                if ($produto['IDCOBTXSERV'] == 'S'){
                    $PRODTX += $vrProduto;
                }
                $PRODTXENTREGA += $vrProduto;
            }
        }
        $totalRateioDesconto = 0;
        $totalRateioTaxa = 0;
        //retirado taxa de entrega para ficar de acordo com o FOS
        // $totalRateioTaxaEntrega = 0;
        //Trecho abaixo valida se mesa ainda esta em aberto
        $params = array(
            $CDFILIAL,
            $dadosMesa[0]['NRCOMANDA'],
            $dadosMesa[0]['NRVENDAREST'],
            $CDLOJA
        );

        if(!$DELIVERY){
            $validaMesa = $this->entityManager->getConnection()->fetchAssoc("SQL_DADOS_MESA", $params);
            if ($validaMesa == false) {
                $this->util->geraLog('Erro: Mesa não encontrada. \nNRCOMANDA: '.$dadosMesa[0]['NRCOMANDA'].
                        ' NRVENDAREST: '.$dadosMesa[0]['NRVENDAREST'].' Produtos: '.$produtos.
                        'Posições: '.$arrayPosicoes.' \nDados mesa: '.$dadosMesa.' \n\n',
                    $CDFILIAL,
                    'NA',
                    new \Datetime()
                );

                throw new \Exception('Mesa não encontrada.', 1);
            }
        }

        foreach ($produtos as $produtoAtual){
            $valorDescontoAtual = 0;
            if ($VRDESCVENDA > 0 && $produtoAtual['IDSTPRCOMVEN'] != '5' &&
                $produtoAtual['IDSTPRCOMVEN'] != '6' && $produtoAtual['IDSTPRCOMVEN'] != '7'){
                $valorDescontoAtual = $this->rateiaDesconto($produtoAtual, $VRDESCVENDA, $TOTALVENDA);
                $totalRateioDesconto += $valorDescontoAtual;
            }
            $novoItem = array();
            $novoItem['NRPRODCOMVEN'] = $produtoAtual['NRPRODCOMVEN'];
            $novoItem['NMPRODUTO'] = $produtoAtual['NMPRODUTO'];
            $novoItem['CDPRODUTO'] = $produtoAtual['CDPRODUTO'];
            $novoItem['QTPRODVEND'] = $produtoAtual['QTPRODCOMVEN'];
            $novoItem['VRUNITVEND'] = $produtoAtual['VRPRECCOMVEN'];
            $novoItem['VRUNITVENDCL'] = $produtoAtual['VRPRECCLCOMVEN'];
            $novoItem['VRDESITVEND'] = (float)$produtoAtual['VRDESCCOMVEN'] + $valorDescontoAtual;
            $novoItem['VRACRITVEND'] = (float)$produtoAtual['VRACRCOMVEN'];
            if ($dadosTaxa['IDCOMISVENDA'] != 'N' && $dadosTaxa['IDTRATTAXASERV'] === 'A' && $produtoAtual['IDCOBTXSERV'] == 'S' && $produtoAtual['IDSTPRCOMVEN'] != '5' && $produtoAtual['IDSTPRCOMVEN'] != '6' && $produtoAtual['IDSTPRCOMVEN'] != '7'){
                if (isset($VRTXSEVENDA) && $VRTXSEVENDA > 0){
                    $taxa = $VRTXSEVENDA / $PRODTX;
                //something wrong happened where
                } else if ($VRTXSEVENDA <= 0) {
                    $taxa = 0;
                } else {
                    $taxa = floatval($dadosTaxa['VRCOMISVENDA'])/100;
                }
                $acrescimo = floatval(bcmul(str_replace(',','.',strval(self::calculaTotalItem($produtoAtual))), str_replace(',','.',strval($taxa)), '2'));
                $novoItem['VRACRITVEND'] += $acrescimo;
                $totalRateioTaxa += $acrescimo;
            }
            if ($produtoAtual['IDSTPRCOMVEN'] != '5' && $produtoAtual['IDSTPRCOMVEN'] != '6' && $produtoAtual['IDSTPRCOMVEN'] != '7'){
                //retirado taxa de entrega para ficar de acordo com o FOS
                // if(isset($VRTXSEVENDA) && $VRACRCOMANDA > 0){
                //     $taxa = $VRACRCOMANDA / $PRODTXENTREGA;
                // }else{
                //     $taxa = 0;
                // }
                $taxa = 0;
                $acrescimo = floatval(bcmul(str_replace(',','.',strval(self::calculaTotalItem($produtoAtual))), str_replace(',','.',strval($taxa)), '2'));
                $novoItem['VRACRITVEND'] += $acrescimo;
                // $totalRateioTaxaEntrega += $acrescimo;
            }

            $novoItem['IDSITUITEM'] = $produtoAtual['IDSTPRCOMVEN'] !== self::ITEM_CANCELADO ? self::ITEM_APROVADO : self::ITEM_CANCELADO;
            $novoItem['IDTIPOITEM'] = null;
            $novoItem['OBSERVACOES'] = self::filtraObservacoes($observacoes, $produtoAtual);
            $novoItem['IDTIPOCOMPPROD'] = '0';
            $novoItem['IDIMPPRODUTO'] = '1';
            $novoItem['CDGRPOCOR'] = $produtoAtual['CDGRPOCOR'];
            $novoItem['DSOBSITEMVENDA'] = $produtoAtual['TXPRODCOMVEN'];
            $novoItem['DSOBSPEDDIGITA'] = $produtoAtual['DSOBSPEDDIGCMD'];
            $novoItem['CDSUPERVISOR'] = $produtoAtual['CDSUPERVISOR'];
            $novoItem['DTHRINCOMVEN'] = $produtoAtual['DTHRINCOMVEN'];
            $novoItem['NRVENDAREST'] = $produtoAtual['NRVENDAREST'];
            $novoItem['NRCOMANDA'] = $produtoAtual['NRCOMANDA'];
            $novoItem['IDCOBTXSERV'] = $produtoAtual['IDCOBTXSERV'];
            $novoItem['PRECOFINAL'] = self::getItemPrice($novoItem, '0');
            $novoItem['IDPRODPRODUZC'] = $produtoAtual['IDPRODPRODUZ'];
            $novoItem['DSOBSDESCIT'] = $produtoAtual['DSOBSDESCIT'];
            $novoItem['IDORIGEMVENDA'] = $produtoAtual['IDORIGEMVENDA'];
            $novoItem['CDGRPOCORDESCIT'] = $produtoAtual['CDGRPOCORDESCIT'];
            $novoItem['CDVENDEDOR'] = $produtoAtual['CDVENDEDOR'];
            $novoItem['CDOPERCANC'] = $produtoAtual['CDOPERCANC'];
            $novoItem['DTHRPRODCANVEN'] = $produtoAtual['DTHRPRODCANVEN'];
            $novoItem['CDPRODPROMOCAO'] = $produtoAtual['CDPRODPROMOCAO'];
            $novoItem['NRLUGARMESA'] = $produtoAtual['NRLUGARMESA'];
            $novoItem['VOUCHER'] = null;
            $novoItem['CDCAMPCOMPGANHE'] = $produtoAtual['CDCAMPCOMPGANHE'];
            $novoItem['DTINIVGCAMPCG'] = $produtoAtual['DTINIVGCAMPCG'];
            $novoItem['IDIMPPRGRPROMO'] = $produtoAtual['IDIMPPRGRPROMO'];
            $novoItem['IDOBSERVACAOAD'] = $produtoAtual['IDOBSERVACAOAD'];
            $novoItem['VOUCHER'] = array(
                'CDCUPOMDESCFOS' => $produtoAtual['CDCUPOMDESCFOS']
            );
            $novoItem['CDSUPERVDESC'] = $produtoAtual['CDSUPERVDESC'];

            array_push($itensTratados, $novoItem);
        }
        // ajuda diferença do desconto quando descontoVenda > descontoTotalRateado
        if ($totalRateioDesconto < $VRDESCVENDA) {
            $diferenca = round($VRDESCVENDA - $totalRateioDesconto, 2);
            $this->ajustaDiferenca($itensTratados, $diferenca, 'VRDESITVEND');
        }

        if ($totalRateioTaxa < $VRTXSEVENDA && $dadosTaxa['IDTRATTAXASERV'] === 'A') {
            $diferenca = round($VRTXSEVENDA - $totalRateioTaxa, 2);
            $this->ajustaDiferenca($itensTratados, $diferenca, 'VRACRITVEND');
        }

        //retirado taxa de entrega para ficar de acordo com o FOS
        // if ($totalRateioTaxaEntrega < $VRACRCOMANDA && $dadosTaxa['IDTRATTAXAENTR'] === 'A') {
        //     $diferenca = round($VRACRCOMANDA - $totalRateioTaxaEntrega, 2);
        //     $this->ajustaDiferenca($itensTratados, $diferenca, 'VRACRITVEND', true);
        // }

        $produtoTaxaServico = self::adicionaProdutoTaxaServico($dadosTaxa, $dadosMesa[0]['NRVENDAREST'], $dadosMesa[0]['NRCOMANDA'], $REPIQUE, $VRTXSEVENDA, $TOTALVENDA);
        if (!empty($produtoTaxaServico['TAXAORIG'])) {
            array_push($itensTratados, $produtoTaxaServico['TAXAORIG']);
            if (!empty($produtoTaxaServico['TAXAEXCE'])){
                array_push($itensTratados, $produtoTaxaServico['TAXAEXCE']);
            }
        }

        //retirado taxa de entrega para ficar de acordo com o FOS
        // TAXA DE ENTREGA
        // if ($dadosTaxa['CDPRODTAXAENTR'] !== null && $VRACRCOMANDA > 0){
        //     $paramProdutoEntrega = array(
        //         'CDPRODUTO'     => $dadosTaxa['CDPRODTAXAENTR']
        //     );
        //     $produtoTaxaEntrega = $this->entityManager->getConnection()->fetchAssoc("PRODUTO", $paramProdutoEntrega);
        //     var_dump($dadosTaxa['CDPRODTAXAENTR'], $produtoTaxaEntrega);die;
        //     $itemTaxa = array(
        //         'NRPRODCOMVEN'   => null,
        //         'NMPRODUTO'      => $produtoTaxaEntrega['NMPRODUTO'],
        //         'CDPRODUTO'      => $dadosTaxa['CDPRODTAXAENTR'],
        //         'QTPRODVEND'     => 1,
        //         'VRUNITVEND'     => 0,
        //         'VRUNITVENDCL'   => 0,
        //         'VRDESITVEND'    => 0,
        //         'VRACRITVEND'    => floatval(bcdiv(str_replace(',','.',strval($VRACRCOMANDA)), '1', '2')),
        //         'IDSITUITEM'     => self::ITEM_APROVADO,
        //         'IDTIPOITEM'     => null,
        //         'OBSERVACOES'    => array(),
        //         'IDTIPOCOMPPROD' => '0',
        //         'IDIMPPRODUTO'   => '1',
        //         'CDGRPOCOR'      => null,
        //         'DSOBSITEMVENDA' => null,
        //         'DSOBSPEDDIGITA' => null,
        //         'CDSUPERVISOR'   => null,
        //         'DTHRINCOMVEN'   => new \DateTime(),
        //         'NRVENDAREST'    => $produtos[0]['NRVENDAREST'],
        //         'NRCOMANDA'      => $produtos[0]['NRCOMANDA'],
        //         'IDPRODPRODUZC'  => null,
        //         'DSOBSDESCIT'    => null,
        //         'IDORIGEMVENDA'  => null,
        //         'CDGRPOCORDESCIT'=> null,
        //         'CDVENDEDOR'     => null,
        //         'CDPRODPROMOCAO' => null,
        //         'VOUCHER'        => null,
        //         'CDCAMPCOMPGANHE' => null,
        //         'DTINIVGCAMPCG' => null,
        //         'IDIMPPRGRPROMO' => 'N',
        //         'IDOBSERVACAOAD' => 'N',
        //         'TAXADESERVICO' => true
        //     );
        //     $itemTaxa['PRECOFINAL'] = self::getItemPrice($itemTaxa, '0');
        //     array_push($itensTratados, $itemTaxa);
        // }
        return $itensTratados;
    }

    private function rateiaDesconto($produtoAtual, $VRDESCVENDA, $TOTALVENDA) {
        $descontoItemAtual = 0;
        $valorTotalItem = $this->calculaTotalItem($produtoAtual);
        $descontoItemAtual = floatval(bcdiv(str_replace(',','.',strval(($valorTotalItem / $TOTALVENDA) * $VRDESCVENDA)), '1', '2'));
        return $descontoItemAtual;
    }

    private function calculaTotalItem($produtoAtual) {
        if (isset($produtoAtual['VRDESITVEND'])){
            return round(floatval(bcdiv(str_replace(',','.',strval(($produtoAtual['VRUNITVEND'] + $produtoAtual['VRUNITVENDCL']) * $produtoAtual['QTPRODVEND'])), '1', '2')) + $produtoAtual['VRACRITVEND'] - $produtoAtual['VRDESITVEND'], 2);
        } else {
            return round(floatval(bcmul(str_replace(',','.',strval(($produtoAtual['VRPRECCOMVEN'] + $produtoAtual['VRPRECCLCOMVEN']))), str_replace(',','.',strval($produtoAtual['QTPRODCOMVEN'])), '2')) + $produtoAtual['VRACRCOMVEN'] - $produtoAtual['VRDESCCOMVEN'], 2);
        }
    }

    private function ajustaDiferenca(&$produtos, $diferenca, $modo, $txentrega = false){
        $qtdRateio = intval(bcdiv(strval($diferenca), '0.01', 2));
        $i = 0;
        while ($qtdRateio > 0) {
            $qtdRateio = self::alteraValor($produtos, $qtdRateio, $modo, $txentrega);
            $i++;
            if ($i == 1000) throw new \Exception("Erro no rateio do desconto/gorjeta.");
        }
    }

    private function alteraValor(&$produtos, $qtdRateio, $modo, $txentrega){
        foreach ($produtos as &$produtoAtual) {
            if (($modo == 'VRDESITVEND' || $produtoAtual['IDCOBTXSERV'] == 'S' || $txentrega) && $produtoAtual['IDSITUITEM'] != '6'){
                if (($produtoAtual['IDTIPOCOMPPROD'] == '3' && $produtoAtual['IDIMPPRODUTO'] == '2') || $produtoAtual['IDTIPOCOMPPROD'] == 'C'){
                    foreach ($produtoAtual['itensCombo'] as &$itemComboAtual) {
                        $totalProduto = self::calculaTotalItem($itemComboAtual);
                        if ($itemComboAtual[$modo] >= 0.01 && $totalProduto > 0.01){
                            $itemComboAtual[$modo] = $itemComboAtual[$modo] + 0.01;
                            $itemComboAtual['PRECOFINAL'] = self::getItemPrice($itemComboAtual, $produtoAtual['IDTIPOCOMPPROD']);
                            $qtdRateio--;
                        }
                        if ($qtdRateio == 0) {
                            break 2;
                        }
                    }
                } else {
                    $totalProduto = self::calculaTotalItem($produtoAtual);
                    if ($modo == 'VRDESITVEND'){
                        if ($produtoAtual[$modo] >= 0.01 && $produtoAtual['IDSITUITEM'] != '6' && $totalProduto > 0.01){
                            $produtoAtual[$modo] = $produtoAtual[$modo] + 0.01;
                            $produtoAtual['PRECOFINAL'] = self::getItemPrice($produtoAtual, $produtoAtual['IDTIPOCOMPPROD']);
                            $qtdRateio--;
                        }
                    }
                    else {
                        if ($produtoAtual['IDSITUITEM'] != '6'){
                            $produtoAtual[$modo] = $produtoAtual[$modo] + 0.01;
                            $produtoAtual['PRECOFINAL'] = self::getItemPrice($produtoAtual, $produtoAtual['IDTIPOCOMPPROD']);
                            $qtdRateio--;
                        }
                    }
                    if ($qtdRateio == 0) {
                        break;
                    }
                }
            }
        }

        return $qtdRateio;
    }

    private function filtraObservacoes($observacoes, $produtoAtual) {
        $arrayFiltrado = array();
        foreach ($observacoes as $obsAtual) {
            if (($obsAtual['NRVENDAREST'] == $produtoAtual['NRVENDAREST']) &&
               ($obsAtual['NRCOMANDA'] == $produtoAtual['NRCOMANDA']) &&
               ($obsAtual['NRPRODCOMVEN'] == $produtoAtual['NRPRODCOMVEN'])) {

                array_push($arrayFiltrado, $obsAtual);
            }
        }
        return $arrayFiltrado;
    }

    public function buscaItensPedidos($CDFILIAL, $CDLOJA, $dadosMesa, $arrayPosicoes) {
        $arrayItens = array();
        foreach ($dadosMesa as $mesaAtual) {
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'NRVENDAREST' => $mesaAtual['NRVENDAREST'],
                'NRCOMANDA' => $mesaAtual['NRCOMANDA']
            );
            $itensMesaAtual = $this->entityManager->getConnection()->fetchAll("BUSCA_ITENS_PEDIDOS", $params);
            $itensMesaAtual = $this->precoService->subgroupDiscountTableInterface($itensMesaAtual, $CDFILIAL, $CDLOJA);
            $arrayItens = array_merge($arrayItens, $itensMesaAtual);
        }
        $arrayItens = self::filtraItensPorPosicoes($arrayItens, $arrayPosicoes);
        return $arrayItens;
    }

    private function filtraItensPorPosicoes($arrayItens, $arrayPosicoes) {
        if (!empty($arrayPosicoes)) {
            $itensFiltrados = array();
            foreach ($arrayItens as $itemAtual) {
                foreach ($arrayPosicoes as $posicaoAtual) {
                    if ($itemAtual['NRLUGARMESA'] == $posicaoAtual) {
                        array_push($itensFiltrados, $itemAtual);
                    }
                }
            }
        } else {
            $itensFiltrados = $arrayItens;
        }
        return $itensFiltrados;
    }

    private function buscaItensPedidosObs($CDFILIAL, $dadosMesa) {
        $arrayObs = array();
        foreach ($dadosMesa as $mesaAtual) {
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'NRVENDAREST' => $mesaAtual['NRVENDAREST'],
                'NRCOMANDA' => $mesaAtual['NRCOMANDA']
            );
            $obsMesaAtual = $this->entityManager->getConnection()->fetchAll("BUSCA_ITENS_PEDIDOS_OBS", $params);
            $arrayObs = array_merge($arrayObs, $obsMesaAtual);
        }
        return $arrayObs;
    }

    private function adicionaObsCobrada(&$ITEMVENDA){
        // função utilizada para adicionar as observações cobradas no array normal de produto
        $produtosObs = array();

        foreach ($ITEMVENDA as $item){
            $IDORIGEMVENDA = !empty($item['IDORIGEMVENDA']) ? $item['IDORIGEMVENDA'] : null;
            // verifica observação dos itens pai
            foreach ($item['OBSERVACOES'] as $obsAtual) {
                $obsCobrada = self::buscaProdutoObs($obsAtual['CDGRPOCOR'], $obsAtual['CDOCORR']);
                if (!empty($obsCobrada)){
                    $QTPRODVEND = $item['QTPRODVEND'] * (!empty($obsAtual['QTPRODVEND']) ? $obsAtual['QTPRODVEND'] : 1);
                    array_push($produtosObs, self::montaItemObs($obsCobrada['CDPRODUTO'], $obsCobrada['NMPRODUTO'], $QTPRODVEND, $IDORIGEMVENDA, $item['CDVENDEDOR']));
                }
            }

            // verifica observação dos itens filhos (promoção ou produto combinado)
            foreach ($item['itensCombo'] as $itensCombo) {
                foreach ($itensCombo['OBSERVACOES'] as $obsAtual) {
                    $obsCobrada = self::buscaProdutoObs($obsAtual['CDGRPOCOR'], $obsAtual['CDOCORR']);
                    if (!empty($obsCobrada)){ // As observações dos produtos IDIMPPRGRPROMO = 'S' são inseridas acima.
                        $QTPRODVEND = $item['QTPRODVEND'] * (!empty($obsAtual['QTPRODVEND']) ? $obsAtual['QTPRODVEND'] : 1);
                        array_push($produtosObs, self::montaItemObs($obsCobrada['CDPRODUTO'], $obsCobrada['NMPRODUTO'], $QTPRODVEND, $IDORIGEMVENDA, $item['CDVENDEDOR']));
                    }
                }
            }
        }

        $ITEMVENDA = array_merge($ITEMVENDA, $produtosObs);
    }

    private function montaItemObs($CDPRODUTO, $NMPRODUTO, $QTPRODVEND, $IDORIGEMVENDA, $CDVENDEDOR) {
        return array(
            'CDPRODUTO' => $CDPRODUTO,
            'NMPRODUTO' => $NMPRODUTO,
            'QTPRODVEND' => $QTPRODVEND,
            'QTPRODCOMVEN' => $QTPRODVEND,
            'VRUNITVEND' => 0, // preços adicionados posteriormente
            'VRUNITVENDCL' => 0,
            'REALSUBSIDY' => 0,
            'VRDESITVEND' => 0,
            'VRACRITVEND' => 0,
            'IDTIPOCOMPPROD' => '0',
            'IDIMPPRODUTO' => '1',
            'IDSITUITEM' => self::ITEM_APROVADO,
            'IDTIPOITEM' => null,
            'DSOBSPEDDIGITA' => null,
            'DSOBSITEMVENDA' => null,
            'CDSUPERVISOR' => null,
            'CDGRPOCOR' => null,
            'OBSERVACOES' => array(),
            'DSOBSDESCIT' => null,
            'CDGRPOCORDESCIT' => null,
            'IDORIGEMVENDA' => $IDORIGEMVENDA,
            'itensCombo' => array(),
            'CDVENDEDOR' => $CDVENDEDOR,
            'CDPRODPROMOCAO' => null,
            'VOUCHER' => null,
            'CDCAMPCOMPGANHE' => null,
            'DTINIVGCAMPCG' => null,
            'IDIMPPRGRPROMO' => 'N',
            'IDOBSERVACAOAD' => 'S'
        );
    }

    private function montaPrecosProdutos($TOTALPRODUTOS, $ITEMVENDA, $VRDESCVENDA, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR) {
        $retorno = array(
            'error' => false,
            'produtosComPreco' => array(),
            'totalDescontos' => 0
        );

        $totalRateioDesconto = 0;
        $totalDescontos = 0;
        foreach ($ITEMVENDA as &$ITEM) {
            if (($ITEM['IDTIPOCOMPPROD'] === '3' && $ITEM['IDIMPPRODUTO'] === '2') || $ITEM['IDTIPOCOMPPROD'] === 'C') {
                // itens do combo
                foreach ($ITEM['itensCombo'] as &$itemComboAtual) {
                    $produtoDefinido = self::definePrecoProduto($itemComboAtual, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR);

                    if (!$produtoDefinido['error']){
                        $itemComboAtual = $produtoDefinido['produto'];
                        $itemComboAtual['VRACRITVEND'] = floatval(bcmul(str_replace(',','.',strval($itemComboAtual['VRACRITVEND'])), str_replace(',','.',strval($itemComboAtual['QTPRODCOMVEN'])), '2'));
                        $itemComboAtual['VRDESITVEND'] = floatval(bcmul(str_replace(',','.',strval($itemComboAtual['VRDESITVEND'])), str_replace(',','.',strval($itemComboAtual['QTPRODCOMVEN'])), '2'));
                        if ($VRDESCVENDA > 0){
                            $valorDescontoAtual = $this->rateiaDesconto($itemComboAtual, $VRDESCVENDA, $TOTALPRODUTOS);
                            $itemComboAtual['VRDESITVEND'] += $valorDescontoAtual;
                            $totalRateioDesconto += $valorDescontoAtual;
                        }
                        $itemComboAtual['PRECOFINAL'] = self::getItemPrice($itemComboAtual, $ITEM['IDTIPOCOMPPROD']);
                        // Aplica o desconto do voucher.
                        if (!empty($itemComboAtual['VOUCHER'])){
                            if ($itemComboAtual['VOUCHER']['IDTIPODESC'] === "P"){
                                $voucherDiscount = bcmul(strval($itemComboAtual['PRECOFINAL']), strval($itemComboAtual['VOUCHER']['VRDESCCUPOM']/100), 2);
                            }
                            else {
                                $voucherDiscount = $itemComboAtual['VOUCHER']['VRDESCCUPOM'];
                            }
                            $itemComboAtual['VRDESITVEND'] += floatval($voucherDiscount);
                            if ($itemComboAtual['VRDESITVEND'] >= $itemComboAtual['PRECOFINAL']) $itemComboAtual['VRDESITVEND'] = $itemComboAtual['PRECOFINAL'] - 0.01 * $itemComboAtual['QTPRODCOMVEN'];
                            $itemComboAtual['PRECOFINAL'] = self::getItemPrice($itemComboAtual, $ITEM['IDTIPOCOMPPROD']);
                        }
                        $totalDescontos = floatval(bcadd(strval($totalDescontos), strval($itemComboAtual['VRDESITVEND']), 2));
                    } else {
                        $retorno = $produtoDefinido;
                        break 2;
                    }
                }
            } else {
                // produto normal
                $produtoDefinido = self::definePrecoProduto($ITEM, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR);

                if (!$produtoDefinido['error']){
                    $ITEM = $produtoDefinido['produto'];
                    $ITEM['VRACRITVEND'] = floatval(bcmul(str_replace(',','.',strval($ITEM['VRACRITVEND'])), str_replace(',','.',strval($ITEM['QTPRODCOMVEN'])), '2'));
                    $ITEM['VRDESITVEND'] = floatval(bcmul(str_replace(',','.',strval($ITEM['VRDESITVEND'])), str_replace(',','.',strval($ITEM['QTPRODCOMVEN'])), '2'));
                    if ($VRDESCVENDA > 0){
                        $valorDescontoAtual = $this->rateiaDesconto($ITEM, $VRDESCVENDA, $TOTALPRODUTOS);
                        $ITEM['VRDESITVEND'] += $valorDescontoAtual;
                        $totalRateioDesconto += $valorDescontoAtual;
                    }
                    $ITEM['PRECOFINAL'] = self::getItemPrice($ITEM, $ITEM['IDTIPOCOMPPROD']);
                    // Aplica o desconto do voucher.
                    if (!empty($ITEM['VOUCHER'])){
                        if ($ITEM['VOUCHER']['IDTIPODESC'] === "P"){
                            $voucherDiscount = bcmul(strval($ITEM['PRECOFINAL']), strval($ITEM['VOUCHER']['VRDESCCUPOM']/100), 2);
                        }
                        else {
                            $voucherDiscount = $ITEM['VOUCHER']['VRDESCCUPOM'];
                        }
                        $ITEM['VRDESITVEND'] += floatval($voucherDiscount);

                        $total = round(floatval(bcdiv(str_replace(',','.',strval(($ITEM['VRUNITVEND'] + $ITEM['VRUNITVENDCL']) * $ITEM['QTPRODVEND'])), '1', '2')) + $ITEM['VRACRITVEND'], 2);
                        if ($ITEM['VRDESITVEND'] >= $total){
                            $ITEM['VRDESITVEND'] = floatval(bcsub(strval($total), strval($ITEM['QTPRODCOMVEN'] * 0.01), 2));
                        }
                        $ITEM['PRECOFINAL'] = self::getItemPrice($ITEM, $ITEM['IDTIPOCOMPPROD']);
                    }
                    $totalDescontos = floatval(bcadd(strval($totalDescontos), strval($ITEM['VRDESITVEND']), 2));
                } else {
                    $retorno = $produtoDefinido;
                    break;
                }
            }
        }

        if (!$retorno['error']) {
            // ajuda diferença do desconto quando descontoVenda > descontoTotalRateado
            if ($totalRateioDesconto < $VRDESCVENDA) {
                $diferenca = round($VRDESCVENDA - $totalRateioDesconto, 2);

                $this->ajustaDiferenca($ITEMVENDA, $diferenca, 'VRDESITVEND');
                $totalDescontos = floatval(bcadd(strval($totalDescontos), strval($diferenca), 2));
            }

            $retorno['produtosComPreco'] = $ITEMVENDA;
            $retorno['totalDescontos'] = $totalDescontos;
        }
        return $retorno;
    }

    private function getItemPrice($ITEM, $IDTIPOCOMPPROD){
        if ($IDTIPOCOMPPROD != 'C'){
            return round(floatval(bcdiv(str_replace(',','.',strval(($ITEM['VRUNITVEND'] + $ITEM['VRUNITVENDCL']) * $ITEM['QTPRODVEND'])), '1', '2')) + $ITEM['VRACRITVEND'] - $ITEM['VRDESITVEND'], 2);
        } else {
            $preco = floatval(bcadd(bcmul(strval($ITEM['VRUNITVEND'] + $ITEM['VRUNITVENDCL']), strval($ITEM['QTPRODCOMVEN']), 3), strval($ITEM['VRACRITVEND'] - $ITEM['VRDESITVEND']), 2));
            if ($preco <= 0) $preco = 0.01;
            return $preco;
        }
    }

    private function definePrecoProduto($produto, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR){
        $result = array(
            'error' => false,
            'produto' => null
        );

        if (empty($produto['VRUNITVEND']) || !empty($produto['VOUCHER'])){
            $precoItem = $this->precoService->buscaPreco($CDFILIAL, $CDCLIENTE, $produto['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);
            if (!$precoItem['error']) {
                $produto['VRUNITVEND'] = $precoItem['PRECO'];
                $produto['VRUNITVENDCL'] = $precoItem['PRECOCLIE'];
                $produto['VRDESITVEND'] = $precoItem['DESC'];
                $produto['VRACRITVEND'] = $precoItem['ACRE'];

                $result['produto'] = $produto;
            } else {
                $result = $precoItem;
            }
        } else {
            $result['produto'] = $produto;
        }

        return $result;
    }

    public function cancelaVenda($CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $CDOPERADOR, $IDSTATUSNFCE, $CDSUPERVISOR, $errorFlag = false, $DSOBSCANCUP = '', $NRACESSOCANC = null){
        // IDSTATUSNFCE - R (recusado) / C (cancelado)
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRORG' => $NRORG,
            'NRSEQVENDA' => $NRSEQVENDA
        );
        $venda = $this->entityManager->getConnection()->fetchAll("BUSCA_VENDA", $params);
        $this->util->geraLog(' -CancelaVenda '.$NRSEQVENDA.', IDSTATUSNFCE='.$IDSTATUSNFCE, $CDFILIAL, $CDCAIXA, $venda[0]['DTVENDA']);
        $IDIMPVENDA = $venda[0]['IDIMPVENDA'] == 'I' ? 'I' : 'A';
		$boExtratoCons = false;
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRORG' => $NRORG,
            'NRSEQVENDA' => $NRSEQVENDA,
            'CDOPERULTATU' => $CDOPERADOR,
            'IDIMPVENDA' => $IDIMPVENDA,
            'IDSITUVENDA' => 'C',
            'IDSTATUSNFCE' => $IDSTATUSNFCE,
            'DSOBSCANCUP' => $DSOBSCANCUP,
            'CDSUPERVISOR' => $CDSUPERVISOR,
            'NRACESSOCANC' => $NRACESSOCANC
        );
        $this->entityManager->getConnection()->executeQuery("CANCELA_VENDA", $params);

        $dadosFilial = $this->entityManager->getConnection()->fetchAssoc("BUSCA_DADOS_FILIAL", array('CDFILIAL' => $CDFILIAL));
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRSEQVENDA' => $NRSEQVENDA
        );
        $movExtrato = $this->entityManager->getConnection()->fetchAll("BUSCA_MOVI_EXTRATOCONS", $params);

        $NRSEQMOVEXT = 1;
        foreach ($movExtrato as $family){
			$boExtratoCons = true;

            if ($this->databaseUtil->databaseIsOracle()){
                $DTABERCAIX = \DateTime::createFromFormat('Y-m-d H:i:s', $family['DTABERCAIX']);
            }
            else {
                $DTABERCAIX = \DateTime::createFromFormat('Y-m-d H:i:s.u', $family['DTABERCAIX']);
            }

            $VRSALDCONEXT = floatval($family['VRSALDCONEXT']) + floatval($family['VRMOVEXTCONS']);

            $dateReference = new \DateTime();
            $this->insertExtratoCons(
                $family['CDCLIENTE'],
                $family['CDCONSUMIDOR'],
                $family['CDFAMILISALD'],
                'WTR - Canc. Venda Caixa ' . $CDCAIXA,
                floatval($family['VRMOVEXTCONS']),
                'C',
                $VRSALDCONEXT,
                str_pad($NRSEQMOVEXT++, 3, '0', STR_PAD_LEFT),
                $family['CDTIPORECE'],
                $family['CDFILIAL'],
                $family['CDCAIXA'],
                $NRSEQVENDA,
                $DTABERCAIX,
                null,
                $dateReference
            );

            if ($dadosFilial['IDEXTCONSONLINE'] === 'S' && !$errorFlag){
                $params = array(
                    'CDCLIENTE' => $family['CDCLIENTE'],
                    'CDCONSUMIDOR' => $family['CDCONSUMIDOR'],
                    'CDFAMILISALD' => $family['CDFAMILISALD'],
                    'DTMOVEXTCONS' => new \DateTime(),
                    'NRSEQMOVEXT' => str_pad($NRSEQMOVEXT, 3, '0', STR_PAD_LEFT),
                    'CDTIPORECE' => $family['CDTIPORECE'],
                    'IDTPMOVEXT' => 'C',
                    'DSOPEEXTCONS' => 'WTR - Canc. Venda Caixa ' . $CDCAIXA,
                    'VRMOVEXTCONS' => floatval($family['VRMOVEXTCONS']),
                    'VRSALDCONEXT' => $VRSALDCONEXT,
                    'CDFILIAL' => $family['CDFILIAL'],
                    'CDCAIXA' => $family['CDCAIXA'],
                    'NRDEPOSICONS' => null,
                    'NRSEQMOVCAIXA' => null,
                    'IDIMPEXTRATO' => 'S'
                );
                $params['DTMOVEXTCONS'] = $params['DTMOVEXTCONS']->format('d/m/Y H:i:s');
                $this->extratocons->insereExtratocons($params);
            }
            else {
                $this->updateBalance($family['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $VRSALDCONEXT);
            }
            $this->updateBalance($family['CDCLIENTE'], $family['CDCONSUMIDOR'], $family['CDFAMILISALD'], $VRSALDCONEXT);

        }

        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'CDOPERULTATU' => $CDOPERADOR,
            'NRORG' => $NRORG,
            'NRSEQVENDA' => $NRSEQVENDA,
            'IDTIPOMOVIVE' => 'C'
        );
        $this->entityManager->getConnection()->executeQuery("CANCELA_MOVCAIXA", $params);

        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRORG' => $NRORG,
            'NRSEQVENDA' => $NRSEQVENDA
        );
        $MOVICLIE = $this->entityManager->getConnection()->fetchAll("BUSCA_MOVICLIE", $params);

        // Caso venda for do tipo 'DÉBITO CONSUMIDOR'
        if (count($MOVICLIE) > 0) {
            $VRSALDOCONS = $MOVICLIE[0]['VRSALDOCONS'];
            $VRMOVCLI = $MOVICLIE[0]['VRMOVCLI'];
            $VRSALDOCONS = $VRSALDOCONS + $VRMOVCLI;
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDCAIXA' => $CDCAIXA,
                'CDOPERULTATU' => $CDOPERADOR,
                'NRORG' => $NRORG,
                'NRSEQVENDA' => $NRSEQVENDA,
                'IDTIPMOCVLI' => 'X',
                'VRSALDOCONS' => $VRSALDOCONS
            );
            $this->entityManager->getConnection()->executeQuery("CANCELA_MOVICLIE", $params);
        }

        $result = array(
            'error' => false
        );
        try {
            if (!empty($venda[0]['NRCPFFID4ALL'])){ // Cancela 4ALL.
                $DTABERTUR = new \DateTime($venda[0]['DTABERTUR']);
                $this->extratocons->cancelaFidelidade($CDFILIAL.$CDCAIXA.$DTABERTUR->format('d/m/Y').$NRSEQVENDA, '4ALL');
            }
        } catch (\Exception $e){
            throw new \Exception('Não foi possível cancelar venda. Ocorreu um erro no cancelamento no programa de fidelidade:<br><br>' . $e->getMessage());
        }
        try {
			$dadosFilial = $this->entityManager->getConnection()->fetchAssoc("BUSCA_DADOS_FILIAL", array('CDFILIAL' => $CDFILIAL));
            if ($dadosFilial['IDUTILFIDUDS'] == 'S'){
				if (
					!empty($venda[0]['NRCPFFIDUDS'])||
					!empty($venda[0]['NRINSCRCONS'])||
					$boExtratoCons
				){ // Cancela UDS.
	                $DTABERTUR = new \DateTime($venda[0]['DTABERTUR']);
	                $fidelityArray = self::arrayCancUDS(
	                    $CDFILIAL, $CDCAIXA, $DTABERTUR, $NRSEQVENDA
	                );
	                $resultFidelidade = $this->fidelidadeUDSAPI->cancelaFidelidadeUDS(json_encode($fidelityArray));
	                $this->util->geraLog(' --Envia cancelamento fidelidade UDS', $CDFILIAL, $CDCAIXA, $venda[0]['DTVENDA']);
	                $result['IDENVIOFIDUDS']='N';
	                if(isset($resultFidelidade['success'])){
	                    if($resultFidelidade['success']){
	                        $result['IDENVIOFIDUDS']='C';
	                        $params = array(
	                            'IDENVIOFIDUDS' => 'C',
	                            'NRORG' => $NRORG,
	                            'NRSEQVENDA' => $NRSEQVENDA,
	                            'CDFILIAL' => $CDFILIAL,
	                            'CDCAIXA' => $CDCAIXA
	                        );
	                        $this->entityManager->getConnection()->executeQuery("UPDATE_VENDA_IDENVIOFIDUDS", $params);
	                    }
	                }
	            }
			}
        } catch (\Exception $e) {
            //throw new \Exception('Não foi possível cancelar venda. Ocorreu um erro no cancelamento no programa de fidelidade:<br><br>' . $e->getMessage());
        }
        return $result;
    }

    private function buscaDadosCaixa($CDFILIAL, $CDCAIXA, $NRORG) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRORG' => $NRORG
        );
        return $this->entityManager->getConnection()->fetchAssoc("GET_DADOS_CAIXA", $params);
    }

    private function trunc($val, $f = "2") {
        if (($p = strpos($val, '.')) !== false) {
            $val = floatval(substr($val, 0, $p + 1 + $f));
        }
        return $val;
    }

    public function calculaImpostoParaImpressao($NRORG, $CDFILIAL, $CDCAIXA, $ITEMVENDA, $TOTALVENDABRT, $VRDESCVENDA, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR, $VRACRCOMANDA) {
        $params = array(
            'NRORG' => $NRORG,
            'CDFILIAL' => $CDFILIAL
        );
        $IDCUMUPISCOFIL = $this->entityManager->getConnection()->fetchAssoc("GET_IDCUMUPISCOFIL", $params);
        $IDCUMUPISCOFIL = $IDCUMUPISCOFIL['IDCUMUPISCOFIL'];

        $dadosCaixa = $this->buscaDadosCaixa($CDFILIAL, $CDCAIXA, $NRORG);

        $ICMSTotal = 0;
        $pis = 0;
        $cofins = 0;
        $totalFederal = 0;
        $totalEstadual = 0;

        $itensImposto = self::separaItensQueSeraoCobrados($ITEMVENDA);
        foreach ($itensImposto as $itemAtual) {
            $porcDescItem = 0;
            $porcAcreItem = 0;

            $TOTALITEM = self::trunc((($itemAtual['VRUNITVEND'] + $itemAtual['VRUNITVENDCL']) * $itemAtual['QTPRODVEND']) -
                ($dadosCaixa['IDHABCAIXAVENDA'] == 'TAA' ? ($itemAtual['VRDESITVEND'] + $itemAtual['VRACRITVEND']) : 0));
            // no FOS, porcDescItem e porcAcreItem apresentam valores sempre zerados (verificar posteriormente)
            // if ($TOTALVENDABRT > 0) {
            //     $porcDescItem = self::trunc(($TOTALITEM * $VRDESCVENDA) / $TOTALVENDABRT);
            //     $porcAcreItem = self::trunc(($TOTALITEM * $VRACRCOMANDA) / $TOTALVENDABRT);
            // }
            $params = array(
                'CDPRODUTO' => $itemAtual['CDPRODUTO'],
                'NRORG' => $NRORG,
                'CDFILIAL' => $CDFILIAL
            );
            $impostoProduto = $this->entityManager->getConnection()->fetchAssoc("GET_IMPOSTOS_PRODUTO", $params);
            // Real(Nao Cumulativo) = 'S' e Presumido(Cumulativo) = 'N'
            if (!empty($impostoProduto) && $impostoProduto['IDINCIDEPISCOF'] === 'N'){
                $VALORPIS = 0.65;
                $VALORCOFINS = 3;
            } else {
                $VALORPIS = 1.65;
                $VALORCOFINS = 7.60;
            }

            $ICMSitem = (($TOTALITEM - $porcDescItem + $porcAcreItem) * ((isset($impostoProduto['VRPEALIMPFIS']) ? $impostoProduto['VRPEALIMPFIS']/ 100 : 0)));
            $ICMSTotal = $ICMSTotal + $ICMSitem;

            if (empty($impostoProduto) || ($impostoProduto['VRALIQIBPT'] == 0) || (empty($impostoProduto['VRALIQIBPT']))){
                $pis = (($TOTALITEM - $porcDescItem + $porcAcreItem) * ($VALORPIS / 100));
                $cofins = (($TOTALITEM - $porcDescItem + $porcAcreItem) * ($VALORCOFINS / 100));
                $totalFederal = $totalFederal + $ICMSitem + $pis + $cofins;
            } else {
                $IBPT = (($TOTALITEM - $porcDescItem + $porcAcreItem) * ($impostoProduto['VRALIQIBPT']) / 100);
                $totalFederal = $totalFederal + $IBPT;
            }

            if (empty($impostoProduto) || ($impostoProduto['VRALIQIBPTES'] == 0) || (empty($impostoProduto['VRALIQIBPTES']))) {
                $IBPTES = 0;
            } else {
                $IBPTES = (($TOTALITEM - $porcDescItem + $porcAcreItem) * ($impostoProduto['VRALIQIBPTES']) / 100);
            }
            $totalEstadual = $totalEstadual + $IBPTES;
        }

        return array(
            'VRTOTTRIBIBPT' => self::trunc($totalFederal + $totalEstadual),
            'impostoEstadual' => self::trunc($ICMSTotal)
        );
    }

    private function separaItensQueSeraoCobrados($itens) {
        $itensImposto = array();
        foreach ($itens as $itemAtual) {
            if ($itemAtual['IDSITUITEM'] == self::ITEM_APROVADO) {
                // se for promoção e será cobrado os itens filhos
                if (($itemAtual['IDTIPOCOMPPROD'] == '3' || $itemAtual['IDTIPOCOMPPROD'] == 'C') && ($itemAtual['IDIMPPRODUTO'] == '2')) {
                    foreach ($itemAtual['itensCombo'] as $itemComboAtual) {
                        array_push($itensImposto, $itemComboAtual);
                    }
                } else {
                    array_push($itensImposto, $itemAtual);
                }
            }
        }
        return $itensImposto;
    }

    private function separaItensCancelados($itens) {
        $itensImposto = array();
        foreach ($itens as $itemAtual) {
            if ($itemAtual['IDSITUITEM'] == self::ITEM_CANCELADO) {
                // se for promoção e será cobrado os itens filhos
                if (($itemAtual['IDTIPOCOMPPROD'] == '3') && ($itemAtual['IDIMPPRODUTO'] == '2')) {
                    foreach ($itemAtual['itensCombo'] as $itemComboAtual) {
                        array_push($itensImposto, $itemComboAtual);
                    }
                } else {
                    array_push($itensImposto, $itemAtual);
                }
            }
        }
        return $itensImposto;
    }

    private function realizaVenda(
        $NRORG, $CDFILIAL, $CDLOJA, $CDCAIXA,
        $CDVENDEDOR, $CDOPERADOR, $DTABERCAIX, $DTVENDA,
        $IDTPEMISVEND, $TOTALVENDABRT, $ITEMVENDA, $TIPORECE,
        $NMCONSVEND, $NRINSCRCONS, $DSENDECONSVENDA, $CDSENHAPED, $NRSEQVENDA,
        $VRTROCOVEND, $EMAIL, $VRDESCVENDA, $VRTOTTRIBIBPT,
        $CDCLIENTE, $CDCONSUMIDOR, $NRVENDAREST, $NRCOMANDA,
        $NRMESA, $IDORIGEMVENDA, $VRTXSEVENDA, $NRPESMESAVENDA,
        $DSCOMANDA, $FAMSALDOPROD, $VRTOTVENDA, $DTHRMESAFECHVEN,
        $fidelityDetails, $FIDELITYDISCOUNT, $cashbackUsed4ALL, $FIDELITYVALUE, $CODPROMO,
        $motivoDesconto, $CDGRPOCORDESC, $SUBSIDY, $DSOBSFINVEN, $ITVENDADES = null, $DELIVERY = false, $REPIQUE = 0,
        $NRCPFFIDUDS, $VRDESCCASHUDS,
        $arrPromocoesIntUtilizadas, $CDSUPERVISOR) {

        $IDIMPVENDA = 'I'; // I (incluida) - A (alterada) - S (esportada com S)
        $IDSITUVENDA = self::APROVADA;
        $fidelityArray = null;

        // PATH DO LOG ATUAL
        if ($this->habilitaLog){
            $paths = ['LOG/', 'vendaApi/', 'Filial - ' . $CDFILIAL . '/', 'Caixa - ' . $CDCAIXA . '/', 'LOGS/'];
            $logVendaPath = dirname(__DIR__) . '/../../../../../../';

            foreach ($paths as $path){
                $logVendaPath = $logVendaPath . $path;

                if (!is_dir($logVendaPath)){
                    mkdir($logVendaPath, 0700);
                }
            }

            $horaAtual = new \DateTime();
            $this->logVendaPath = $logVendaPath . 'VENDAS ' . $horaAtual->format('d-m-Y') . '.txt';
        }
        $this->insereLinhaLogVenda('Inicio da Venda');

        // Dados consumidor
        $NMCONSVEND = isset($NMCONSVEND) ? $NMCONSVEND : null;
        $NRINSCRCONS = isset($NRINSCRCONS) ? $NRINSCRCONS : null;
        $DSENDECONSVENDA = isset($DSENDECONSVENDA) ? $DSENDECONSVENDA : null;
        $CDCLIENTE = isset($CDCLIENTE) ? $CDCLIENTE : null;
        $CDCONSUMIDOR = isset($CDCONSUMIDOR) ? $CDCONSUMIDOR : null;
        $NRCPFFID4ALL = null;
        if (!empty($fidelityDetails) && $motivoDesconto !== 'CRM'){
            $NRCPFFID4ALL = $fidelityDetails['userInfo']['cpf'];
        }

        $connection = $this->entityManager->getConnection();

        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA,
            'NRORG' => $NRORG
        );
        $dadosEmpresa = $this->entityManager->getConnection()->fetchAll("GET_EMPRESAFILIAL", $params);
        $CDEMPRESA = $dadosEmpresa[0]['CDEMPRESA'];
        $NRINSJURFILI = $dadosEmpresa[0]['NRINSJURFILI'];

        $dadosFilial = $this->entityManager->getConnection()->fetchAll("GET_ENDEFILI_FULL", $params);
        $SGESTADO = $dadosFilial[0]['SGESTADO'];

        $dadosPadrao = $this->buscaClientePadrao($CDFILIAL, $NRORG);
        if (empty($CDCLIENTE)){
           $CDCLIENTE = $dadosPadrao['CDCLIENTE'];
        }

        if (empty($CDCONSUMIDOR)){
            $CDCCUSCLIE = $dadosPadrao['CDCCUSCLIE'];
        }
        else {
            $params = array(
                'CDCLIENTE' => $CDCLIENTE,
                'CDCONSUMIDOR' => $CDCONSUMIDOR
            );
            $dadosConsumidor = $this->entityManager->getConnection()->fetchAssoc("TIPO_CONSUMIDOR", $params);
            $CDCCUSCLIE = $dadosConsumidor['CDCCUSCLIE'];
        }

        //--- Generates a sale code
        if (empty($NRSEQVENDA)) {
            $CDCONTADOR = 'PEDIDOVENDA' . $CDFILIAL . $CDCAIXA;
            $NRSEQVENDA = $this->util->geraCodigo($connection, $CDCONTADOR, $NRORG, 1, 10);
        }
        $this->insereLinhaLogVenda('Sequencial da tabela VENDA: ' . $NRSEQVENDA);
        $this->insereLinhaLogVenda('-----------------------------------------------------');


		$IDPEDINTEPROMO = null;
		$NRCPFINTEPROMO = null;
		$dadosFilial2 = $this->entityManager->getConnection()->fetchAssoc("BUSCA_DADOS_FILIAL", array('CDFILIAL' => $CDFILIAL));
		if ($dadosFilial2['IDUTILINTEPROMO'] == 'S'){
			$params = array(
				'CDFILIAL'=>$CDFILIAL,
				'NRCOMANDA'=>$NRCOMANDA,
				'NRVENDAREST'=>$NRVENDAREST
			);
			$dadosPROMOCOMANDA = $this->entityManager->getConnection()->fetchAssoc("GET_PROMOCOMANDA_TUDO_API", $params);
			if( isset($dadosPROMOCOMANDA['IDPEDINTEPROMO']) ? $dadosPROMOCOMANDA['IDPEDINTEPROMO'] : null){
				$IDPEDINTEPROMO = $dadosPROMOCOMANDA['IDPEDINTEPROMO'];
				$NRCPFINTEPROMO = $dadosPROMOCOMANDA['NRCPFINTEPROMO'];
			} else {
				if($arrPromocoesIntUtilizadas && empty($NRCOMANDA) && empty($NRVENDAREST)){
					if(is_array($arrPromocoesIntUtilizadas)){
						if(!empty($arrPromocoesIntUtilizadas)){
							if(isset($arrPromocoesIntUtilizadas['IDPEDINTEPROMO'])){
								$IDPEDINTEPROMO = $arrPromocoesIntUtilizadas['IDPEDINTEPROMO'];
								$NRCPFINTEPROMO = $arrPromocoesIntUtilizadas['NRCPFINTEPROMO'];
							} else if(isset($arrPromocoesIntUtilizadas[0]['IDPEDINTEPROMO'])){
								$IDPEDINTEPROMO = $arrPromocoesIntUtilizadas[0]['IDPEDINTEPROMO'];
								$NRCPFINTEPROMO = $arrPromocoesIntUtilizadas[0]['NRCPFINTEPROMO'];
							} else if(isset($arrPromocoesIntUtilizadas[0][0]['IDPEDINTEPROMO'])){
								$IDPEDINTEPROMO = $arrPromocoesIntUtilizadas[0][0]['IDPEDINTEPROMO'];
								$NRCPFINTEPROMO = $arrPromocoesIntUtilizadas[0][0]['NRCPFINTEPROMO'];
							}
						}
					}
				}
			}
		}
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA,
            'NRORG' => $NRORG
        );
        if (!empty($CDGRPOCORDESC)){
            $CDOCORDESC = $CDGRPOCORDESC;
            $CDGRPOCORDESC = $this->entityManager->getConnection()->fetchAssoc("SQL_GRUPO_OBS_DESC", $params);
            $CDGRPOCORDESC = !empty($CDGRPOCORDESC['CDGRPOCORDESC']) ? $CDGRPOCORDESC['CDGRPOCORDESC'] : null;
            if($CDGRPOCORDESC != null){
                $params = array(
                    'CDGRPOCORDESC' => $CDGRPOCORDESC,
                    'CDOCORDESC' => $CDOCORDESC
                );
                $DSOCORRDESC = $this->entityManager->getConnection()->fetchAssoc("GET_MOT_DESC", $params);
                if(empty($motivoDesconto)){
                    $motivoDesconto = $DSOCORRDESC["DSOCORR"] . ";";
                } else {
                    $motivoDesconto .= "; " . $DSOCORRDESC["DSOCORR"] . ";";
                }
            }
        }

        if(!empty($EMAIL)) {
            $DSEMAILENVCUPOM = $EMAIL;
            $IDSITENVCUPOM = 'A';
        } else {
            $DSEMAILENVCUPOM = null;
            $IDSITENVCUPOM = 'U';
        }

        $parametrosVendaDlv =  $this->montaParametrosVendaDlv($CDFILIAL, $NRVENDAREST, $NRCOMANDA);

        //--- Build the sale params
        $parametrosDaVenda = $this->montaParametrosDaVenda(
            $CDFILIAL, $CDLOJA, $CDCAIXA, $CDVENDEDOR,
            $DTVENDA, $CDEMPRESA, $SGESTADO, $IDIMPVENDA,
            $DTABERCAIX, $VRDESCVENDA, $CDCLIENTE, $CDCCUSCLIE,
            $IDSITUVENDA, $NRSEQVENDA, $NRORG, $IDTPEMISVEND,
            $CDOPERADOR, $NRINSJURFILI, $CDCONSUMIDOR, $NMCONSVEND,
            $NRINSCRCONS, $DSENDECONSVENDA, $IDORIGEMVENDA, $CDSENHAPED,
            $NRVENDAREST, $NRCOMANDA, $NRMESA, $VRTXSEVENDA,
            $NRPESMESAVENDA, $DSCOMANDA, $VRTOTVENDA, $DTHRMESAFECHVEN,
            $motivoDesconto, $CDGRPOCORDESC, $DSOBSFINVEN,
            $parametrosVendaDlv, $REPIQUE, $NRCPFFID4ALL, $CODPROMO,
            $FIDELITYDISCOUNT, $cashbackUsed4ALL, $NRCPFFIDUDS, $VRDESCCASHUDS,
			$IDPEDINTEPROMO, $NRCPFINTEPROMO, $DSEMAILENVCUPOM, $IDSITENVCUPOM, $CDSUPERVISOR
        );

        //--- Persists the sale
        $this->insereLinhaLogVenda('Gravando na tabela VENDA');
        $this->geraLogVenda($parametrosDaVenda[0]);
        $this->entityManager->getConnection()->executeQuery("INSERT_VENDA", $parametrosDaVenda[0], $parametrosDaVenda[1]);
        $this->insereLinhaLogVenda('Fim da gravação na VENDA');
        $this->insereLinhaLogVenda('-----------------------------------------------------');

        //--- Saves the sale items
        $cdprodutoItems = array();

        foreach($ITEMVENDA as &$produto) {
            if ($produto['IDSITUITEM'] == self::ITEM_APROVADO) {
                $cdprodesto = null;
                $nrfeaturegrupo = null;
                if (isset($produto['DIMENSAO'])) {
                    $dimensoes = $produto['DIMENSAO'];
                    $nrfeaturegrupo = $this->getNrfeatureGrupoByDimensoes($dimensoes);
                    $cdprodesto = $this->getProdEstoByNrfeaturegrupo($nrfeaturegrupo);
                } else {
                    $cdprodesto = $produto['CDPRODUTO'];
                }
                $produto['NRFEATUREGRUPO'] = $nrfeaturegrupo;
                $produto['CDPRODESTO'] = $cdprodesto;
                $cdprodutoItems[] = $cdprodesto;
            }
        }

        $NRLANCESTQ = null;
        $PARAVEND = $this->buscaPARAVEND($CDFILIAL, $NRORG);
        $IDESTTEMPOREAL = $PARAVEND['IDESTTEMPOREAL'];
        if ($IDESTTEMPOREAL == self::SINCRONIZA_ESTOQUE) {
            $paramsEstqFili = $this->estoque->getParamsEstqFili($CDFILIAL, $NRORG);
            if ($paramsEstqFili['IDCTRLESTQ']) {
                $prodsCntrEstq = $this->estoque->prodsVendCntrlEst($NRORG, $CDFILIAL, $cdprodutoItems);
                if ($prodsCntrEstq) {
                    $dslancestq = 'LANCTO. REF. À VENDA: ' . $NRSEQVENDA . ' DO CAIXA: ' . $CDCAIXA;
                    $idtplancto = 'S';
                    $NRLANCESTQ = $this->estoque->geraLancamentoEstq($CDFILIAL, $NRORG, $CDOPERADOR, $DTVENDA, $CDFILIAL, $dslancestq, $idtplancto);
                }
            }
        }

        $NRORDITCUPFIS = 0;
        $nrsequitvendProdutoPai = null;
        $CDPRODPROMO = null;
        foreach ($ITEMVENDA as $item) {
            $NRORDITCUPFIS = $NRORDITCUPFIS + 1;
            $NRORDITCUPFIS = str_pad($NRORDITCUPFIS, 5, '0', STR_PAD_LEFT);
            if ($item['IDSITUITEM'] == self::ITEM_APROVADO) {
                if(($CDPRODPROMO != null || $nrsequitvendProdutoPai != null) && $CDPRODPROMO != $item['CDPRODPROMOCAO']){
                    $nrsequitvendProdutoPai = null;
                    $CDPRODPROMO = null;
                }
                $this->insertSaleItem(
                    $CDFILIAL,
                    $CDLOJA,
                    $CDCAIXA,
                    $NRORG,
                    $NRSEQVENDA,
                    $item,
                    $NRORDITCUPFIS,
                    $nrfeaturegrupo,
                    $CDOPERADOR,
                    $IDTPEMISVEND,
                    $VRDESCVENDA,
                    $TOTALVENDABRT,
                    $item['CDPRODPROMOCAO'],
                    null,
                    $item['CDVENDEDOR'],
                    $VRTOTTRIBIBPT,
                    $CDCLIENTE,
                    $CDCONSUMIDOR,
                    $IDESTTEMPOREAL,
                    $IDORIGEMVENDA,
                    $nrsequitvendProdutoPai
                );
                $itensCombo = array_key_exists('itensCombo', $item) ? $item['itensCombo'] : array();
                $tipoProduto = $this->defineTipoProduto($item['CDPRODPROMOCAO'], $item['IDTIPOCOMPPROD'], $item['IDIMPPRODUTO'], $itensCombo);
                if($tipoProduto == 'filho'){
                    $CDPRODPROMO = $item['CDPRODPROMOCAO'];
                }

            } else {
                $this->insertSaleItemCanc($CDFILIAL, $CDLOJA, $CDCAIXA, $NRORG, $NRSEQVENDA, $item, $NRORDITCUPFIS);
            }
        }

        if (!empty($ITVENDADES)){
            $this->insertItemsDesistencia($ITVENDADES, $NRSEQVENDA, $CDFILIAL, $CDLOJA, $CDCAIXA, $NRORG, $CDOPERADOR, $connection);
        }

        //----- atualiza estoque ------
        if ($IDESTTEMPOREAL == self::SINCRONIZA_ESTOQUE) {
            if ($NRLANCESTQ !== null) {
                $dtini = date_create_from_format('d/m/Y H:i:s', $DTVENDA);
                $dtini = $dtini->format('d/m/Y');
                $dtfin = date_create_from_format('d/m/Y H:i:s', $DTVENDA);
                $dtfin = $dtfin->format('d/m/Y');
                $this->estoque->atualizaEstoque($CDFILIAL, $CDFILIAL, $NRLANCESTQ, $dtini, $dtfin);
            }
        }

        $dadosCaixa = $this->buscaDadosCaixa($CDFILIAL, $CDCAIXA, $NRORG);
        $EXTRATOCONSPARAMS = array();
		$boUsaCreditoPesoal = false;
        foreach ($TIPORECE as $pagamentoAtual) {
            if (!empty($pagamentoAtual['NRSEQMOV'])) {
                $this->updateIntegracaomov($pagamentoAtual['NRSEQMOV'], $NRSEQVENDA, $CDFILIAL, $CDCAIXA, $NRORG);
            }
            // para recebimentos por transação, é considerado CDTIPORECE de acordo com a bandeira retornada na integração
            if (isset($pagamentoAtual['CDBANCARTCR'])){
                $NRCONFTELA = $this->util->getConfTela($CDFILIAL, $CDCAIXA);
                $pagamentoAtual['CDTIPORECE'] = $this->alteraTiporecePorBandeira($pagamentoAtual, $NRCONFTELA['CDFILIAL'], $NRCONFTELA['NRCONFTELA'], $NRCONFTELA['DTINIVIGENCIA'], $NRORG);
            }
            if($DELIVERY){
                $pagamentoAtual = $this->getPagamentoDlv($CDFILIAL, $NRVENDAREST, $pagamentoAtual);
            }
            $nrseqmovi = $this->insertPayment($pagamentoAtual, $CDFILIAL, $CDCAIXA, $DTABERCAIX, $DTVENDA, $NRSEQVENDA, $NRORG, $CDOPERADOR, $CDCLIENTE);
            $idtipoRece = $this->getIdtiporece($pagamentoAtual, $NRORG);

            $vrmovcli = $pagamentoAtual['VRMOVIVEND'];
            if ($idtipoRece == 'A'){ // Venda com débito consumidor.
				$boUsaCreditoPesoal = true;
                $vrmovcli = floatval(bcsub(str_replace(',','.',strval($vrmovcli)), str_replace(',','.',strval($SUBSIDY)), '2'));
                $cdcontadorMovclie = 'MOVCLIE' . $CDCLIENTE;
                $this->util->geraCodigo($connection, $cdcontadorMovclie, $NRORG, 1, 10);
                $nrseqmovclie      = $this->util->geraCodigo($connection, $cdcontadorMovclie, $NRORG, 1, 10);
                $cdtiporece        = $pagamentoAtual['CDTIPORECE'];
                $vrsaldocons       = $this->consumidorService->getSaldoConsumidor($CDCLIENTE, $CDCONSUMIDOR);
                $novoSaldoCons = $vrsaldocons + ($vrmovcli * (-1));
                $vrmovpescons = 0;
                $vrsaldpecons = 0;
                $vrpesubsidio = 0;
                $vrtetosubsid = 0;

                $IDSALDEXTDEBCON = $PARAVEND['IDSALDEXTDEBCON'];

                if($IDSALDEXTDEBCON == 'S'){
                    $totalProdBasiExtra = $this->consumidorService->getTotProdBasicoExtra($ITEMVENDA, $CDFILIAL, $NRORG, $CDCLIENTE, $CDCONSUMIDOR);

                    $VALOR_BASICO = $totalProdBasiExtra['VALOR_BASICO'];
                    $VALOR_EXTRA = $totalProdBasiExtra['VALOR_EXTRA'];
                
                    $this->insertMovclie(
                        $CDFILIAL,
                        $CDCAIXA,
                        $DTABERCAIX,
                        $nrseqmovclie,
                        $CDCLIENTE,
                        $CDCONSUMIDOR,
                        $VALOR_EXTRA,
                        $DTVENDA,
                        $NRSEQVENDA,
                        $nrseqmovi,
                        $cdtiporece,
                        $novoSaldoCons,
                        $CDOPERADOR,
                        $NRORG,
                        $vrmovpescons,
                        $vrsaldpecons,
                        $vrpesubsidio,
                        $vrtetosubsid
                    );
    
                    if ($VALOR_BASICO > 0){
                        $nrseqmovclie = $this->util->geraCodigo($connection, $cdcontadorMovclie, $NRORG, 1, 10);
                        $this->insertMovclie(
                            $CDFILIAL,
                            $CDCAIXA,
                            $DTABERCAIX,
                            $nrseqmovclie,
                            $CDCLIENTE,
                            null,
                            $VALOR_BASICO,
                            $DTVENDA,
                            $NRSEQVENDA,
                            $nrseqmovi,
                            $cdtiporece,
                            $VALOR_BASICO,
                            $CDOPERADOR,
                            $NRORG,
                            $vrmovpescons,
                            $vrsaldpecons,
                            $vrpesubsidio,
                            $vrtetosubsid
                        );
                    }
                } else {
                    $this->insertMovclie(
                        $CDFILIAL,
                        $CDCAIXA,
                        $DTABERCAIX,
                        $nrseqmovclie,
                        $CDCLIENTE,
                        $CDCONSUMIDOR,
                        $vrmovcli,
                        $DTVENDA,
                        $NRSEQVENDA,
                        $nrseqmovi,
                        $cdtiporece,
                        $novoSaldoCons,
                        $CDOPERADOR,
                        $NRORG,
                        $vrmovpescons,
                        $vrsaldpecons,
                        $vrpesubsidio,
                        $vrtetosubsid
                    );

                    if ($SUBSIDY > 0){
                        $nrseqmovclie = $this->util->geraCodigo($connection, $cdcontadorMovclie, $NRORG, 1, 10);
                        $this->insertMovclie(
                            $CDFILIAL,
                            $CDCAIXA,
                            $DTABERCAIX,
                            $nrseqmovclie,
                            $CDCLIENTE,
                            null,
                            $SUBSIDY,
                            $DTVENDA,
                            $NRSEQVENDA,
                            $nrseqmovi,
                            $cdtiporece,
                            $SUBSIDY,
                            $CDOPERADOR,
                            $NRORG,
                            $vrmovpescons,
                            $vrsaldpecons,
                            $vrpesubsidio,
                            $vrtetosubsid
                        );
                    }
                }

                $this->consumidorService->atualizaSaldoMoviClie($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDOPERADOR);
            }
            else if ($idtipoRece == '9'){ // Venda com crédito pessoal.
            	$boUsaCreditoPesoal = true;
                $DSOPEEXTCONS = $dadosCaixa['IDHABCAIXAVENDA'] != 'TAA' ? 'WTR - Venda Caixa ' . $CDCAIXA : 'TAA - Venda Caixa ' . $CDCAIXA;
                $NRSEQMOVEXT = 1;

                $dadosFilial = $this->entityManager->getConnection()->fetchAssoc("BUSCA_DADOS_FILIAL", array('CDFILIAL' => $CDFILIAL));
                foreach ($FAMSALDOPROD as $key => $family){
                    $this->updateItemVendaFamilia($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $key);
                    $currentBalance = $this->consumidorService->buscaSaldoExtrato($CDCLIENTE, $CDCONSUMIDOR, $key, $CDFILIAL);
                    $VRSALDCONEXT = 0.000;
                    if (!empty($currentBalance)){
                        $VRSALDCONEXT = round(floatval($currentBalance['VRSALDCONEXT']) - $vrmovcli, 2);
                    }

                    $dateReference = new \DateTime();
                    $this->insertExtratoCons($CDCLIENTE, $CDCONSUMIDOR, $key, $DSOPEEXTCONS,
                                             $vrmovcli, 'V', $VRSALDCONEXT, str_pad($NRSEQMOVEXT, 3, '0', STR_PAD_LEFT),
                                             $pagamentoAtual['CDTIPORECE'], $CDFILIAL, $CDCAIXA,
                                             $NRSEQVENDA, $DTABERCAIX, null, $dateReference);

                    if ($dadosFilial['IDEXTCONSONLINE'] === 'S'){
                        $params = array(
                            'CDCLIENTE' => $CDCLIENTE,
                            'CDCONSUMIDOR' => $CDCONSUMIDOR,
                            'CDFAMILISALD' => $key,
                            'DTMOVEXTCONS' => $dateReference,
                            'NRSEQMOVEXT' => str_pad($NRSEQMOVEXT, 3, '0', STR_PAD_LEFT),
                            'CDTIPORECE' => $pagamentoAtual['CDTIPORECE'],
                            'IDTPMOVEXT' => 'V',
                            'DSOPEEXTCONS' => $DSOPEEXTCONS,
                            'VRMOVEXTCONS' => $vrmovcli,
                            'VRSALDCONEXT' => $VRSALDCONEXT,
                            'CDFILIAL' => $CDFILIAL,
                            'CDCAIXA' => $CDCAIXA,
                            'NRDEPOSICONS' => null,
                            'NRSEQMOVCAIXA' => null,
                            'IDIMPEXTRATO' => 'S'
                        );
                        $params['DTMOVEXTCONS'] = $params['DTMOVEXTCONS']->format('d/m/Y H:i:s');
                        array_push($EXTRATOCONSPARAMS, $params);
                    }
                    $this->updateBalance($CDCLIENTE, $CDCONSUMIDOR, $key, $VRSALDCONEXT);

                    $NRSEQMOVEXT++;
                }
            } else if ($idtipoRece == 'K'){ // Venda Cartao Todo
                $this->insereLinhaLogVenda('Venda utilizando Cartao Todo');
                if(isset($pagamentoAtual['cartaoTodo'])){
                    $pagamentoAtual['cartaoTodo']['card_number'] = isset($pagamentoAtual['cartaoTodo']['card_number'])?$pagamentoAtual['cartaoTodo']['card_number']:"";
                    $pagamentoAtual['cartaoTodo']['card_password'] = isset($pagamentoAtual['cartaoTodo']['card_password'])?$pagamentoAtual['cartaoTodo']['card_password']:"";
                    $pagamentoAtual['cartaoTodo']['saldoutilizado'] = isset($pagamentoAtual['cartaoTodo']['saldoutilizado'])?$pagamentoAtual['cartaoTodo']['saldoutilizado']:null;

                    if(!empty($pagamentoAtual['cartaoTodo']['saldoutilizado'])){
                        $valor = $pagamentoAtual['cartaoTodo']['saldoutilizado'];
                        $valor = str_replace(',', '.', $valor);
                        $valor = floatval($valor);
                    }
                    $params = array(
                        'store_cnpj' => $NRINSJURFILI,
                        'card_number' => $pagamentoAtual['cartaoTodo']['card_number'],
                        'card_password' => $pagamentoAtual['cartaoTodo']['card_password'],
                        'total' => $valor
                    );
                    $result = $this->cartaoTodo->chamadaAPICartaoTodo('useCard',$params);
                    if(!$result){
                        throw new \Exception("Erro: Não houve retorno da chamada da API do Cartao Todo.");
                    }
                    if($result['error']){
                        if(isset($result['message'])){
                            if(!empty($result['message'])){
                                throw new \Exception($result['message']);
                            }
                        }
                        throw new \Exception("Erro: Um erro inesperado aconteceu na chamada da API Cartao Todo.");
                    }
                    $this->insereLinhaLogVenda('Chamada a api Cartao Todo não retornou erros.');
                    $this->insereLinhaLogVenda('FIM chamada a api Cartao Todo.');
                } else {
                    throw new \Exception("Erro: Os dados do Cartão Todo não foram informados.");
                }
            }
        }
        // Trata troco, caso exista.
        if (!empty($VRTROCOVEND['VRMOVIVEND'])){
            $this->insertChangePayment($VRTROCOVEND, $CDFILIAL, $CDCAIXA, $DTABERCAIX, $DTVENDA, $NRSEQVENDA, $NRORG, $CDOPERADOR, $CDCLIENTE);
        }

        // TRATA TRANSAÇÕES DE ADIANTAMENTO.
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'NRVENDAREST' => $NRVENDAREST,
            'NRCOMANDA' => $NRCOMANDA
        );
        $statusAdiantamento = $this->entityManager->getConnection()->fetchAssoc("GET_ADVANCE_COMANDAVEN", $params);

        // Verifica se a mesa teve adiantamento.
        if (!empty($statusAdiantamento) && $statusAdiantamento['IDADIANTAMENTO'] == 'S'){
            // Busca adiantamentos.
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'NRVENDAREST' => $NRVENDAREST,
                'NRCOMANDA' => $NRCOMANDA
            );
            $adiantamentos = $this->entityManager->getConnection()->fetchAll("GET_ADVANCED_PAYMENTS", $params);

            // Insere adiantamentos na MOVCAIXA.
            foreach($adiantamentos as $adiantamento){
                if ($adiantamento['IDADIANTAMENTO'] == 'S'){
                    // Define valores.
                    $adiantamento['VRMOVIVEND'] = $adiantamento['VRMOV'];
                    $adiantamento['NRPARCELAS'] = 1;
                    $adiantamento['CDNSUHOSTTEF'] = $adiantamento['CDNSUTEFMOB'];
                    $adiantamento['CDNSUTEF'] = !empty($adiantamento['CDNSUTEFINTMOB']) ? $adiantamento['CDNSUTEFINTMOB'] : $adiantamento['CDNSUTEFMOB'];
                    $adiantamento['NRCONTROLTEF'] = null;
                    // Insere na MOVCAIXA.
                    $nrseqmovi = $this->insertPayment($adiantamento, $CDFILIAL, $CDCAIXA, $DTABERCAIX, $DTVENDA, $NRSEQVENDA, $NRORG, $CDOPERADOR, $CDCLIENTE);
                }
            }
        }
        $utilizaUDS = $this->entityManager->getConnection()->fetchAssoc("BUSCA_DADOS_FILIAL", array('CDFILIAL' => $CDFILIAL));
        if ($utilizaUDS['IDUTILFIDUDS'] == 'S'){
            $paramsToConsumer = array(
                ':CDFILIAL'   => $CDFILIAL,
                ':CDCAIXA'    => $CDCAIXA,
                ':NRSEQVENDA' => $NRSEQVENDA
            );
            $infoConsumer = $this->entityManager->getConnection()->fetchAssoc("GET_CONSUMER_SALE", $paramsToConsumer);
            if( !empty($infoConsumer) ){
                if( !empty($infoConsumer['NRINSCRCONS']) ){
                    $infoConsumer = $infoConsumer['NRINSCRCONS'];
                } else {
                    $infoConsumer = "";
                }
            }
            // TRANSMISSÃO DE DADOS FIDELIDADE UDS
            $fidelityArray = null;
            //vendas com CPF/CNPJ no cashback ou nota fiscal, e vendas crédito pessoal
            $boEnviaVendaUDS = false;
            if ( !empty($NRCPFFIDUDS) || !empty($FIDELITYDISCOUNT) || $boUsaCreditoPesoal || !empty($infoConsumer) ){
                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDCAIXA' => $CDCAIXA,
                    'NRORG' => $NRORG,
                    'NRSEQVENDA' => $NRSEQVENDA
                );
                $venda = $this->entityManager->getConnection()->fetchAll("BUSCA_VENDA", $params);
                $venda=$venda[0];
                $vendaUDS = $this->entityManager->getConnection()->fetchAll("BUSCA_VENDA_UDS", $params);
                $vendaUDS=$vendaUDS[0];
                $itemvendaUDS = $this->entityManager->getConnection()->fetchAll("BUSCA_ITEM_VENDA_UDS", $params);
                $movcaixaUDS = $this->entityManager->getConnection()->fetchAll("BUSCA_MOVCAIXA_UDS", $params);
                $fidelityArray = self::arrayUDS(
                    $venda, $vendaUDS, $itemvendaUDS, $movcaixaUDS, $fidelityDetails
                );
            }
        }

        if (!empty($fidelityDetails) && $motivoDesconto !== 'CRM'){
            $dadosFilial = $this->entityManager->getConnection()->fetchAssoc("BUSCA_DADOS_FILIAL", array('CDFILIAL' => $CDFILIAL));
            if ($dadosFilial['IDUTILFID4ALL'] != 'S'){
                throw new \Exception('A filial não está configurada para acessar o sistema de crédito fidelidade.');
            }
            // Calcula o valor dos descontos sem cashback.
            if (empty($cashbackUsed4ALL)){
                // Se não tiver cashback, aplica o valor REAL do desconto do cupom, para ignorar o ajuste que faz o preço fiscal não ficar zerado).
                $otherDiscounts = floatval(bcadd(strval($VRDESCVENDA-$FIDELITYVALUE), strval($FIDELITYDISCOUNT), 2));
            }
            else {
                // Se tiver cashback, precisamos tirar o desconto total do fidelidade e aplicar somente o do cupom.
                $otherDiscounts = floatval(bcsub(strval($VRDESCVENDA), strval($FIDELITYVALUE), 2));
                $otherDiscounts = floatval(bcadd(strval($otherDiscounts), strval($FIDELITYDISCOUNT), 2));
            }

            // Se a busca de CPF não tiver dado certo, o CPF informado é CPF anônimo.
            $anonymousCPF = !$fidelityDetails['success'];

            // Verifica se o CPF faz parte de um funcionário do MADERO.
            $isFuncionarioMadero = $this->checkEmployeeCPF($fidelityDetails["userInfo"]["cpf"]);
            if ($anonymousCPF || $isFuncionarioMadero){
                // Se for funcionário do MADERO, zera o valor do cashback para não puntuar.
                $valueToCashback = 0;
            }
            else {
                $valueToCashback = $this->castInteger(floatval(bcsub(strval($TOTALVENDABRT), strval($otherDiscounts+$cashbackUsed4ALL), 2)));
            }

            $fidelityArray = self::array4ALL(
                $dadosFilial, $ITEMVENDA, $fidelityDetails,
                $CDFILIAL, $CDCAIXA, $DTABERCAIX, $NRSEQVENDA, $CODPROMO,
                $TOTALVENDABRT, $otherDiscounts, $cashbackUsed4ALL, $valueToCashback
            );
            
            $this->insereLinhaLogVenda(json_encode($fidelityArray));
        }

		$arrPROMOCOMANDA = null;
		if ($dadosFilial2['IDUTILINTEPROMO'] == 'S'){
			if($arrPromocoesIntUtilizadas){
				foreach ($arrPromocoesIntUtilizadas as $value) {
					$params = array (
						'CDFILIAL' => $CDFILIAL,
						'NRVENDAREST' => $NRVENDAREST,
						'NRCOMANDA' => $NRCOMANDA,
						'NRSEQPROMOCOM' => $value[0]['NRSEQPROMOCOM'],
						'IDUTPROMOCOM' => 'S'
					);
					$this->entityManager->getConnection()->fetchAssoc("UPDATE_STATUS_PROMOCOMANDA_API", $params);
				}
			}
			$params = array(
				'CDFILIAL'=>$CDFILIAL,
				'NRCOMANDA'=>$NRCOMANDA,
				'NRVENDAREST'=>$NRVENDAREST
			);
			$dadosPROMOCOMANDA = $this->entityManager->getConnection()->fetchAll("GET_PROMOCOMANDA_TUDO_API", $params);
			foreach ( $dadosPROMOCOMANDA as $promocao ){
				if($promocao['IDUTPROMOCOM']!='S'){
					continue;
				}
				if($arrPROMOCOMANDA == null){
					$arrPROMOCOMANDA = array();
				}
				array_push($arrPROMOCOMANDA, $promocao);
			}
			if($arrPromocoesIntUtilizadas){
				foreach ($arrPromocoesIntUtilizadas as $value){
					$value = $value[0];
					if($value['IDUTPROMOCOM']=='S'){
						$boEncontrou = false;
						if(is_array($arrPROMOCOMANDA)){
							foreach ( $dadosPROMOCOMANDA as $promocao ){
								if((
									$promocao['IDUTPROMOCOM'] 	== $value['IDUTPROMOCOM'] &&
									$promocao['IDPROMOCOM'] 	== $value['IDPROMOCOM'] &&
									$promocao['IDTPPROMOCOM'] 	== $value['IDTPPROMOCOM'] &&
									$promocao['IDTPDESCPROMO'] 	== $value['IDTPDESCPROMO'] &&
									$promocao['VRDESCPROMO']	== $value['VRDESCPROMO'] &&
									$promocao['VRVENDAPROMO']	== $value['VRVENDAPROMO']
								)){
									$boEncontrou = true;
								}
							}
						}
						if(!$boEncontrou){
							if($arrPROMOCOMANDA == null){
								$arrPROMOCOMANDA = array();
							}
							array_push($arrPROMOCOMANDA, $value);
						}
					}
				}
			}
			if($arrPROMOCOMANDA!=null){
				$contador = 1;
				foreach ( $arrPROMOCOMANDA as &$promocao ){
					$promocao['CDFILIAL'] = $CDFILIAL;
					$promocao['NRSEQVENDA'] = $NRSEQVENDA;
					$promocao['CDCAIXA'] = $CDCAIXA;
					$promocao['IDPROMOVEN'] = $promocao['IDPROMOCOM'];
					$promocao['NMPROMOVEN'] = $promocao['NMPROMOCOM'];
					$promocao['DSPROMOVEN'] = $promocao['DSPROMOCOM'];
					$promocao['IDTPPROMOVEN'] = $promocao['IDTPPROMOCOM'];
					$promocao['IDUTPROMOVEN'] = $promocao['IDUTPROMOCOM'];
					$promocao['NRSEQPROMOVEN'] = str_pad("".$contador, 3, '0', STR_PAD_LEFT);
					$contador = $contador + 1;
					$this->entityManager->getConnection()->fetchAssoc("INSERE_PROMOVENDA", $promocao);
					$this->util->geraLog(" -Insere promoção na PROMOVENDA. \r\n ---NMPROMOVEN: " . $promocao['NMPROMOVEN'] . ",  IDPROMOCOM: " . $promocao['IDPROMOCOM'], $CDFILIAL, $CDCAIXA, $DTVENDA);
				}
				$arrPROMOCOMANDA = self::arrayPromoInt($arrPROMOCOMANDA);
			} else {
				if(!empty($dadosPROMOCOMANDA)){
					$arrPROMOCOMANDA = self::arrayPromoInt(array(array(
						'IDPEDINTEPROMO'=>$dadosPROMOCOMANDA[0]['IDPEDINTEPROMO'],
						'IDTPPROMOCOM'=>''
					)));
				}
			}
		}
        $this->insereLinhaLogVenda('FIM DA GRAVAÇÃO DA VENDA');
        $this->insereLinhaLogVenda('-----------------------------------------------------');

        return array(
            'NRSEQVENDA' => $NRSEQVENDA,
            'EXTRATOCONSPARAMS' => $EXTRATOCONSPARAMS,
            '4ALLDATA' => null,
            'UDSDATA' => $fidelityArray,
			'PROMOINTDATA' => $arrPROMOCOMANDA
        );
    }

	public function arrayPromoInt($arrPROMOCOMANDA){
		$arrayRetorno = array();
		$arrayRetorno['offerApllied']=null;
		$arrayRetorno['payment'] = null;
		$arrayRetorno['wallet'] = false;
		$arrayRetorno['coupon'] = null;
		$OrderCloseDate = (new \DateTime())->format('Y-m-d')."T".(new \DateTime())->format('H:i:s');
		$arrayRetorno['orderCloseDate'] = $OrderCloseDate;
		$arrayRetorno['orderStatus'] = '1';
		if(isset($arrPROMOCOMANDA[0])){
			if(isset($arrPROMOCOMANDA[0]['IDPEDINTEPROMO'])){
				$arrayRetorno['cartId'] = $arrPROMOCOMANDA[0]['IDPEDINTEPROMO'];
			}
		}
		foreach ($arrPROMOCOMANDA as $promo) {
			if(!$promo){continue;}
			$arrayRetorno['cartId'] = $arrPROMOCOMANDA[0]['IDPEDINTEPROMO'];
			if($promo['IDTPPROMOCOM']=='O'){
				$arrayRetorno['offerApllied']['data'] = array();
				$arrayRetorno['offerApllied']['data']['id']=$promo['IDPROMOCOM'];
			} else if($promo['IDTPPROMOCOM']=='W'){
				$arrayRetorno['wallet'] = true;
			} else if($promo['IDTPPROMOCOM']=='P'){
				$arrayRetorno['payment']['data'] = array();
				$arrayRetorno['payment']['data']['id']=$promo['IDPROMOCOM'];
			}
		}
		return $arrayRetorno;
	}

    public function array4ALL(
        $dadosFilial, $ITEMVENDA, $fidelityDetails,
        $CDFILIAL, $CDCAIXA, $DTABERCAIX, $NRSEQVENDA, $CODPROMO,
        $TOTALVENDABRT, $otherDiscounts, $cashbackUsed4ALL, $valueToCashback
        ){
        $fidelityProducts = array();
        foreach ($ITEMVENDA as $product){
            if (!array_key_exists("TAXADESERVICO", $product)){
                $productArray = array(
                    "idUnityItem" => intval($product["CDPRODUTO"]),
                    "name" => $product["NMPRODUTO"],
                    "price" => 0,
                    "externalId" => "",
                    "amount" => $product["QTPRODVEND"],
                    "comboItems" => array()
                );
                array_push($fidelityProducts, $productArray);
            }
        }
        return array(
            "customer" => array(
                "cpf" => $fidelityDetails["userInfo"]["cpf"]
            ),
            "unityDetails" => array(
                "unityId" => $CDFILIAL, //Obrigatório - ID da loja
                "unityName" => $dadosFilial["NMRAZSOCFILI"] //Obrigatório - Nome da loja
            ),
            "promotion" => array(
                "usedPromocodeId" => !empty($CODPROMO['id']) ? $CODPROMO['id'] : ""
            ),
            "orderInfo" => array(
                "orderOrigin" => "APP", //Obrigatório - Origem da compra (PDV, APP, TOTEM, IFOOD, VIDEOSOFT, etc)
                "orderId" => $CDFILIAL.$CDCAIXA.$DTABERCAIX->format('d/m/Y').$NRSEQVENDA, //Obrigatório - Código unico do pedido
                "productTotal" => $this->castInteger($TOTALVENDABRT), //Obrigatório - Valor total de produto
                "deliveryFee" => 0, //Obrigatório - Valor total de frete (pode ser 0)
                "discount" => $this->castInteger($otherDiscounts), // Cashback não é incluído como desconto.
                "cashbackUsed" => $this->castInteger($cashbackUsed4ALL),
                "valueToCashback" => $valueToCashback
            ),
            "products" => $fidelityProducts
        );
    }

    public function arrayUDS( $venda, $vendaUDS, $itemvendaUDS, $movcaixaUDS, $fidelityDetails ){
        $arrPagamentos = array();
		$valorValePresente = 0;
        foreach ($movcaixaUDS as $movcaixa) {
            $arrPayment = array(
                "id"              => $movcaixa['CDTIPORECE'],
                "canalPagamento"  => self::getCanalPag($movcaixa['IDTIPORECE'],''),
                "bandeiraCartao"  => self::getBandeiraCartao($movcaixa['CDBANCARTCR']),
                "valorVenda"      => floatval($movcaixa['VRMOVIVEND']),
                "statusPagamento" => 0
            );
            array_push($arrPagamentos, array( "data"=>$arrPayment) );
			if($movcaixa['IDTIPORECE']=='9'){
				$valorValePresente = floatval($valorValePresente+$movcaixa['VRMOVIVEND']);
			}
        }
        $arrProdutos = array();
		$valorTransporte = 0;
		$valorGorjeta = 0;
        foreach ($itemvendaUDS as $itemvenda){
            $itemvendaArray = array(
                "idProduto"  => $itemvenda["CDPRODUTO"],
				"descProduto" => $itemvenda["NMPRODUTO"],
                "quantidade"  => $itemvenda["QTPRODVEND"],
                "valorItem"   => array(
					"unitario"  => $itemvenda["VRUNITVEND"],
                    "acrescimo" => floatval($itemvenda["VRACRITVEND"]),
                    "desconto"  => floatval($itemvenda["VRDESITVEND"])
                )
            );
            array_push($arrProdutos, array( "data"=>$itemvendaArray) );
			if($itemvenda["CDPRODUTO"] == $vendaUDS['CDPRODTAXAENTR']){
				$valorTransporte = floatval($itemvenda["VRACRITVEND"]);
			}
			if($itemvenda["CDPRODUTO"] == $vendaUDS['CDPRODTAXASERV']){
				$valorGorjeta = floatval($itemvenda["VRACRITVEND"]);
			}
        }
        return array(
            "idVenda"           => $vendaUDS['IDVENDA'],
            "idEstabelecimento" => $venda['CDFILIAL'],
            "codigoLoja"        => $venda['CDLOJA'],
            "idInterface"       => $venda['CDCAIXA'],
            "tipoInterface"     => 1,
            "idPedido"          => (empty($venda['NRVENDAREST'])?null:$vendaUDS['IDPEDIDO']),
            "conteudoVenda"     => $arrProdutos,

            "statusVenda"                  => 1,
            "valorVendaBruto"              => floatval($vendaUDS['VRTOTBRUTO']),
            "valorVendaLiquido"            => floatval($vendaUDS['VRTOTVENDA']),
            "valorVendaDeduzidoFidelidade" => floatval($vendaUDS['VRDESCCASH']),
            "valorDeduzidoFidelidade"      => floatval($vendaUDS['VRDESCCASH']),
            "tipoPagamento"                => (count($arrPagamentos)>1?1:0),

            "listaPagamentoSplit" => $arrPagamentos,

            "usarCashback"     => ( $vendaUDS['VRDESCCASH'] > 0 ? 'true' : 'false' ),
            "cpfCli"           => $vendaUDS['NRCPF'],//$fidelityDetails["userInfo"]["cpf"],
            "timestamp" 	   => (new \DateTime())->format('Y-m-d')."T".(new \DateTime())->format('H:i:s'),
            "origemVenda"      => $vendaUDS['IDORIGEMVENDA'],
			"origem" => "0",
            "modoAutonomo"     => false,

			"sistema" => 'TEK',
			"valorTransporte" 	=> $valorTransporte,
			"valorGorjeta" 		=> $valorGorjeta,
			"valorValePresente" => $valorValePresente,
			"cdCliente" 		=> $vendaUDS['CDCLIENTE'],
			"cdTipoCons" 		=> empty($vendaUDS['CDTIPOCONS'])?null:$vendaUDS['CDTIPOCONS'],
			"cdConsumidor" 		=> empty($vendaUDS['CDCONSUMIDOR'])?null:$vendaUDS['CDCONSUMIDOR'],
			"cdEXConsumidor" 	=> empty($vendaUDS['CDEXCONSUMID'])?null:$vendaUDS['CDEXCONSUMID']
        );
	}

	public function getCanalPag($stIdTipoRece, $stDsCartDigital=''){
		// (0)cartão (1)dinheiro (2)pix (3)mercado pago (4)vale refeição (5)vale presente (6)voucher (7)ifood
		if ($stIdTipoRece == '1') {
			return '0';
		} else if ($stIdTipoRece == '2') {
			return '0';
		} else if ($stIdTipoRece == '4') {
			return '1';
		} else if ($stIdTipoRece == 'H' && $stDsCartDigital == 'PIX') {
			return '2';
		} else if ($stIdTipoRece == 'H' && $stDsCartDigital != 'PIX') {
			return '3';
		} else if ($stIdTipoRece == '9') {
			return '5';
		} else if ($stIdTipoRece == '8') {
			return '7';
		} else {
			return '0';
		}
    }

    public function getBandeiraCartao($CDBANCARTCR){
        $bandeiraCartao = "0";
        if($CDBANCARTCR == '00001'){
            $bandeiraCartao = 1;
        } else if($CDBANCARTCR == '00002'){
            $bandeiraCartao = 2;
        } else if($CDBANCARTCR == '00004'){
            $bandeiraCartao = 3;
        } else if($CDBANCARTCR == '00003'){
            $bandeiraCartao = 4;
        } else if($CDBANCARTCR == '00031' || $CDBANCARTCR == '20032' || $CDBANCARTCR == '10082'){
            $bandeiraCartao = 5;
        } else if($CDBANCARTCR == '10014'){
            $bandeiraCartao = 6;
        } else if($CDBANCARTCR == '00030' || $CDBANCARTCR == '20003' || $CDBANCARTCR == '10012'){
            $bandeiraCartao = 7;
        }
        return $bandeiraCartao;
    }

    public function arrayCancUDS($CDFILIAL, $CDCAIXA, $DTABERCAIX, $NRSEQVENDA){
        return array(
            "idVenda" => $CDFILIAL.$CDCAIXA.$DTABERCAIX->format('dmy').$NRSEQVENDA,
            "idEstabelecimento" => $CDFILIAL,
            "idInterface" => $CDCAIXA,
            "tipoInterface" => 1,
			"timestamp" => (new \DateTime())->format('Y-m-d')."T".(new \DateTime())->format('H:i:s')
        );
    }

    private function castInteger($number){
        return intval(str_replace('.', '', number_format($number, 2)));
    }

    private function checkEmployeeCPF($CPF){
        $params = array(
            'NRCPFRESPCON' => $CPF
        );
        $employees = $this->entityManager->getConnection()->fetchAssoc("CONSULTA_CPF", $params);
        return !empty($employees);
    }

    private function updateItemVendaFamilia($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $CDFAMILISALD) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' =>$CDCAIXA,
            'NRSEQVENDA' =>$NRSEQVENDA,
            'CDFAMILISALD' => $CDFAMILISALD
        );
        $this->entityManager->getConnection()->executeQuery("UPDATE_ITEMVENDA_FAMILIA", $params);
    }

    private function defineTipoProduto($CDPRODPROMOCAO, $IDTIPOCOMPPROD, $IDIMPPRODUTO, $itensCombo) {
        $tipoProduto = 'normal';
        if (!empty($CDPRODPROMOCAO) && empty($itensCombo)) {
            $tipoProduto = 'filho';
        } else if (($IDTIPOCOMPPROD == '3' && $IDIMPPRODUTO == '2') || $IDTIPOCOMPPROD == 'C') {
            $tipoProduto = 'promocaoInsereFilhos';
        } else if (($IDTIPOCOMPPROD == '3') && ($IDIMPPRODUTO == '1')) {
            $tipoProduto = 'promocaoInserePai';
        }
        return $tipoProduto;
    }

    private function insertSaleItem($CDFILIAL, $CDLOJA, $CDCAIXA, $NRORG, $NRSEQVENDA, $item, $NRORDITCUPFIS, &$nrfeaturegrupo, $CDOPERADOR, $IDTPEMISVEND, $VRDESCVENDA, $TOTALVENDABRT, $CDPRODPROMOCAO, $NRSEQPRODCOMIT, $CDVENDEDOR, $VRTOTTRIBIBPT, $CDCLIENTE, $CDCONSUMIDOR, $IDESTTEMPOREAL, $IDORIGEMVENDA, &$nrsequitvendProdutoPai) {
        $connection = $this->entityManager->getConnection();
        if (empty($item['IDTIPOCOMPPROD'])) {
            $item['IDTIPOCOMPPROD'] = null;
        }
        if (empty($item['IDIMPPRODUTO'])) {
            $item['IDIMPPRODUTO'] = null;
        }
        if (empty($item['CDCAMPCOMPGANHE'])) {
            $item['CDCAMPCOMPGANHE'] = null;
        }
        if (empty($item['DTINIVGCAMPCG'])) {
            $item['DTINIVGCAMPCG'] = null;
        }
        if(empty($item['CDSUPERVDESC'])){
            $item['CDSUPERVDESC'] = null;
        }
        $itensCombo = array_key_exists('itensCombo', $item) ? $item['itensCombo'] : array();
        $itensComandaEst = array();
        $tipoProduto = $this->defineTipoProduto($CDPRODPROMOCAO, $item['IDTIPOCOMPPROD'], $item['IDIMPPRODUTO'], $itensCombo);

        if ($tipoProduto == 'promocaoInsereFilhos' || $tipoProduto == 'promocaoInserePai'){
            $CDCONTADOR = 'ITEMVENDAPROMO' . $CDFILIAL . $CDCAIXA . $NRSEQVENDA;
            $NRSEQPRODCOMIT = $this->util->geraCodigo($connection, $CDCONTADOR, $NRORG, 1, 3);
        }

        // se for promoção e será cobrado os itens filhos
        if ($tipoProduto == 'promocaoInsereFilhos') {
            //var_dump($item['CDPRODUTO'].' - promocaoInsereFilhos' );
            foreach ($item['itensCombo'] as $currentComboItem) {
                $NRORDITCUPFIS = $NRORDITCUPFIS + 1;
                $NRORDITCUPFIS = str_pad($NRORDITCUPFIS, 5, '0', STR_PAD_LEFT);
                //var_dump($currentComboItem['CDPRODUTO'].' - chamada recursiva insertSaleItem' );

                $this->insertSaleItem(
                    $CDFILIAL,
                    $CDLOJA,
                    $CDCAIXA,
                    $NRORG,
                    $NRSEQVENDA,
                    $currentComboItem,
                    $NRORDITCUPFIS,
                    $nrfeaturegrupo,
                    $CDOPERADOR,
                    $IDTPEMISVEND,
                    $VRDESCVENDA,
                    $TOTALVENDABRT,
                    $item['IDTIPOCOMPPROD'] == 'C' ? null :$item['CDPRODUTO'],
                    $NRSEQPRODCOMIT,
                    $CDVENDEDOR,
                    $VRTOTTRIBIBPT,
                    $CDCLIENTE,
                    $CDCONSUMIDOR,
                    $IDESTTEMPOREAL,
                    $IDORIGEMVENDA,
                    $nrsequitvendProdutoPai
                );
            }
        } else {
            $CDCONTADOR = 'ITEMVENDA' . $CDFILIAL . $CDCAIXA . $NRSEQVENDA;
            $NRSEQUITVEND = $this->util->geraCodigo($connection, $CDCONTADOR, $NRORG, 1, 3);

            $cdproduto = $item['CDPRODUTO'];
            $qtprodvend = $item['QTPRODVEND'];
            $vrunitvend = $item['VRUNITVEND'];
            $vrunitvendcl = $item['VRUNITVENDCL'];
            $idtipoitem = $item['IDTIPOITEM'];
            $idsituitem = $item['IDSITUITEM'];
            $vrdesitvend = $item['VRDESITVEND'];
            //Variável criada para controle de couvert em posições que já houve o recebimento.
            $nrlugarmesa = !empty($item['NRLUGARMESA']) ? $item['NRLUGARMESA'] : null;

            $dadosProdParams = array(
                'CDPRODUTO' => $cdproduto,
                'NRORG' => $NRORG,
                'CDFILIAL' => $CDFILIAL
            );
            $dadosProd = $this->entityManager->getConnection()->fetchAll("GET_DADOSPROD", $dadosProdParams);
            if (empty($dadosProd)){
                throw new \Exception('O produto  ' . $item['NMPRODUTO'] . ' não possui imposto parametrizado.<br><br>Caso este produto for parte da taxa de serviço, a venda não podera ser realizada.<br><br>Favor verificar a parametrização, ou possivelmente remover a taxa de serviço da venda.');
            }
            $sgunidade = $dadosProd[0]['SGUNIDADE'];
            $CDCLASFISC = $dadosProd[0]['CDCLASFISC'];
            $cdarvprod = $dadosProd[0]['CDARVPROD'];
            $CDCBENEFIT = $dadosProd[0]['CDCBENEF'];
            $vrpercvend = '0'; // VERIFICAR COMO CALCULA ESSE CAMPO
            $vraliqimpr = '0'; // VERIFICAR COMO CALCULA ESSE CAMPO4
            $nratraproditve = '0'; // VERIFICAR COMO CALCULA ESSE CAMPO
            $vrrattxserv = '0'; // VERIFICAR COMO CALCULA ESSE CAMPO
            $vrratdescven = '0'; // VERIFICAR COMO CALCULA ESSE CAMPO
            $dthrpedido = date('Y-m-d H:i:s'); // verificar se é o horário do front ou não (em caso de Sync pode dar diferença de horário)
            $baseCalcItVend = ($qtprodvend * ($vrunitvend + $vrunitvendcl)) - $item['VRDESITVEND'] + $item['VRACRITVEND'];
            //var_dump($cdproduto.' - '.$item['TXPRODCOMVEN'].' - '.$item['DSOBSITEMVENDA']);
            $saleItemParams = self::buildSaleItemParams(
                $CDFILIAL, $CDLOJA, $CDCAIXA, $qtprodvend,
                $vrunitvend, $vrunitvendcl, $item['VRDESITVEND'],
                $cdproduto, $NRORG, $NRSEQVENDA, $NRSEQUITVEND,
                $item['VRACRITVEND'], $NRORDITCUPFIS, $vrpercvend,
                $vraliqimpr, $nratraproditve, $vrrattxserv, $vrratdescven,
                $dthrpedido, $CDPRODPROMOCAO, $NRSEQPRODCOMIT, $CDVENDEDOR,
                $item['DSOBSITEMVENDA'], $item['DSOBSPEDDIGITA'], $CDCLASFISC,
                $item['DSOBSDESCIT'], $item['CDGRPOCORDESCIT'], $item['IDORIGEMVENDA'],
                $CDCBENEFIT, $nrlugarmesa, $item['CDCAMPCOMPGANHE'],
                $item['DTINIVGCAMPCG'], $item['IDIMPPRGRPROMO'], $item['IDOBSERVACAOAD'], $item['CDSUPERVDESC']
            );

            $this->insereLinhaLogVenda('Gravando item na ITEMVENDA');
            $this->geraLogVenda($saleItemParams[0]);
            $this->entityManager->getConnection()->executeQuery("INSERT_ITEMVENDA", $saleItemParams[0], $saleItemParams[1]);
            $this->insereLinhaLogVenda('Fim da gravação na ITEMVENDA');
            $this->insereLinhaLogVenda('-----------------------------------------------------');
            
            $itensComandaEst = array();
            if ($IDORIGEMVENDA == 'MES_PKR' || $IDORIGEMVENDA == 'CMD_PKC'){
                //var_dump(' MES_PKR || CMD_PKC' );
                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'NRVENDAREST' => $item['NRVENDAREST'],
                    'NRCOMANDA' => $item['NRCOMANDA'],
                    'NRPRODCOMVEN' => $item['NRPRODCOMVEN']
                );
                $itensComandaEst = $this->entityManager->getConnection()->fetchAll("GET_ITENS_EST", $params);

                foreach ($itensComandaEst as $itens){
                    //var_dump($itens['CDPRODUTO'].' - insereITEMVENDAEST - '.$itens['NRPRODCOMVEN'] );
                    $this->insereITEMVENDAEST(
                        $CDFILIAL,
                        $CDCAIXA,
                        $NRSEQVENDA,
                        $NRSEQUITVEND,
                        $itens['CDPRODUTO'],
                        $itens['QTPROCOMEST'],
                        $itens['VRPRECCOMEST'],
                        $itens['VRDESITCOMEST'],
                        $itens['NRATRAPRODCOES'],
                        $itens['DSOBSPEDDIGEST'],
                        $itens['IDIMPPRGRPROMO'],
                        $itens['VRACRITCOMEST'],
                        $itens['TXPRODCOMVENEST']
                    );
                    $params = array(
                        'CDFILIAL' => $CDFILIAL,
                        'NRVENDAREST' => $item['NRVENDAREST'],
                        'NRCOMANDA' => $item['NRCOMANDA'],
                        'NRPRODCOMVEN' => $item['NRPRODCOMVEN'],
                        'CDPRODUTO' => $itens['CDPRODUTO']
                    );
                    $obsComandaEst = $this->entityManager->getConnection()->fetchAll("GET_OBS_EST", $params);

                    foreach ($obsComandaEst as $obs){
                        $this->insereOBSITEMVENDAEST(
                            $CDFILIAL,
                            $CDCAIXA,
                            $NRSEQVENDA,
                            $NRSEQUITVEND,
                            $obs['CDPRODUTO'],
                            $obs['CDGRPOCOR'],
                            $obs['CDOCORR']
                        );
                    }
                }
            }
            // Insere o voucher na USOCUPOMDESCFOS.
            if ($item['VOUCHER']){
                if($item['VOUCHER']['CDCUPOMDESCFOS']){
                    $params = array(
                        'CDCUPOMDESCFOS'=> $item['VOUCHER']['CDCUPOMDESCFOS'],
                        'CDFILIAL'      => $CDFILIAL,
                        'CDCAIXA'       => $CDCAIXA,
                        'NRSEQVENDA'    => $NRSEQVENDA,
                        'NRSEQUITVEND'  => $NRSEQUITVEND
                    );
                    $this->entityManager->getConnection()->executeQuery("INSERT_USOCUPOMDESCFOS", $params);
                }
            }

            if (isset($item['DTHRINCOMVEN'])) {
                if (is_a($item['DTHRINCOMVEN'], 'DateTime')) {
                    $DTITHRCOMANDA = $item['DTHRINCOMVEN'];
                } else {
                    $DTITHRCOMANDA = explode('.', $item['DTHRINCOMVEN'])[0];
                }
            } else {
                $DTITHRCOMANDA = $dthrpedido;
            }
            $NRVENDAREST = isset($item['NRVENDAREST']) ? $item['NRVENDAREST'] : null;
            $NRSEQUITVEND_ITHRCOMANDA = $NRSEQUITVEND;
            $this->insereITHRCOMANDA(
                $CDFILIAL,
                $NRSEQUITVEND_ITHRCOMANDA,
                $cdproduto,
                $DTITHRCOMANDA,
                $qtprodvend,
                $CDCAIXA,
                $NRVENDAREST,
                $NRSEQVENDA
            );

            foreach ($item['OBSERVACOES'] as $currentObs) {
                self::insereOBSITEMVENDA(
                    $CDFILIAL,
                    $CDCAIXA,
                    $NRSEQVENDA,
                    $NRSEQUITVEND,
                    $currentObs['CDGRPOCOR'],
                    $currentObs['CDOCORR']
                );
            }

            if ($item['IDTIPOITEM'] == self::ITEM_ABASTECIMENTO) {

                $dataconclusaoabastecimento = $item['DATACONCLUSAOABASTECIMENTO'];
                $nrbico = $item['NRBICO'];
                $vrencerranteinicial = $item['VRENCERRANTEINICIAL'];
                $vrencerrantefinal = $item['VRENCERRANTEFINAL'];

                $getBombaParams = array(
                    'NRBICO' => $nrbico
                );

                $nrbombaResult = $this->entityManager->getConnection()->fetchAll("GET_BOMBA_BY_BICO", $getBombaParams);
                $nrbomba = $nrbombaResult[0]['NRBOMBA'];
                $cdbomba = $nrbombaResult[0]['CDBOMBA'];
                $cdbico = $nrbombaResult[0]['CDBICO'];

                $getTanquebicohParams = array(
                    'NRSEQBICO' => $nrbico,
                    'DTABASTECIMENTO' => $this->databaseUtil->convertToDateTime($dataconclusaoabastecimento . ':00')
                );
                $tanquebicohResult = $this->entityManager->getConnection()->fetchAll("GET_TANQUEBICOH_BY_BICO", $getTanquebicohParams);
                $CDFILIALalmoxarifado = $tanquebicohResult[0]['CDFILIALALMOXARIFADO'];
                $cdalmoxarifado = $tanquebicohResult[0]['CDALMOXARIFADO'];
                $nritemvendaauxResult = $this->entityManager->getConnection()->fetchAll("GET_NOVO_NRITEMVENDAUXILIAR");
                $nritemvendauxiliar = $nritemvendaauxResult[0]['NRITEMVENDAUXILIAR'];
                $getTipocombustivelParams = array(
                    'CDPRODUTO' => $cdproduto,
                    'NRORG'     => $NRORG
                );
                $nrtipocombustivelResult = $this->entityManager->getConnection()->fetchAll("GET_TIPOCOMBUSTIVEL_BY_PRODUTO", $getTipocombustivelParams);
                $nrtipocombustivel = null;
                if (isset($nrtipocombustivelResult[0]['NRTIPOCOMBUSTIVEL'])) {
                    $nrtipocombustivel = $nrtipocombustivelResult[0]['NRTIPOCOMBUSTIVEL'];
                }

                $auxSaleItemParams = self::buildAuxSaleItemParams($CDFILIAL, $CDCAIXA, $nritemvendauxiliar, $CDFILIALalmoxarifado, $cdalmoxarifado, $dataconclusaoabastecimento, $nrtipocombustivel, $nrbomba, $nrbico, $NRORG, $NRSEQVENDA, $NRSEQUITVEND, $vrencerranteinicial, $vrencerrantefinal);
                $this->entityManager->getConnection()->executeQuery("INSERT_ITEMVENDAAUXILIAR", $auxSaleItemParams);
            }

            if (isset($item['NRFEATUREGRUPO'])) {
                $nrfeaturegrupo = $item['NRFEATUREGRUPO'];
                if ($nrfeaturegrupo != null) {
                    $nritemvendaauxResult = $this->entityManager->getConnection()->fetchAll("GET_NOVO_NRITEMVENDAUXILIAR");
                    $nritemvendauxiliar = $nritemvendaauxResult[0]['NRITEMVENDAUXILIAR'];
                    $auxSaleItemFeatureParams = self::buildAuxSaleItemFeatureParams($nritemvendauxiliar, $CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $NRSEQUITVEND, $nrfeaturegrupo, $CDOPERADOR);
                    $this->entityManager->getConnection()->executeQuery("INSERT_ITEMVENDAAUXILIAR_FEATUREGRUPO", $auxSaleItemFeatureParams);
                }
            }
            //Insere os possiveis beneficios
            if(!empty($item['beneficios'])){
                $item['beneficios'] = $item['beneficios'][0];
                $CDCAMPANHA = $item['beneficios']['CDCAMPANHA'];
                $NRSEQPRODCAMP = $item['beneficios']['NRSEQPRODCAMP'];
                $NRSEQBENEFICIO = $item['beneficios']['NRSEQBENEFICIO'];
                $QTDEBENEFICIO = $item['beneficios']['QTDEBENEFICIO'];
                $params = array(
                    ':QTDEBENEFICIO'  => (string)$QTDEBENEFICIO,
                    ':CDCLIENTE'      => $CDCLIENTE,
                    ':CDCONSUMIDOR'   => $CDCONSUMIDOR,
                    ':CDCAMPANHA'     => $CDCAMPANHA,
                    ':NRSEQUITVEND'   => $NRSEQUITVEND,
                    ':NRSEQPRODCAMP'  => $NRSEQPRODCAMP,
                    ':NRSEQBENEFICIO' => $NRSEQBENEFICIO
                );
                $this->entityManager->getConnection()->executeQuery("UPDATE_MOVBENEFICIOCTR", $params);
                $NRSEQBENECONS = $this->util->geraCodigo($this->entityManager->getConnection(), 'MOVBENEFICIOCONS', $NRORG, 1, 10);
                $params = array(
                    ':CDCAMPANHA'     => $CDCAMPANHA,
                    ':NRSEQBENECONS'  => $NRSEQBENECONS,
                    ':NRSEQPRODCAMP'  => $NRSEQPRODCAMP,
                    ':NRSEQBENEFICIO' => $NRSEQBENEFICIO,
                    ':CDCLIENTE'      => $CDCLIENTE,
                    ':CDCONSUMIDOR'   => $CDCONSUMIDOR,
                    ':DTUTBENECONS'   => $this->databaseUtil->getCurrentDateTime(),
                    ':IDIMPTMOV'      => 'A',//A de aalterado, quando subir a venda vira N
                    ':CDFILIAL'       => $CDFILIAL,
                    ':CDCAIXA'        => $CDCAIXA,
                    ':NRSEQVENDA'     => $NRSEQVENDA,
                    ':NRSEQUITVEND'   => $NRSEQUITVEND
                );
                $this->entityManager->getConnection()->executeQuery("INSERT_MOVBENEFICIOCONS", $params);
            }
            // TRATAMENTO DOS IMPOSTOS
            if (!isset($item['IMPOSTOS'])) {
                $this->calculaImposto($CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $NRSEQUITVEND, $item, $IDTPEMISVEND, $baseCalcItVend,
                 $VRDESCVENDA, $TOTALVENDABRT, $VRTOTTRIBIBPT);
            }

            // @todo - verificar com Thais
            $NRLANCESTQ = null;
            if ($IDESTTEMPOREAL == self::SINCRONIZA_ESTOQUE) {
                if ($NRLANCESTQ !== null) {
                    $keys = array(
                        'NRLANCESTQ'   => $NRLANCESTQ,
                        'NRSEQVENDA'   => $NRSEQVENDA,
                        'NRSEQUITVEND' => $NRSEQUITVEND
                    );
                    $this->estoque->updateProductStock($item, $NRORG, $CDFILIAL, $CDLOJA, $CDCAIXA, $keys, $DTVENDA, null, null, null, null, $CDOPERADOR);
                }
            }
        }
        // ITEMVENDAEST é inserido apenas após o PAI estiver na ITEMVENDA
        if ($tipoProduto == 'promocaoInserePai') {
            $nrsequitvendProdutoPai = $NRSEQUITVEND;
            // Agrupa itens semelhantes antes de inserir na ITEMVENDAEST.
            $itensCombo = array();
            $absorbPrice = false; // Utilizado depois.
            foreach ($item['itensCombo'] as $currentComboItem){
                if (!array_key_exists($currentComboItem['CDPRODUTO'], $itensCombo)){
                    foreach ($currentComboItem['itensCombo'] as $comboItemDentrodoCombo){
                    	//quando o item do combo é um combo, percorre os seus itens
                        if (!array_key_exists($comboItemDentrodoCombo['CDPRODUTO'], $comboItemDentrodoCombo)){
                            $itensCombo[$comboItemDentrodoCombo['CDPRODUTO']] = $comboItemDentrodoCombo;
                        } else {
                            $itensCombo[$comboItemDentrodoCombo['CDPRODUTO']]['QTPRODVEND']++;
                        }
                        // Define se o produto pai está entre os filhos. Caso esteja, o preço dos filhos vai para ele.
                        if ($currentComboItem['CDPRODUTO'] == $item['CDPRODUTO'] && $absorbPrice == false) $absorbPrice = true;
                    }
                    $itensCombo[$currentComboItem['CDPRODUTO']] = $currentComboItem;
                }
                else {
                    $itensCombo[$currentComboItem['CDPRODUTO']]['QTPRODVEND']++;
                }
                // Define se o produto pai está entre os filhos. Caso esteja, o preço dos filhos vai para ele.
                if ($currentComboItem['CDPRODUTO'] == $item['CDPRODUTO'] && $absorbPrice == false) $absorbPrice = true;
            }

            foreach ($itensCombo as $currentComboItem) {

                if ($currentComboItem['IDIMPPRGRPROMO'] == 'S'){
                    $price = $currentComboItem['VRUNITVEND'];
                    $discount = $currentComboItem['VRDESITVEND'];
                    $increase = $currentComboItem['VRACRITVEND'];
                }
                else {
                    if ($absorbPrice){
                        if ($currentComboItem['CDPRODUTO'] == $item['CDPRODUTO']){
                            $price = $currentComboItem['VRUNITVEND'];
                            $discount = $currentComboItem['VRDESITVEND'];
                            $increase = $currentComboItem['VRACRITVEND'];
                        }
                        else {
                            $price = 0;
                            $discount = 0;
                            $increase = 0;
                        }
                    }
                    else {
                        $qtItens = 0;
                        foreach ($itensCombo as $prodEst){
                            if ($prodEst['IDIMPPRGRPROMO'] != 'S') $qtItens += $prodEst['QTPRODVEND'];
                        }
                        $price = bcdiv(strval($item['VRUNITVEND']), strval($qtItens), 3);
                        $discount = bcdiv(strval($item['VRDESITVEND']), strval($qtItens), 3);
                        $increase = bcdiv(strval($item['VRACRITVEND']), strval($qtItens), 3);
                    }
                }
                //var_dump($currentComboItem['CDPRODUTO'].' - item da promocaoInserePai insereITEMVENDAEST - '.$currentComboItem['NRPRODCOMVEN'] );
                $this->insereITEMVENDAEST(
                    $CDFILIAL,
                    $CDCAIXA,
                    $NRSEQVENDA,
                    $NRSEQUITVEND,
                    $currentComboItem['CDPRODUTO'],
                    $currentComboItem['QTPRODVEND'],
                    $price,
                    $discount,
                    null,
                    null,
                    $currentComboItem['IDIMPPRGRPROMO'],
                    $increase,
                    $currentComboItem['TXPRODCOMVEN']
                );
                foreach ($currentComboItem['OBSERVACOES'] as $currentObs) {
                    $this->insereOBSITEMVENDAEST(
                        $CDFILIAL,
                        $CDCAIXA,
                        $NRSEQVENDA,
                        $NRSEQUITVEND,
                        $currentComboItem['CDPRODUTO'],
                        $currentObs['CDGRPOCOR'],
                        $currentObs['CDOCORR']
                    );
                }
            }
        }
        // se for item filho de um combo tambem insere na itcomandaest
        // quando é venda mesa/comanda os itens do combo não vem no array, eles vem da consulta na itcomandaest
        if ($tipoProduto == 'filho' && empty($itensComandaEst) && !($IDORIGEMVENDA == 'MES_PKR' || $IDORIGEMVENDA == 'CMD_PKC')) {
            $price = $item['VRUNITVEND'];
            $discount = $item['VRDESITVEND'];
            $increase = $item['VRACRITVEND'];
            $item['TXPRODCOMVEN'] = isset($item['TXPRODCOMVEN'])?$item['TXPRODCOMVEN']:'';
            $boJaInseriu = false;
            foreach ($itensComandaEst as $itemEst) {
                if($itemEst['CDPRODUTO'] == $item['CDPRODUTO'] && $itemEst['NRPRODCOMVEN'] == $item['NRPRODCOMVEN']){
                    $boJaInseriu = true;
                    //var_dump($item['CDPRODUTO'].' - boJaInseriu ');
                    break;
                }
            }
            if(!$boJaInseriu) {
                //var_dump($item['CDPRODUTO'].' - filho insereITEMVENDAEST - '.$item['NRPRODCOMVEN'] );
                $this->insereITEMVENDAEST(
                    $CDFILIAL,
                    $CDCAIXA,
                    $NRSEQVENDA,
                    $nrsequitvendProdutoPai == null ? $NRSEQUITVEND : $nrsequitvendProdutoPai,
                    $item['CDPRODUTO'],
                    $item['QTPRODVEND'],
                    $price,
                    $discount,
                    null,
                    null,
                    $item['IDIMPPRGRPROMO'],
                    $increase,
                    $item['TXPRODCOMVEN']
                );
                foreach ($item['OBSERVACOES'] as $currentObs) {
                    $this->insereOBSITEMVENDAEST(
                        $CDFILIAL,
                        $CDCAIXA,
                        $NRSEQVENDA,
                        $NRSEQUITVEND,
                        $item['CDPRODUTO'],
                        $currentObs['CDGRPOCOR'],
                        $currentObs['CDOCORR']
                    );
                }
            }
    	}

        $this->insereLinhaLogVenda('Fim da gravação do item atual');
        $this->insereLinhaLogVenda('-----------------------------------------------------');
    }

    public function insereITHRCOMANDA($CDFILIAL, $NRSEQUITEM, $CDPRODUTO, $DTHRPEDIDO, $QTPRODCOMVEN, $CDCAIXA, $NRVENDAREST, $NRSEQVENDA, $IDIMPITHRCOMANDA = 'N'){
        if (!is_a($DTHRPEDIDO, 'DateTime')) {
            $DTHRPEDIDO = new \DateTime($DTHRPEDIDO);
        }
        $params = array(
            'CDFILIAL'         => $CDFILIAL,
            'NRSEQUITEM'       => substr($NRSEQUITEM,-3),
            'CDPRODUTO'        => $CDPRODUTO,
            'DTHRPEDIDO'       => $DTHRPEDIDO,
            'QTPRODCOMVEN'     => $QTPRODCOMVEN,
            'CDCAIXA'          => $CDCAIXA,
            'NRVENDAREST'      => $NRVENDAREST,
            'NRSEQVENDA'       => $NRSEQVENDA,
            'IDIMPITHRCOMANDA' => $IDIMPITHRCOMANDA
        );
        $type = array(
            'DTHRPEDIDO' => \Doctrine\DBAL\Types\Type::DATETIME
        );

        $this->insereLinhaLogVenda('Gravando na tabela ITHRCOMANDA');
        $this->geraLogVenda($params);
        $this->entityManager->getConnection()->executeQuery("INSERT_ITHRCOMANDA", $params, $type);
        $this->insereLinhaLogVenda('Fim da gravação na ITHRCOMANDA');
        $this->insereLinhaLogVenda('-----------------------------------------------------');
    }

    private function buscaProdutoObs($CDGRPOCOR, $CDOCORR) {
        $result = array();
        $params = array(
            'CDGRPOCOR' => $CDGRPOCOR,
            'CDOCORR' => $CDOCORR
        );
        $OCORRENCIA = $this->entityManager->getConnection()->fetchAssoc("BUSCA_OCORRENCIA", $params);
        if (!empty($OCORRENCIA) && $OCORRENCIA['IDCONTROLAOBS'] == self::OBS_ADICIONA_PRODUTO) {
            $result = $OCORRENCIA;
        }
        return $result;
    }

    private function insereOBSITEMVENDA($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND, $CDGRPOCOR, $CDOCORR) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRSEQVENDA' => $NRSEQVENDA,
            'NRSEQUITVEND' => $NRSEQUITVEND,
            'CDGRPOCOR' => $CDGRPOCOR,
            'CDOCORR' => $CDOCORR
        );
        $this->entityManager->getConnection()->executeQuery("INSERT_OBSITEMVENDA", $params);
    }

    private function insereOBSITEMVENDAEST($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND, $CDPRODUTO, $CDGRPOCOR, $CDOCORR) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRSEQVENDA' => $NRSEQVENDA,
            'NRSEQUITVEND' => $NRSEQUITVEND,
            'CDPRODUTO' => $CDPRODUTO,
            'CDGRPOCOR' => $CDGRPOCOR,
            'CDOCORR' => $CDOCORR
        );
        $this->entityManager->getConnection()->executeQuery("INSERT_OBSITEMVENDAEST", $params);
    }

    private function insereITEMVENDAEST($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND, $CDPRODUTO, $QTITVENDAEST, $VRUNVENDAEST, $VRDESITVENDAEST, $NRATRAPRODITES, $DSOBSPEDDIGITE, $IDIMPPRGRPROMO, $VRACRITVENDAEST, $DSOBSITVENDAEST){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRSEQVENDA' => $NRSEQVENDA,
            'NRSEQUITVEND' => $NRSEQUITVEND,
            'CDPRODUTO' => $CDPRODUTO,
            'QTITVENDAEST' => $QTITVENDAEST,
            'VRUNVENDAEST' => $VRUNVENDAEST,
            'VRDESITVENDAEST' => $VRDESITVENDAEST,
            'NRATRAPRODITES' => $NRATRAPRODITES,
            'DSOBSPEDDIGITE' => $DSOBSPEDDIGITE,
            'IDIMPPRGRPROMO' => $IDIMPPRGRPROMO,
            'VRACRITVENDAEST' => $VRACRITVENDAEST,
            'DSOBSITVENDAEST' => $DSOBSITVENDAEST
        );
        $this->entityManager->getConnection()->executeQuery("INSERE_ITEMVENDAEST", $params);
    }

    private function trataMascaraImposto($valor) {
        $valor = str_replace(' ', '', $valor);
        $valor = str_replace(',', '.', $valor);
        return floatval($valor);
    }

    private function calculaImposto($CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $NRSEQUITVEND, $item, $IDTPEMISVEND, $baseCalcItVend, $VRDESCVENDA, $TOTALVENDABRT, $VRTOTTRIBIBPT) {
        $getAliqImpFisParams = array(
            'CDFILIAL' => $CDFILIAL,
            'CDPRODUTO'  => $item['CDPRODUTO']
        );
        $aliqImpFisResult = $this->entityManager->getConnection()->fetchAssoc("GET_ALIQIMPFIS_IMPOSTO", $getAliqImpFisParams);
        $aliqImpFisResult['VRALIQPIS'] = $this->trataMascaraImposto($aliqImpFisResult['VRALIQPIS']);
        $aliqImpFisResult['VRALIQCOFINS'] = $this->trataMascaraImposto($aliqImpFisResult['VRALIQCOFINS']);
        $aliqImpFisResult['VRPEALIMPFIS'] = $this->trataMascaraImposto($aliqImpFisResult['VRPEALIMPFIS']);

        $doDescItem = round($item['VRDESITVEND'], 2);
        $doAcreItem = round($item['VRACRITVEND'], 2);

        $baseValue = floatval(bcmul(str_replace(',','.',strval($item['QTPRODVEND'])), str_replace(',','.',strval(bcadd(str_replace(',','.',strval($item['VRUNITVEND'])), str_replace(',','.',strval($item['VRUNITVENDCL'])), '2'))), '2'));
        $vrBaseCalcIcmsPisCofins = floatval(bcsub(str_replace(',','.',strval(bcadd(str_replace(',','.',strval($baseValue)), str_replace(',','.',strval($doAcreItem)), '2'))), str_replace(',','.',strval($doDescItem)), '2'));
        if($vrBaseCalcIcmsPisCofins < 0) $vrBaseCalcIcmsPisCofins = 0;
        $vrBaseCalcIcms = $vrBaseCalcIcmsPisCofins;
        $reductionPercentage = isset($aliqImpFisResult['VRPERCREDUCAO']) ? floatval($aliqImpFisResult['VRPERCREDUCAO']) : 0;

        //Cálcula ICMS com redução da base de cálculo
        $vrBaseCalcReduz = 0;
        if($reductionPercentage > 0){
            $vrBaseCalcReduz = $aliqImpFisResult['IDTRATIMPO'] == 'A' ?
                $vrBaseCalcIcmsPisCofins - $this->util->roundABNT($this->percentageToAbsolute(floatval(bcsub(str_replace(',','.',strval($baseValue)), str_replace(',','.',strval($doDescItem)), '2')), $reductionPercentage), 2) :
                $vrBaseCalcIcmsPisCofins - $this->util->roundABNT($this->percentageToAbsolute($vrBaseCalcIcmsPisCofins, $reductionPercentage), 2);
            $vrBaseCalcReduz = self::trunc($vrBaseCalcReduz);
        }

        $doVrImpoProdIt = $this->percentageToAbsolute($vrBaseCalcIcmsPisCofins, $aliqImpFisResult['VRPEALIMPFIS']);

        $params = array(
            'CDFILIAL' => $CDFILIAL
        );
        $dadosParavend = $this->entityManager->getConnection()->fetchAll('BUSCA_DADOS_PARAVEND', $params);
        //Verifica se a filial está parametrizada para usar a Composição de Base de Cálculo do PIS/Cofins
        $verifyBaseCalcPisCof = null;
        if($dadosParavend[0]['IDCOMPPISCOFINS'] === 'S'){
            $params = array(
                'CDFILIAL'     => $CDFILIAL,
                'CDCAIXA'      => $CDCAIXA,
                'NRSEQVENDA'   => $NRSEQVENDA,
                'NRSEQUITVEND' => $NRSEQUITVEND
            );
            $verifyBaseCalcPisCof = $this->entityManager->getConnection()->fetchAssoc('GET_INFO_IMPOSTO_TIPO_OPERACAO', $params);
            if(empty($verifyBaseCalcPisCof)){
                $verifyBaseCalcPisCof = $this->entityManager->getConnection()->fetchAssoc('GET_INFO_IMPOSTO_GERAL', $params);
            }
            //Verifica se possui algum imposto parametrizado para compor a base de cálculo do PIS/COFINS
            if(!empty($verifyBaseCalcPisCof)){
                $vrBaseCalcIcmsPisCofins =  ($vrBaseCalcReduz > 0) ? $vrBaseCalcIcmsPisCofins - $vrBaseCalcReduz : $vrBaseCalcIcmsPisCofins - $doVrImpoProdIt;
            }
        }
        $cdImpostoItVendaImp = $verifyBaseCalcPisCof ?
        $verifyBaseCalcPisCof['CDIMPOSTO'] : $aliqImpFisResult['CDIMPOSTO'];
        $doVrImpPis = $this->percentageToAbsolute($vrBaseCalcIcmsPisCofins, $aliqImpFisResult['VRALIQPIS']);
        $doVrImpCofins = $this->percentageToAbsolute($vrBaseCalcIcmsPisCofins, $aliqImpFisResult['VRALIQCOFINS']);

        $paramsItvendaimpos = $this->initParamsFor('ITVENDAIMPOS');
        $paramsItvendaimpos['CDFILIAL'] = $CDFILIAL;
        $paramsItvendaimpos['CDCAIXA'] = $CDCAIXA;
        $paramsItvendaimpos['NRSEQVENDA'] = $NRSEQVENDA;
        $paramsItvendaimpos['NRSEQUITVEND'] = $NRSEQUITVEND;
        $paramsItvendaimpos['VRTOTTRIBIBPT'] = $VRTOTTRIBIBPT;
        $paramsItvendaimpos['CDIMPOSTO'] = $cdImpostoItVendaImp;
        $paramsItvendaimpos['VRPEALPRODIT'] = $aliqImpFisResult['VRPEALIMPFIS'];
        $paramsItvendaimpos['CDIMPOSTOEX'] = $aliqImpFisResult['CDINTIMPOSTO'];
        $paramsItvendaimpos['CDCSTPRODI'] = $aliqImpFisResult['CDCSTICMS'];
        $paramsItvendaimpos['CDCSTPRODPC'] = $aliqImpFisResult['CDCSTPISCOF'];
        $paramsItvendaimpos['CDCFOPPROD'] = $aliqImpFisResult['CDCFOPPFIS'];
        $paramsItvendaimpos['CDNATUREC'] = $aliqImpFisResult['CDNATUREC'];
        $paramsItvendaimpos['VRPERCOFINS'] = $aliqImpFisResult['VRALIQCOFINS'];
        $paramsItvendaimpos['VRPERPIS'] = $aliqImpFisResult['VRALIQPIS'];
        $paramsItvendaimpos['IDMODALBASECALC'] = $aliqImpFisResult['IDMODALBASECALC'];
        $paramsItvendaimpos['VRBASECALCREDUZ'] = $vrBaseCalcReduz;
        $paramsItvendaimpos['VRIMPOPRODREDUZ'] = $this->util->roundABNT($this->percentageToAbsolute($vrBaseCalcReduz, $aliqImpFisResult['VRPEALIMPFIS']), 2);
        $paramsItvendaimpos['VRBASECALCICMS'] = $aliqImpFisResult['IDTRATIMPO'] == 'A' ? floatval(bcsub(str_replace(',','.',strval($baseValue)), str_replace(',','.',strval($doDescItem)), '2')) : $vrBaseCalcIcms;
        $paramsItvendaimpos['VRBCREDUZICMS'] = $reductionPercentage == 0 ? 0 : floatval(bcsub(str_replace(',','.',strval($vrBaseCalcIcmsPisCofins)), str_replace(',','.',strval($vrBaseCalcReduz)), '2'));
        $paramsItvendaimpos['VRPRBCREDUICMS'] = $reductionPercentage;
        $paramsItvendaimpos['VRIMPOPRODIT'] = $this->util->roundABNT($doVrImpoProdIt, 2);
        $paramsItvendaimpos['VRIMPPIS'] = $this->util->roundABNT($doVrImpPis, 2);
        $paramsItvendaimpos['VRIMPCOFINS'] = $this->util->roundABNT($doVrImpCofins, 2);
        $paramsItvendaimpos['VRBCPISCOFINS'] = $this->util->roundABNT($vrBaseCalcIcmsPisCofins, 2);
        $paramsItvendaimpos['NRORG'] = $NRORG;
        $paramsItvendaimpos['VRIMPICMBPIS'] = $this->util->roundABNT($paramsItvendaimpos['VRBASECALCICMS'] * $aliqImpFisResult['VRALIQPIS'] /100 , 2);
        $paramsItvendaimpos['VRIMPICMBCOFINS'] = $this->util->roundABNT($vrBaseCalcIcmsPisCofins * $aliqImpFisResult['VRALIQCOFINS'] / 100 , 2);

        //var_dump($paramsItvendaimpos);

        $this->insereLinhaLogVenda('Gravando na tabela ITVENDAIMPOS');
        $this->geraLogVenda($paramsItvendaimpos);
        $this->entityManager->getConnection()->executeQuery("INSERT_ITVENDAIMPOS", $paramsItvendaimpos);
        $this->insereLinhaLogVenda('Fim da gravação na ITVENDAIMPOS');
        $this->insereLinhaLogVenda('-----------------------------------------------------');
    }



    private function percentageToAbsolute($baseValue, $percentualValue) {
        return $baseValue * $percentualValue * 0.01;
    }

    private function applyPercentage($baseValue, $percentualValue, $mode) {
        if ($mode == "SUM" || $mode == "SUBTRACTION") {
            throw new \Exception("Invalid mode passed to applyPercentage function.");
        }
        $multiplier = $mode == "SUM" ? 1 : -1;
        return $baseValue + ($multiplier * $this->percentageToAbsolute($baseValue, $percentualValue));
    }

    private function initParamsFor($paramsKey) {
        switch ($paramsKey) {
            case 'ITVENDAIMPOS':
                return array(
                    'NRSEQITIMPOS' => '001',
                    'VRIMPOPRODIT' => 0,
                    'CDIMPOSTOEX' => null,
                    'CDCSTPRODI' => 0,
                    'CDCSTPRODPC' => 0,
                    'CDCFOPPROD' => 0,
                    'VRPERCOFINS' => 0,
                    'VRPERPIS' => 0,
                    'IDMODALBASECALC' => 0,
                    'VRBASECALCREDUZ' => 0,
                    'VRIMPOPRODREDUZ' => 0,
                    'VRBASECALCICMS' => 0,
                    'VRBCREDUZICMS' => 0,
                    'VRPRBCREDUICMS' => 0,
                    'VRIMPPIS' => 0,
                    'VRIMPCOFINS' => 0/* ,
                    'VRTOTTRIBIBPT' => 0 */
                );
            case 'PRODITEM':
                return array(
                    'VRPEALIMPFIS' => 0
                );
            default: return array();
        }
    }

    private function insertSaleItemCanc($CDFILIAL, $CDLOJA, $CDCAIXA, $NRORG, $NRSEQVENDA, $item, $NRORDITCUPFISca) {

        //--- Gets the firm properties
        $cdproduto       = $item['CDPRODUTO'];
        $qtprodvendc     = $item['QTPRODVEND'];
        $vrunitvendc     = $item['VRUNITVEND'];
        $vrunitvenccl    = empty($item['VRUNITVENDCL']) ? 0 : $item['VRUNITVENDCL'];
        $idtipoitemc     = $item['IDTIPOITEM'];
        $cdgrpocorcanite = $item['CDGRPOCOR'];
        $dsobscanite     =  !empty($item['DSOBSITEMVENDA']) ? substr($item['DSOBSITEMVENDA'], 0, 100) : null;
        $cdsupervisor    = $item['CDSUPERVISOR'];
        $IDPRODPRODUZC   = $item['IDPRODPRODUZC'];
        $DTHRPRODCAN     = $item['DTHRPRODCANVEN'];
        $CDOPERCANC      = isset($item['CDOPERCANC']) ? $item['CDOPERCANC'] : '';

        //---- TEMPORARIO
        $vracritvendc = isset($item['VRACRITVEND']) ? $item['VRACRITVEND'] : 0;
        //----
        $vrdesitvendc = $item['VRDESITVEND'];
        $CDCONTADOR = 'ITEMVENDACANC' . $CDFILIAL . $CDCAIXA . $NRSEQVENDA;
        $NRSEQITVENDC = $this->util->geraCodigo($this->entityManager->getConnection(), $CDCONTADOR, $NRORG, 1, 3);
        $saleItemCancParams = self::buildSaleItemCancParams($CDFILIAL, $CDCAIXA, $qtprodvendc, $vrunitvendc, $vrunitvenccl, $vrdesitvendc, $cdproduto, $NRORG, $NRSEQVENDA, $NRSEQITVENDC, $vracritvendc, $NRORDITCUPFISca, $cdgrpocorcanite, $dsobscanite, $cdsupervisor, $IDPRODPRODUZC, $DTHRPRODCAN, $CDOPERCANC);
        $this->entityManager->getConnection()->executeQuery("INSERT_ITEMVENDA_CANCELADO", $saleItemCancParams[0], $saleItemCancParams[1]);
    }

    private function insertPayment($payment, $CDFILIAL, $CDCAIXA, $DTABERCAIX, $DTVENDA, $NRSEQVENDA, $NRORG, $CDOPERADOR, $CDCLIENTE) {
        $idtipomovive = self::ENTRADA;
        $vrmovivend   = $payment['VRMOVIVEND'];
        $cdtiporece   = $payment['CDTIPORECE'];
        $qtparcreceb  = isset($payment['NRPARCELAS']) ? $payment['NRPARCELAS'] : 1;
        $CDNSUHOSTTEF = $payment['CDNSUHOSTTEF'];
        $CDNSUTEF = !empty($payment['CDNSUTEF']) ? $payment['CDNSUTEF'] : $payment['CDNSUHOSTTEF'];
        $NRCONTROLTEF = $payment['NRCONTROLTEF'];
        $NRCARTBANCO  = substr($payment['NRCARTBANCO'], 0, 10);


        // @toDo: Verificar como os dois campos abaixo são calculados
        $vrmoviVeout = 0;
        $vrcotindout = 0;

        $connection = $this->entityManager->getConnection();
        // Gera código de pagamento
        $cdcontadorMovcaixa = 'MOVCAIXA' . $CDFILIAL . $CDCAIXA . $this->databaseUtil->dateTimeToString($DTABERCAIX);
        $nrsequmovi = $this->util->geraCodigo($connection, $cdcontadorMovcaixa, $NRORG, 1, 10);

        $paymentParams = self::buildPaymentParams(
            $CDFILIAL,
            $CDCAIXA,
            $DTABERCAIX,
            $nrsequmovi,
            $DTVENDA,
            $idtipomovive,
            $vrmovivend,
            $qtparcreceb,
            $vrmoviVeout,
            $vrcotindout,
            $NRSEQVENDA,
            $NRORG,
            $cdtiporece,
            $CDOPERADOR,
            $CDCLIENTE,
            $CDNSUHOSTTEF,
            $NRCONTROLTEF,
            $NRCARTBANCO,
            !empty($payment['TRANSACTIONCODE']) ? $payment['TRANSACTIONCODE'] : null,
            !empty($payment['TRANSACTIONID']) ? $payment['TRANSACTIONID'] : null,
            $CDNSUTEF,
            !empty($payment['NRAUTORIZATEF']) ? $payment['NRAUTORIZATEF'] : null
        );
        $this->insereLinhaLogVenda('Gravando na tabela MOVCAIXA');
        $this->geraLogVenda($paymentParams[0]);
        $this->entityManager->getConnection()->executeQuery("INSERT_MOVCAIXA_SALE", $paymentParams[0], $paymentParams[1]);
        $this->insereLinhaLogVenda('Fim da gravação na MOVCAIXA');
        $this->insereLinhaLogVenda('-----------------------------------------------------');

        return $nrsequmovi;
    }

    private function insertMovclie($CDFILIAL, $CDCAIXA, $DTABERCAIX, $nrseqmovclie, $CDCLIENTE, $CDCONSUMIDOR, $vrmovcli,
        $dtmovclie, $NRSEQVENDA, $nrsequmovi, $cdtiporece, $vrsaldocons, $CDOPERADOR, $NRORG,$vrmovpescons,
        $vrsaldpecons, $vrpesubsidio, $vrtetosubsid){

        $paramsMovclie = self::buildMoviclieParams($CDFILIAL, $CDCAIXA, $DTABERCAIX, $nrseqmovclie, $CDCLIENTE, $CDCONSUMIDOR, $vrmovcli,
        $dtmovclie, $NRSEQVENDA, $nrsequmovi, $cdtiporece, $vrsaldocons, $CDOPERADOR, $NRORG,$vrmovpescons,
        $vrsaldpecons, $vrpesubsidio, $vrtetosubsid);

        $this->entityManager->getConnection()->executeQuery("INSERT_MOVCLIE", $paramsMovclie[0], $paramsMovclie[1]);
    }

    private function insertChangePayment($VRTROCOVEND, $CDFILIAL, $CDCAIXA, $DTABERCAIX, $DTVENDA, $NRSEQVENDA, $NRORG, $CDOPERADOR, $CDCLIENTE){
        $connection = $this->entityManager->getConnection();
        // Gera código de pagamento
        $cdcontadorMovcaixa = 'MOVCAIXA' . $CDFILIAL . $CDCAIXA . $this->databaseUtil->dateTimeToString($DTABERCAIX);
        $nrsequmovi = $this->util->geraCodigo($connection, $cdcontadorMovcaixa, $NRORG, 1, 10);
        $paymentParams = self::buildPaymentParams(
            $CDFILIAL,
            $CDCAIXA,
            $DTABERCAIX,
            $nrsequmovi,
            $DTVENDA,
            'S',
            $VRTROCOVEND['VRMOVIVEND'],
            1,
            0,
            0,
            $NRSEQVENDA,
            $NRORG,
            $VRTROCOVEND['CDTIPORECE'],
            $CDOPERADOR,
            $CDCLIENTE,
            null,
            null,
            null,
            null, // TRANSACTIONCODE
            null, // TRANSACTIONID
            null, // CDNSUTEF
            null //NRAUTORIZATEF
        );

        $this->entityManager->getConnection()->executeQuery("INSERT_MOVCAIXA_SALE", $paymentParams[0], $paymentParams[1]);
    }

    private function getIdtiporece($payment, $NRORG) {
        $cdtiporece = $payment['CDTIPORECE'];
        $idtiporeceParams = array(
            'CDTIPORECE' => $cdtiporece,
            'NRORG' => $NRORG
        );
        $idtiporeceQuery = $this->entityManager->getConnection()->fetchAll("GET_IDTIPORECE", $idtiporeceParams);
        $idtiporeceSAAS = $idtiporeceQuery[0]['IDTIPORECE'];
        return $idtiporeceSAAS;
    }

    private function getProdEstoByNrfeaturegrupo($nrfeaturegrupo) {
        $cdprodesto = null;
        $getProdEstFeatureGruParams = self::buildGetProdEstFeatureGruParams($nrfeaturegrupo);
        $getProdEstFeatureGruResult = $this->entityManager->getConnection()->fetchAll("GET_PRODESTFEATUREGRU", $getProdEstFeatureGruParams);
        if (isset($getProdEstFeatureGruResult[0])) {
            $cdprodesto = $getProdEstFeatureGruResult[0]['CDPRODUTOESTOQUE'];
        }
        return $cdprodesto;
    }

    private function getNrfeatureGrupoByDimensoes($dimensoes) {
        $nrfeaturegrupo = null;
        $qtdimensoes = count($dimensoes);
        $nrprodutodimensaovalorArray = array();
        foreach ($dimensoes as $dimensao) {
            $nrprodutodimensaovalorArray[] = $dimensao['NRPRODUTODIMENSAOVALOR'];
        }
        $getNrfeaturegrupoParams = self::buildGetNrfeaturegrupoParams($nrprodutodimensaovalorArray, $qtdimensoes);

        $params = array($getNrfeaturegrupoParams['NRPRODUTODIMENSAOVALOR'], $getNrfeaturegrupoParams['QTDIMENSOES']);
        $types = array(\Doctrine\DBAL\Connection::PARAM_STR_ARRAY, \PDO::PARAM_INT);
        $dts = $this->entityManager->getConnection()->executeQuery("GET_NRFEATUREGRUPO_BY_DIMENSOES", $params, $types);
        $getNrfeaturegrupoResult = $dts->fetchAll();
        if (isset($getNrfeaturegrupoResult[0])) {
            $nrfeaturegrupo = $getNrfeaturegrupoResult[0]['NRFEATUREGRUPO'];
        }
        return $nrfeaturegrupo;
    }

    private function buscaPARAVEND($CDFILIAL, $NRORG) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'NRORG' => $NRORG
        );
        return $this->entityManager->getConnection()->fetchAssoc("BUSCA_PARAVEND", $params);
    }

    private function montaParametrosDaVenda(
        $CDFILIAL, $CDLOJA, $CDCAIXA, $CDVENDEDOR,
        $DTVENDA, $CDEMPRESA, $SGESTADO, $IDIMPVENDA,
        $DTABERCAIX, $VRDESCVENDA, $CDCLIENTE, $CDCCUSCLIE,
        $IDSITUVENDA, $NRSEQVENDA, $NRORG, $IDTPEMISVEND,
        $CDOPERADOR, $NRINSJURFILI, $CDCONSUMIDOR, $NMCONSVEND,
        $NRINSCRCONS, $DSENDECONSVENDA, $IDORIGEMVENDA, $CDSENHAPED,
        $NRVENDAREST, $NRCOMANDA, $NRMESA, $VRTXSEVENDA,
        $NRPESMESAVENDA, $DSCOMANDA, $VRTOTVENDA, $DTHRMESAFECHVEN,
        $DSOBSDESC, $CDGRPOCORDESC, $DSOBSFINVEN,
        $parametrosVendaDlv, $VRREPIQUEVENDA, $NRCPFFID4ALL, $CODPROMO,
        $FIDELITYDISCOUNT, $cashbackUsed4ALL, $NRCPFFIDUDS, $VRDESCCASHUDS,
        $IDPEDINTEPROMO, $NRCPFINTEPROMO, $DSEMAILENVCUPOM, $IDSITENVCUPOM, $CDSUPERVISOR){

        $dateTime = new \DateTime();

        if (!empty($CODPROMO)) $CDPROMOFID4ALL = $CODPROMO['id'];
        else $CDPROMOFID4ALL = null;

        $ENDERECO = $DSENDECONSVENDA;
        if (empty($parametrosVendaDlv)) $DTABERVENDA = $dateTime;
        else {
            if ($this->databaseUtil->databaseIsOracle()){
                $DTABERVENDA = \DateTime::createFromFormat('Y-m-d H:i:s', $parametrosVendaDlv['DTHRABERMESA']);
            }
            else {
                $DTABERVENDA = \DateTime::createFromFormat('Y-m-d H:i:s.u', $parametrosVendaDlv['DTHRABERMESA']);
            }

            $ENDERECO = isset($parametrosVendaDlv['DSENDECONSCOMAN']) ? $parametrosVendaDlv['DSENDECONSCOMAN'] : $DSENDECONSVENDA;
        }

        $parametrosDaVenda = array(
            'CDFILIAL'         => $CDFILIAL,
            'CDLOJA'           => $CDLOJA,
            'CDCAIXA'          => $CDCAIXA,
            'DTENTRVENDA'      => $DTABERCAIX,
            'DTVENDA'          => $DTVENDA,
            'DTABERVENDA'      => $DTABERVENDA,
            'DTFECHAVENDA'     => $dateTime,
            'CDEMPRESA'        => $CDEMPRESA,
            'SGESTADO'         => $SGESTADO,
            'IDIMPVENDA'       => $IDIMPVENDA,
            'DTABERTUR'        => $DTABERCAIX,
            'CDFILIALTUR'      => $CDFILIAL,
            'CDCAIXATUR'       => $CDCAIXA,
            'VRDESCVENDA'      => $VRDESCVENDA,
            'CDCLIENTE'        => $CDCLIENTE,
            'CDCCUSCLIE'       => $CDCCUSCLIE,
            'IDSITUVENDA'      => $IDSITUVENDA,
            'NRSEQVENDA'       => $NRSEQVENDA,
            'NRORG'            => $NRORG,
            'IDTPEMISVEND'     => $IDTPEMISVEND,
            'CDOPERADOR'       => $CDOPERADOR,
            'NRINSJURIEST'     => $NRINSJURFILI,
            'CDCONSUMIDOR'     => $CDCONSUMIDOR,
            'NMCONSVEND'       => $NMCONSVEND,
            'NRINSCRCONS'      => $NRINSCRCONS,
            'IDORIGEMVENDA'    => $IDORIGEMVENDA,
            'DTHRREALCONCONS'  => $dateTime,
            'CDPREFINTGCOLVEN' => 'TEK',
            'IDTIPOVENDA'      => 'D',
            'IDVENDEBCONS'     => 'N',
            'NRPESMESAVENDA'   => $NRPESMESAVENDA,
            'CDSENHAPED'       => $CDSENHAPED,
            'CDVENDEDOR'       => $CDVENDEDOR,
            'NRVENDAREST'      => $NRVENDAREST,
            'NRCOMANDAVND'     => strlen($NRCOMANDA) > 0 ? $NRCOMANDA: null,
            'NRMESA'           => $NRMESA,
            'VRTXSEVENDA'      => !empty($VRTXSEVENDA) ? $VRTXSEVENDA : 0,
            'DSCOMANDAVND'     => !empty($DSCOMANDA) ? $DSCOMANDA : null,
            'VRTOTVENDA'       => $VRTOTVENDA,
            'DTHRMESAFECHVEN'  => $this->databaseUtil->convertToDateDB($DTHRMESAFECHVEN),
            'DSOBSDESC'        => !empty($DSOBSDESC) ? substr($DSOBSDESC, 0, 100) : null,
            'CDGRPOCORDESC'    => $CDGRPOCORDESC,
            'DSOBSFINVEN'      => $DSOBSFINVEN,
            'DSBAIRRO'         => isset($parametrosVendaDlv['DSBAIRRO'])? $parametrosVendaDlv['DSBAIRRO']:null,
            'IDRETBALLOJA'     => isset($parametrosVendaDlv['IDRETBALLOJA'])? $parametrosVendaDlv['IDRETBALLOJA']:null,
            'DSAREAATENDVEN'   => isset($parametrosVendaDlv['DSAREAATEND'])? $parametrosVendaDlv['DSAREAATEND']:null,
            'NRCEPCONSVENDA'   => isset($parametrosVendaDlv['NRCEPCONSCOMAND'])? $parametrosVendaDlv['NRCEPCONSCOMAND']:null,
            'DSCOMPLENDCOCOM'  => isset($parametrosVendaDlv['DSCOMPLENDCOCOM'])? $parametrosVendaDlv['DSCOMPLENDCOCOM']:null,
            'CDBAIRRO'         => isset($parametrosVendaDlv['CDBAIRRO'])? $parametrosVendaDlv['CDBAIRRO']:null,
            'CDMUNICIPIO'      => isset($parametrosVendaDlv['CDMUNICIPIO'])? $parametrosVendaDlv['CDMUNICIPIO']:null,
            'DSENDECONSVENDA'  => $ENDERECO,
            'VRREPIQUEVENDA'   => $VRREPIQUEVENDA,
            'NRCPFFID4ALL'     => $NRCPFFID4ALL,
            'CDPROMOFID4ALL'   => $CDPROMOFID4ALL,
            'VRDESCPROMO4ALL'  => $FIDELITYDISCOUNT,
            'VRDESCCASH4ALL'   => $cashbackUsed4ALL,
            'NRCPFFIDUDS'      => $NRCPFFIDUDS,
            'VRDESCCASHUDS'    => $VRDESCCASHUDS,
			'IDPEDINTEPROMO'   => $IDPEDINTEPROMO,
            'NRCPFINTEPROMO'   => $NRCPFINTEPROMO,
            'DSEMAILENVCUPOM'  => $DSEMAILENVCUPOM,
			'IDSITENVCUPOM'    => $IDSITENVCUPOM,
            'CDSUPERVISOR'     => !empty($CDSUPERVISOR) ? $CDSUPERVISOR : null
        );

        $type = array(
            'DTENTRVENDA' => \Doctrine\DBAL\Types\Type::DATE,
            'DTVENDA' => \Doctrine\DBAL\Types\Type::DATETIME,
            'DTABERVENDA' => \Doctrine\DBAL\Types\Type::DATETIME,
            'DTFECHAVENDA' => \Doctrine\DBAL\Types\Type::DATETIME,
            'DTABERTUR' => \Doctrine\DBAL\Types\Type::DATETIME,
            'DTHRREALCONCONS' => \Doctrine\DBAL\Types\Type::DATETIME,
            'DTHRMESAFECHVEN' => \Doctrine\DBAL\Types\Type::DATETIME
        );

        return array($parametrosDaVenda, $type);
    }

    private function buildSaleItemParams(
            $CDFILIAL, $CDLOJA, $CDCAIXA, $QTPRODVEND, $VRUNITVEND, $VRUNITVENDCL,
            $VRDESITVEND, $CDPRODUTO, $NRORG, $NRSEQVENDA, $NRSEQUITVEND,
            $VRACRITVEND, $NRORDITCUPFIS, $VRPERCVEND, $VRALIQIMPR,
            $NRATRAPRODITVE, $VRRATTXSERV, $VRRATDESCVEN, $DTHRPEDIDO,
            $CDPRODPROMOCAO, $NRSEQPRODCOMIT, $CDVENDEDOR,
            $DSOBSITEMVENDA, $DSOBSPEDDIGITA, $CDCLASFISC, $DSOBSDESCIT,
            $CDGRPOCORDESCIT, $IDORIGEMVENDA, $CDCBENEFIT, $NRLUGARMESA,
            $CDCAMPCOMPGANHE, $DTINIVGCAMPCG, $IDIMPPRGRPROMO, $IDOBSERVACAOAD, $CDSUPERVDESC
        ){
        $dadosProdutoLoja = $this->getInfoTipoItemvenda($CDFILIAL, $CDLOJA);
        //var_dump($DTINIVGCAMPCG);die;
        $saleItemParams = array(
            'CDFILIAL'       => $CDFILIAL,
            'CDLOJA'         => $CDLOJA,
            'CDCAIXA'        => $CDCAIXA,
            'QTPRODVEND'     => $QTPRODVEND,
            'VRUNITVEND'     => $VRUNITVEND,
            'VRDESITVEND'    => ($VRDESITVEND > $VRUNITVEND*$QTPRODVEND) ? $VRUNITVEND*$QTPRODVEND : $VRDESITVEND,
            'CDPRODUTO'      => $CDPRODUTO,
            'NRORG'          => $NRORG,
            'NRSEQVENDA'     => $NRSEQVENDA,
            'NRSEQUITVEND'   => $NRSEQUITVEND,
            'VRACRITVEND'    => $VRACRITVEND,
            'VRUNITVENDCL'   => $VRUNITVENDCL,
            'NRORDITCUPFIS'  => $NRORDITCUPFIS,
            'VRPERCVEND'     => $VRPERCVEND,
            'VRALIQIMPR'     => $VRALIQIMPR,
            'NRATRAPRODITVE' => $NRATRAPRODITVE,
            'VRRATTXSERV'    => $VRRATTXSERV,
            'VRRATDESCVEN'   => $VRRATDESCVEN,
            'DTHRPEDIDO'     => new \DateTime($DTHRPEDIDO),
            'CDCAIXAPEDVEN'  => $CDCAIXA,
            'CDPRODPROMOCAO' => $CDPRODPROMOCAO,
            'NRSEQPRODCOMIT' => $NRSEQPRODCOMIT,
            'CDVENDEDOR'     => $CDVENDEDOR,
            'IDORIGPEDVEN'   => 'MOB', // @TODO - MELHORAR ISSO
            'DSOBSITEMVENDA' => $DSOBSITEMVENDA,
            'DSOBSPEDDIGITA' => $DSOBSPEDDIGITA,
            'QTPRODREFIL'    => 0,
            'CDCLASFISC'     => $CDCLASFISC,
            'DSOBSDESCIT'    => !empty($DSOBSDESCIT) ? substr($DSOBSDESCIT, 0, 100) : null,
            'CDGRPOCORDESCIT' => $CDGRPOCORDESCIT,
            'IDORIGEMVENDA'  => $IDORIGEMVENDA,
            'CDCBENEFIT'     => $CDCBENEFIT,
            'NRPESMESAVENIT' => $NRLUGARMESA,
            'CDCAMPCOMPGANHE' => $CDCAMPCOMPGANHE,
            'DTINIVGCAMPCG' => !is_string($DTINIVGCAMPCG) ? $DTINIVGCAMPCG : new \DateTime($DTINIVGCAMPCG),
            'IDIMPPRGRPROMO' => $IDIMPPRGRPROMO,
            'IDOBSERVACAOAD' => $IDOBSERVACAOAD,
            'IDPRODTAXASERV' => $this->getTipoItemvenda($dadosProdutoLoja, $CDPRODUTO),
            'CDSUPERVDESC'   => !empty($CDSUPERVDESC) ? $CDSUPERVDESC : null
        );
        $type = array(
            'DTHRPEDIDO' => \Doctrine\DBAL\Types\Type::DATETIME,
            'DTINIVGCAMPCG' => \Doctrine\DBAL\Types\Type::DATETIME
        );

        return array($saleItemParams, $type);
    }

    private function buildSaleItemCancParams($CDFILIAL, $CDCAIXA, $QTPRODVENDC, $VRUNITVENDC, $VRUNITVENCCL, $VRDESITVENDC, $CDPRODUTO, $NRORG, $NRSEQVENDA, $NRSEQITVENDC, $VRACRITVENDC, $NRORDITCUPFISCA, $cdgrpocorcanite, $dsobscanite, $cdsupervisor, $IDPRODPRODUZC, $DTHRPRODCAN, $CDOPERCANC) {
        $saleItemCancParams = array(
            'CDFILIAL'        => $CDFILIAL,
            'CDCAIXA'         => $CDCAIXA,
            'QTPRODVENDC'     => $QTPRODVENDC,
            'VRUNITVENDC'     => $VRUNITVENDC,
            'VRDESITVENDC'    => $VRDESITVENDC,
            'CDPRODUTO'       => $CDPRODUTO,
            'NRORG'           => $NRORG,
            'NRSEQVENDA'      => $NRSEQVENDA,
            'NRSEQITVENDC'    => $NRSEQITVENDC,
            'VRACRITVENDC'    => $VRACRITVENDC,
            'NRORGINCLUSAO'   => $NRORG,
            'VRUNITVENCLC'    => $VRUNITVENCCL,
            'NRORDITCUPFISCA' => $NRORDITCUPFISCA,
            'CDGRPOCORCANITE' => $cdgrpocorcanite,
            'DSOBSCANITE'     => $dsobscanite,
            'CDSUPERVISOR'    => $cdsupervisor,
            'IDPRODPRODUZC'   => $IDPRODPRODUZC,
            'CDOPERCANC'      => $CDOPERCANC,
            'DTHRPRODCAN'     => $this->databaseUtil->convertToDateDB($DTHRPRODCAN)
        );

        $type = array(
            'DTHRPRODCAN' => \Doctrine\DBAL\Types\Type::DATETIME
        );
        return array($saleItemCancParams, $type);
    }

    private function buildMoviclieParams($CDFILIAL, $CDCAIXA, $DTABERCAIX, $NRSEQMOVCLI, $CDCLIENTE, $CDCONSUMIDOR, $VRMOVCLI,
        $DTMOVCLI, $NRSEQVENDA, $NRSEQUMOVI, $CDTIPORECE, $VRSALDOCONS, $CDOPERADOR, $NRORG, $VRMOVPESCONS,
        $VRSALDPECONS, $VRPESUBSIDIO, $VRTETOSUBSID) {

        $movclieParams = array(
            'CDFILIAL'       => $CDFILIAL,
            'CDCAIXA'        => $CDCAIXA,
            'DTABERCAIX'     => $DTABERCAIX,
            'NRSEQMOVCLI'    => $NRSEQMOVCLI,
            'CDCLIENTE'      => $CDCLIENTE,
            'CDCONSUMIDOR'   => $CDCONSUMIDOR,
            'VRMOVCLI'       => $VRMOVCLI,
            'DTMOVCLI'       => $this->databaseUtil->convertToDateTime($DTMOVCLI),
            'DTMOVIMCLIE'    => $this->databaseUtil->convertToDate($DTMOVCLI),
            'IDTIPMOCVLI'    => 'D',
            'NRSEQVENDA'     => $NRSEQVENDA,
            'NRSEQUMOVI'     => $NRSEQUMOVI,
            'CDTIPORECE'     => $CDTIPORECE,
            'VRSALDOCONS'    => $VRSALDOCONS,
            'CDOPERADOR'     => $CDOPERADOR,
            'NRORG'          => $NRORG,
            'DTINCLUSAO'     => $DTMOVCLI,
            'NRORGINCLUSAO'  => $NRORG,
            'CDOPERINCLUSAO' => $CDOPERADOR,
            'NRORGULTATU'    => $NRORG,
            'VRMOVPESCONS'   => $VRMOVPESCONS,
            'VRSALDPECONS'   => $VRSALDPECONS,
            'VRPESUBSIDIO'   => $VRPESUBSIDIO,
            'VRTETOSUBSID'   => $VRTETOSUBSID,
            'CDOPERULTATU'   => $CDOPERADOR,
            'DTINCLUSAO'     => $this->databaseUtil->convertToDate($this->databaseUtil->getCurrentDate())
        );

        $type = array(
            'DTABERCAIX' => \Doctrine\DBAL\Types\Type::DATETIME,
            'DTMOVCLI' => \Doctrine\DBAL\Types\Type::DATETIME,
            'DTMOVIMCLIE' => \Doctrine\DBAL\Types\Type::DATE,
            'DTINCLUSAO' => \Doctrine\DBAL\Types\Type::DATE
        );
        return array($movclieParams, $type);
    }

    private function buildPaymentParams($CDFILIAL, $CDCAIXA, $DTABERCAIX, $NRSEQUMOVI, $DTVENDA, $IDTIPOMOVIVE, $VRMOVIVEND, $QTPARCRECEB, $VRMOVIVEOUT, $VRCOTINDOUT, $NRSEQVENDA, $NRORG, $CDTIPORECE, $CDOPERADOR, $CDCLIENTE, $CDNSUHOSTTEF, $NRCONTROLTEF, $NRCARTBANCO, $TRANSACTIONCODE, $TRANSACTIONID, $CDNSUTEF, $NRAUTORIZATEF) {
        $paymentParams = array(
            'CDFILIAL'        => $CDFILIAL,
            'CDCAIXA'         => $CDCAIXA,
            'DTABERCAIX'      => $DTABERCAIX,
            'NRSEQUMOVI'      => $NRSEQUMOVI,
            'DTHRINCMOV'      => $DTVENDA,
            'IDTIPOMOVIVE'    => $IDTIPOMOVIVE,
            'VRMOVIVEND'      => $VRMOVIVEND,
            'QTPARCRECEB'     => $QTPARCRECEB,
            'VRMOVIVEOUT'     => $VRMOVIVEOUT,
            'VRCOTINDOUT'     => $VRCOTINDOUT,
            'NRSEQVENDA'      => $NRSEQVENDA,
            'NRORG'           => $NRORG,
            'CDTIPORECE'      => $CDTIPORECE,
            'DTMOVIMCAIXA'    => $DTABERCAIX,
            'CDOPERINCLUSAO'  => $CDOPERADOR,
            'CDCLIENTE'       => $CDCLIENTE,
            'NRORGINCLUSAO'   => $NRORG,
            'CDNSUHOSTTEF'    => $CDNSUHOSTTEF,
            'NRCONTROLTEF'    => $NRCONTROLTEF,
            'NRCARTBANCO'     => $NRCARTBANCO,
            'TRANSACTIONCODE' => $TRANSACTIONCODE,
            'TRANSACTIONID'   => $TRANSACTIONID,
            'CDNSUTEF'        => $CDNSUTEF,
            'NRAUTORIZATEF'   => $NRAUTORIZATEF
        );
        $type = array(
            'DTABERCAIX' => \Doctrine\DBAL\Types\Type::DATETIME,
            'DTHRINCMOV' => \Doctrine\DBAL\Types\Type::DATETIME,
            'DTMOVIMCAIXA' => \Doctrine\DBAL\Types\Type::DATE
        );

        return array($paymentParams, $type);
    }

    private function buildAuxSaleItemParams($CDFILIAL, $CDCAIXA, $nritemvendauxiliar, $CDFILIALalmoxarifado, $cdalmoxarifado, $dataconclusaoabastecimento, $nrtipocombustivel, $nrbomba, $nrbico, $NRORG, $NRSEQVENDA, $nrsequitvend, $vrencerranteinicial, $vrencerrantefinal) {
        $saleItemAuxParams = array(
            'CDFILIAL'                   => $CDFILIAL,
            'CDCAIXA'                    => $CDCAIXA,
            'NRITEMVENDAUXILIAR'         => $nritemvendauxiliar,
            'CDFILIALALMOXARIFADO'       => $CDFILIALalmoxarifado,
            'CDALMOXARIFADO'             => $cdalmoxarifado,
            'DATACONCLUSAOABASTECIMENTO' => $this->databaseUtil->convertToDateTime($dataconclusaoabastecimento),
            'NRTIPOCOMBUSTIVEL'          => $nrtipocombustivel,
            'NRBOMBA'                    => $nrbomba,
            'NRBICO'                     => $nrbico,
            'NRORG'                      => $NRORG,
            'NRSEQVENDA'                 => $NRSEQVENDA,
            'NRSEQUITVEND'               => $nrsequitvend,
            'VRENCERRANTEINICIAL'        => $vrencerranteinicial,
            'VRENCERRANTEFINAL'          => $vrencerrantefinal
        );
        return $saleItemAuxParams;
    }

    private function buildGetNrfeaturegrupoParams($nrprodutodimensaovalorArray, $qtdimensoes) {
        $getNrfeaturegrupoParams = array(
            'NRPRODUTODIMENSAOVALOR' => $nrprodutodimensaovalorArray,
            'QTDIMENSOES'            => $qtdimensoes
        );
        return $getNrfeaturegrupoParams;
    }

    private function buildGetProdEstFeatureGruParams($nrfeaturegrupo) {
        $getProdEstFeatureGruParams = array(
            'NRFEATUREGRUPO' => $nrfeaturegrupo
        );
        return $getProdEstFeatureGruParams;
    }

    private function buildAuxSaleItemFeatureParams($nritemvendauxiliar, $CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $nrsequitvend, $nrfeaturegrupo, $CDOPERADOR) {
        $auxSaleItemFeatureParams = array(
            'NRITEMVENDAUXILIAR' => $nritemvendauxiliar,
            'CDFILIAL'           => $CDFILIAL,
            'CDCAIXA'            => $CDCAIXA,
            'NRORG'              => $NRORG,
            'NRSEQVENDA'         => $NRSEQVENDA,
            'NRSEQUITVEND'       => $nrsequitvend,
            'NRFEATUREGRUPO'     => $nrfeaturegrupo,
            'NRORGINCLUSAO'      => $NRORG,
            'NRORGULTATU'        => $NRORG,
            'CDOPERINCLUSAO'     => $CDOPERADOR,
            'CDOPERULTATU'       => $CDOPERADOR
        );
        return $auxSaleItemFeatureParams;
    }

    private function updateIntegracaomov($nrseqmov, $NRSEQVENDA, $CDFILIAL, $CDCAIXA, $NRORG) {
        $params = self::buildIntegracaomovParams($nrseqmov, $NRSEQVENDA, $CDFILIAL, $CDCAIXA, $NRORG);
        $this->entityManager->getConnection()->executeQuery("UPDATE_NRSEQVENDA_VND_INTEGRACAOMOV", $params);
    }

    private function buildIntegracaomovParams($nrseqmov, $NRSEQVENDA, $CDFILIAL, $CDCAIXA, $NRORG) {
        return array(
            'NRSEQMOV'   => $nrseqmov,
            'CDFILIAL'   => $CDFILIAL,
            'CDCAIXA'    => $CDCAIXA,
            'NRSEQVENDA' => $NRSEQVENDA,
            'NRORG'      => $NRORG
        );
    }

    private function buscaClientePadrao($CDFILIAL, $NRORG){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'NRORG' => $NRORG
        );
        return $this->entityManager->getConnection()->fetchAssoc("GET_CLIENTEPADRAO", $params);
    }

    private function calculaValorDaVenda($items) {
        $produtosCobrados = self::separaItensQueSeraoCobrados($items);
        $total = 0;
        $subsidy = 0;
        $discounts = 0;
        foreach ($produtosCobrados as $item) {
            $totalItem = self::calculaTotalItem($item);
            $total = $total + $totalItem;
            $subsidyCalc = self::calculateSubsidy($item['VRUNITVEND'], $item['VRUNITVENDCL'], bcdiv(str_replace(',','.',strval($item['VRDESITVEND'])), str_replace(',','.',strval($item['QTPRODVEND'])), '2'));
            $subsidy += floatval(bcmul(str_replace(',','.',strval($item['QTPRODVEND'])), str_replace(',','.',strval($subsidyCalc)), '2'));
            $discounts = floatval(bcadd(strval($discounts), strval($item['VRDESITVEND']), 2));
        }
        $produtosCancelados = self::separaItensCancelados($items);
        $totalCancelamento = 0;
        foreach ($produtosCancelados as $item) {
            $totalCancelamento += self::calculaTotalItem($item);
        }

        return array(
            'total' => $total,
            'subsidy' => $subsidy,
            'totalCancelamento' => $totalCancelamento,
            'discounts' => $discounts
        );
    }

    private function calculateSubsidy($VRPRECITEM, $VRPRECITEMCL, $VRDESITVEND){
        if ($VRPRECITEMCL > 0 && $VRDESITVEND > 0){

            $finalPrice = floatval(bcsub(str_replace(',','.',strval($VRPRECITEM + $VRPRECITEMCL)), str_replace(',','.',strval($VRDESITVEND)), '2'));
            if ($finalPrice <= 0.01) return 0;

            $totalPrice = $VRPRECITEM + $VRPRECITEMCL;
            $discount = ($VRPRECITEMCL / $totalPrice) * $VRDESITVEND;

            return floatval(bcsub(str_replace(',','.',strval($VRPRECITEMCL)), str_replace(',','.',strval($discount)), '2'));
        }
        else return $VRPRECITEMCL;
    }

    public function cancelaCupom($CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $CDOPERADOR, $CDSUPERVISOR, $DSOBSCANCUP = '', $CDCAIXA_CUPOM = null){
        $NRACESSOCANC = null;
		if($CDCAIXA_CUPOM == null){
			$CDCAIXA_CUPOM=$CDCAIXA;
		}
        $params = array(
            'CDFILIAL'   => $CDFILIAL,
            'CDCAIXA'    => $CDCAIXA_CUPOM,
            'NRORG'      => $NRORG,
            'NRSEQVENDA' => $NRSEQVENDA
        );
        $venda = $this->entityManager->getConnection()->fetchAssoc("GET_SALE_PARAMETERS", $params);

        if ($venda){
            $IDTPEMISVEND = $this->vendaNormalizacao->normalizaTipoEmissao($CDFILIAL, $CDCAIXA, $NRORG);
            if ($IDTPEMISVEND == self::TIPO_SAT){
                $cancelaCupom = $this->sat->cancelaVendaSAT($CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $venda);
                $NRACESSOCANC = $cancelaCupom['NRACESSOCANC'];
            } else {
                if ($venda['IDSTATUSNFCE'] == 'A'){
                    $cancelaCupom = $this->notaFiscalService->cancelaVendaNFCE($CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $venda, 'Venda Cancelada no Sistema.', $CDOPERADOR);
                } else {
                    $cancelaCupom = array(
                        'error' => false,
                        'message' => '',
                        'mensagemNfce' => 'A venda será cancelada apenas no sistema. Para cancelamento na SEFAZ, realize a rotina pelo Gestão de Vendas.',
                        'dadosImpressao' => array()
                    );
                }
            }

            if (!$cancelaCupom['error']){
                $IDSTATUSNFCE = 'C';
                if ($venda['IDSTATUSNFCE'] == 'P'){
                    $IDSTATUSNFCE = 'R';
                }
                $this->cancelaVenda($CDFILIAL, $CDCAIXA_CUPOM, $NRORG, $NRSEQVENDA, $CDOPERADOR, $IDSTATUSNFCE, $CDSUPERVISOR, false, $DSOBSCANCUP, $NRACESSOCANC);
            }
            $result = $cancelaCupom;
        }
        else {
            $result = array(
                'error'   => true,
                'message' => 'Venda não encontrada.'
            );
        }

        return $result;
    }

    public function createBalance($CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $VRSALDCONFAM){
        $params = array(
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD,
            ':VRSALDCONFAM' => $VRSALDCONFAM
        );
        $this->entityManager->getConnection()->executeQuery("CREATE_BALANCE", $params);
    }

    public function updateBalance($CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $VRSALDCONFAM){
        $params = array(
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR,
            'CDFAMILISALD' => $CDFAMILISALD
        );
        $balance = $this->entityManager->getConnection()->fetchAll("CHECK_BALANCE", $params);
        if (empty($balance)){
            $params = array(
                'CDCLIENTE' => $CDCLIENTE,
                'CDCONSUMIDOR' => $CDCONSUMIDOR,
                'CDFAMILISALD' => $CDFAMILISALD,
                'VRSALDCONFAM' => $VRSALDCONFAM
            );
            $this->entityManager->getConnection()->executeQuery("CREATE_BALANCE", $params);
        } else {
            $params = array(
                ':CDCLIENTE' => $CDCLIENTE,
                ':CDCONSUMIDOR' => $CDCONSUMIDOR,
                ':CDFAMILISALD' => $CDFAMILISALD,
                ':VRSALDCONFAM' => $VRSALDCONFAM
            );
            $this->entityManager->getConnection()->executeQuery("UPDATE_BALANCE", $params);
        }
    }

    public function permiteSaldoNegativo($CDFILIAL, $CDFAMILISALD){
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':CDFAMILISALD' => $CDFAMILISALD
        );

        $result = $this->entityManager->getConnection()->fetchAssoc("GET_PERMITE_SALDO_NEGATIVO", $params);
        return $result;
    }

    public function insertExtratoCons($CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $DSOPEEXTCONS, $VRMOVEXTCONS, $IDTPMOVEXT, $VRSALDCONEXT, $NRSEQMOVEXT,
                                      $CDTIPORECE = null, $CDFILIAL = null, $CDCAIXA = null, $NRSEQVENDA = null, $DTABERCAIX, $NRSEQMOVCAIXA, $dateReference){
        $params = array(
            ':CDCLIENTE'    => $CDCLIENTE,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDFAMILISALD' => $CDFAMILISALD,
            ':DSOPEEXTCONS' => $DSOPEEXTCONS,
            ':VRMOVEXTCONS' => $VRMOVEXTCONS,
            ':IDTPMOVEXT'   => $IDTPMOVEXT,
            ':VRSALDCONEXT' => $VRSALDCONEXT,
            ':CDTIPORECE'   => $CDTIPORECE,
            ':CDFILIAL'     => $CDFILIAL,
            ':CDCAIXA'      => $CDCAIXA,
            ':NRSEQVENDA'   => $NRSEQVENDA,
            ':NRSEQMOVCAIXA'   => $NRSEQMOVCAIXA,
            ':NRDEPOSICONS'   => null,
            ':NRSEQMOVEXT'   => str_pad($NRSEQMOVEXT, 3, '0', STR_PAD_LEFT),
            ':DTABERCAIX'   => $DTABERCAIX,
            ':DTMOVEXTCONS' => $dateReference
        );
        $type = array(
            ':DTABERCAIX' => \Doctrine\DBAL\Types\Type::DATETIME,
            ':DTMOVEXTCONS' => \Doctrine\DBAL\Types\Type::DATETIME
        );
        $this->insereLinhaLogVenda('Gravando na tabela EXTRATOCONS');
        $this->geraLogVenda($params);
        $this->entityManager->getConnection()->executeQuery("INSERT_EXTRATOCONS", $params, $type);
        $this->insereLinhaLogVenda('Fim da gravação na EXTRATOCONS');
        $this->insereLinhaLogVenda('-----------------------------------------------------');
    }

    public function alteraTiporecePorBandeira($pagamentoAtual, $CDFILIAL, $NRCONFTELA, $DTINIVIGENCIA, $NRORG){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'NRCONFTELA' => $NRCONFTELA,
            'DTINIVIGENCIA' => $DTINIVIGENCIA,
            'NRORG' => $NRORG,
            'IDTIPORECE' => $pagamentoAtual['IDTIPORECE'],
            'CDBANCARTCR' => $pagamentoAtual['CDBANCARTCR']
        );
        $types = array(
            ':DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME
        );
        $tiporece = $this->entityManager->getConnection()->fetchAssoc("GET_CDTIPORECE_BY_BANCART", $params, $types);

        return $tiporece ? $tiporece['CDTIPORECE'] : $pagamentoAtual['CDTIPORECE'];
    }

    public function montaItemsDesistencia($CDFILIAL, $NRVENDAREST){
        $params = array(
            'NRVENDAREST' => $NRVENDAREST,
            'CDFILIAL' => $CDFILIAL
        );
        $ITVENDADES = $this->entityManager->getConnection()->fetchAll("SELECT_ITCOMANDAVENDES", $params);
        return $ITVENDADES;
    }

    public function insertItemsDesistencia($itensDesistencia, $NRSEQVENDA, $CDFILIAL, $CDLOJA, $CDCAIXA, $NRORG, $CDOPERADOR, $connection) {
        $chave = $CDFILIAL.$CDCAIXA.$NRSEQVENDA;

        foreach ($itensDesistencia as $itemDesistencia) {
            $NRSEQITVENDADES = $this->util->geraCodigo($connection, $chave, $NRORG, 1, 3);
            $params = array(
                'CDFILIAL'          => $CDFILIAL,
                'CDCAIXA'           => $CDCAIXA,
                'NRORG'             => $NRORG,
                'NRSEQVENDA'        => $NRSEQVENDA,
                'NRSEQITVENDADES'   => $NRSEQITVENDADES,
                'QTPRODVENDADES'    => $itemDesistencia['QTPRODITCOMVENDES'],
                'CDPRODUTO'         => $itemDesistencia['CDPRODUTO'],
                'VRPRECCOMVEN'      => $itemDesistencia['VRPRECCOMVEN'],
                'VRDESCCOMVEN'      => $itemDesistencia['VRDESCCOMVEN'],
                'VRACRCOMVEN'       => $itemDesistencia['VRACRCOMVEN']
            );

            $this->entityManager->getConnection()->executeQuery("INSERT_ITVENDADES", $params);
            // Trata dos acréscimos que contém preço.
            if (array_key_exists('CDOCORR', $itemDesistencia)) {
                if (!empty($itemDesistencia['CDOCORR'])) {
                    foreach ($itemDesistencia['CDOCORR'] as $CDOCORR) {
                        $params = array(
                            $CDFILIAL,
                            $CDLOJA,
                            $CDOCORR
                        );
                        $obs_type = $this->entityManager->getConnection()->fetchAssoc("GET_OBSERVATION_TYPE", $params);
                        if ($obs_type['IDCONTROLAOBS'] === 'A' && !empty($obs_type['CDPRODUTO'])) {
                            // Busca o preço do acréscimo.
                            $r_retornaPreco = $this->precoService->buscaPreco($CDFILIAL, null, $obs_type['CDPRODUTO'], $CDLOJA, null);
                            $preco     = floatval($r_retornaPreco['PRECO']);
                            $desconto  = floatval($r_retornaPreco['DESC']);
                            $acrescimo = floatval($r_retornaPreco['ACRE']);

                            // Valida o preço.
                            if (!empty($preco) && $preco > 0) {
                                $NRSEQITVENDADES = $this->util->geraCodigo($connection, $chave, $NRORG, 1, 3);
                                $params = array(
                                    'CDFILIAL'          => $CDFILIAL,
                                    'CDCAIXA'           => $CDCAIXA,
                                    'NRORG'             => $NRORG,
                                    'NRSEQVENDA'        => $NRSEQVENDA,
                                    'NRSEQITVENDADES'   => $NRSEQITVENDADES,
                                    'QTPRODVENDADES'    => $itemDesistencia['QTPRODITCOMVENDES'],
                                    'CDPRODUTO'         => $obs_type['CDPRODUTO'],
                                    'VRPRECCOMVEN'      => $preco,
                                    'VRDESCCOMVEN'      => $desconto,
                                    'VRACRCOMVEN'       => $acrescimo
                                );
                                $this->entityManager->getConnection()->executeQuery("INSERT_ITVENDADES", $params);
                            }
                        }
                    }
                }
            }
        }
    }

    public function getPagamentoDlv($CDFILIAL, $NRVENDAREST, $pagamentoAtual){
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'NRVENDAREST' => $NRVENDAREST
        );
        $pagamentoDlv = $this->entityManager->getConnection()->fetchAssoc("GET_PAYMENT_PARAM_DLV", $params);
        return $pagamentoAtual + $pagamentoDlv;
    }

    public function montaParametrosVendaDlv($CDFILIAL, $NRVENDAREST, $NRCOMANDA){
        $params = array(
            'CDFILIAL'      => $CDFILIAL,
            'NRVENDAREST'   => $NRVENDAREST,
            'NRCOMANDA'     => $NRCOMANDA
        );
        $paramsDlv = $this->entityManager->getConnection()->fetchAssoc("GET_PARAMS_DELIVERY_SALE", $params);
        return $paramsDlv;
    }

    private function geraLogVenda($params){
        if ($this->habilitaLog){
            $horaAtual = new \DateTime();
            $horaAtual = $horaAtual->format('d/m/Y H:i:s');

            foreach ($params as $key => $value){
                try {
                    if (gettype($value) == "object"){
                        if (get_class($value) == "DateTime"){
                            $value = $value->format('d/m/Y H:i:s');
                        }
                    }
                    file_put_contents($this->logVendaPath, $horaAtual . ' - ' . $key . ': ' . $value . "\r\n", FILE_APPEND);
                } catch(\Exception $e){
                    throw new \Exception($e->getMessage());
                }
            }
        }
    }

    private function insereLinhaLogVenda($texto){
        if ($this->habilitaLog){
            $horaAtual = new \DateTime();
            $horaAtual = $horaAtual->format('d/m/Y H:i:s');

            file_put_contents($this->logVendaPath, $horaAtual . ' - ' . $texto . "\r\n", FILE_APPEND);
        }
    }

    private function getLogHeader($CDFILIAL, $CDLOJA, $CDCAIXA, $CDOPERADOR, $DTVENDA, $VRMOVIVEND) {
        $VRMOVIVEND = number_format($VRMOVIVEND, 2, ',', '' );
        $logHeaderUp = '| LOG DE VENDA - Filial: ' . $CDFILIAL . ' - Loja: ' . $CDLOJA . ' - Caixa: ' . $CDCAIXA . ' - Operador: ' . $CDOPERADOR . ' |';
        $logHeaderDown = '| Valor: R$' . $VRMOVIVEND . ' - Data: ' . $DTVENDA->format('Y-m-d H:i:s');
        $logHeaderDown = str_pad($logHeaderDown, strlen($logHeaderUp) - 1, " ", STR_PAD_RIGHT) . '|';
        $logHeaderBox = "" . str_pad(null, strlen($logHeaderUp), "-", STR_PAD_LEFT);
        $this->util->geraLog("\r\n".$logHeaderBox."\r\n".$logHeaderUp."\r\n".$logHeaderDown."\r\n".$logHeaderBox, $CDFILIAL, $CDCAIXA, $DTVENDA);
    }

    private function validaInsercaoVenda($CDFILIAL, $CDCAIXA, $DTVENDA, $NRSEQVENDA){
        if ($this->habilitaLog){
            $params = array(
                $CDFILIAL,
                $CDCAIXA,
                $NRSEQVENDA
            );
            $NRSEQVENDA = $this->entityManager->getConnection()->fetchAssoc("VALIDA_VENDA", $params);
            if (!empty($NRSEQVENDA)){
                $this->util->geraLog("\r\n" . '  REGISTRO DO BANCO: ' . $NRSEQVENDA['NRSEQVENDA'], $CDFILIAL, $CDCAIXA, $DTVENDA);
            }
            else {
                $this->util->geraLog("\r\n" . '  VENDA NÃO ENCONTRADA NO BANCO!', $CDFILIAL, $CDCAIXA, $DTVENDA);
            }
        }
    }

    // Quando o produto taxa de serviço está parametrizado, o valor do repique vai para ele independente da parametrização de taxa de serviço.
    private function adicionaProdutoTaxaServico($dadosTaxa, $NRVENDAREST, $NRCOMANDA, $REPIQUE, $VRTXSEVENDA, $TOTALVENDA){
        $itemTaxa = [];
        $itemTaxaExce = [];
        if ($dadosTaxa['IDTRATTAXASERV'] === 'P' && $dadosTaxa['CDPRODTAXASERV'] !== null && $dadosTaxa['IDCOMISVENDA'] != 'N' && ($VRTXSEVENDA > 0 || !empty($REPIQUE))) {
            // Calcula valor da taxa total (com o repique).
            $TOTALTAXA = floatval(bcdiv(str_replace(',','.',strval($VRTXSEVENDA)), '1', '2') + $REPIQUE);
            // Se o produto de taxa excedida estiver parametrizado e a taxa for maior que 10% do valor dos produtos.
            if ($dadosTaxa['CDPRODTXEXCEDE'] && floatval(bcdiv(strval($TOTALTAXA), strval($TOTALVENDA), 2)) > 0.1){
                // Calcula os valores das duas taxas.
                $VRTXSEVENDA = floatval(bcmul(strval($TOTALVENDA), strval(0.1), 2)); // Taxa 10%.
                $VRTAXAEXCED = floatval(bcsub(strval($TOTALTAXA), strval($VRTXSEVENDA), 2)); // Valor restante.
                $TOTALTAXA = $VRTXSEVENDA; // Ajusta para inserir depois.

                $itemTaxaExce = array(
                    'NRPRODCOMVEN'   => null,
                    'NMPRODUTO'      => $dadosTaxa['NMPRODEXCE'],
                    'CDPRODUTO'      => $dadosTaxa['CDPRODTXEXCEDE'],
                    'QTPRODVEND'     => 1,
                    'VRUNITVEND'     => 0,
                    'VRUNITVENDCL'   => 0,
                    'VRDESITVEND'    => 0,
                    'VRACRITVEND'    => $VRTAXAEXCED,
                    'IDSITUITEM'     => self::ITEM_APROVADO,
                    'IDTIPOITEM'     => null,
                    'OBSERVACOES'    => array(),
                    'IDTIPOCOMPPROD' => '0',
                    'IDIMPPRODUTO'   => '1',
                    'CDGRPOCOR'      => null,
                    'DSOBSITEMVENDA' => null,
                    'DSOBSPEDDIGITA' => null,
                    'CDSUPERVISOR'   => null,
                    'DTHRINCOMVEN'   => new \DateTime(),
                    'NRVENDAREST'    => $NRVENDAREST,
                    'NRCOMANDA'      => $NRCOMANDA,
                    'IDPRODPRODUZC'  => null,
                    'DSOBSDESCIT'    => null,
                    'IDORIGEMVENDA'  => null,
                    'CDGRPOCORDESCIT'=> null,
                    'CDVENDEDOR'     => null,
                    'CDPRODPROMOCAO' => null,
                    'REALSUBSIDY'    => 0,
                    'VOUCHER'        => null,
                    'CDCAMPCOMPGANHE' => null,
                    'DTINIVGCAMPCG'  => null,
                    'IDIMPPRGRPROMO' => 'N',
                    'IDOBSERVACAOAD' => 'N',
                    'TAXADESERVICO'  => true
                );
                $itemTaxaExce['PRECOFINAL'] = self::getItemPrice($itemTaxaExce, '0');
            }

            $itemTaxa = array(
                'NRPRODCOMVEN'   => null,
                'NMPRODUTO'      => $dadosTaxa['NMPRODUTO'],
                'CDPRODUTO'      => $dadosTaxa['CDPRODTAXASERV'],
                'QTPRODVEND'     => 1,
                'VRUNITVEND'     => 0,
                'VRUNITVENDCL'   => 0,
                'VRDESITVEND'    => 0,
                'VRACRITVEND'    => $TOTALTAXA,
                'IDSITUITEM'     => self::ITEM_APROVADO,
                'IDTIPOITEM'     => null,
                'OBSERVACOES'    => array(),
                'IDTIPOCOMPPROD' => '0',
                'IDIMPPRODUTO'   => '1',
                'CDGRPOCOR'      => null,
                'DSOBSITEMVENDA' => null,
                'DSOBSPEDDIGITA' => null,
                'CDSUPERVISOR'   => null,
                'DTHRINCOMVEN'   => new \DateTime(),
                'NRVENDAREST'    => $NRVENDAREST,
                'NRCOMANDA'      => $NRCOMANDA,
                'IDPRODPRODUZC'  => null,
                'DSOBSDESCIT'    => null,
                'IDORIGEMVENDA'  => null,
                'CDGRPOCORDESCIT'=> null,
                'CDVENDEDOR'     => null,
                'CDPRODPROMOCAO' => null,
                'REALSUBSIDY'    => 0,
                'VOUCHER'        => null,
                'CDCAMPCOMPGANHE' => null,
                'DTINIVGCAMPCG'  => null,
                'IDIMPPRGRPROMO' => 'N',
                'IDOBSERVACAOAD' => 'N',
                'TAXADESERVICO'  => true
            );
            $itemTaxa['PRECOFINAL'] = self::getItemPrice($itemTaxa, '0');
        }

        return array(
            'TAXAORIG' => $itemTaxa,
            'TAXAEXCE' => $itemTaxaExce
        );
    }

    public function getProductDetails($CDPRODUTO){
        $params = array(
            'CDPRODUTO' => $CDPRODUTO
        );

        return $this->entityManager->getConnection()->fetchAssoc('GET_PRODUCT_DETAILS', $params);
    }

    private function handleFidelity($CDFILIAL, $NRVENDAREST, $NRCOMANDA, $TOTALVENDABRT, $ITEMVENDA){
        try {
            $params = array(
                'NRVENDAREST' => $NRVENDAREST,
                'NRCOMANDA' => $NRCOMANDA
            );
            $result = $this->entityManager->getConnection()->fetchAssoc('CHECK_FIDELITY', $params);

            // Busca os dados do consumidor.
            if (empty($result['NRCPFFID4ALL']) ){
                return null;
            }
            else {
                $CPFFIDELIDADE = $result['NRCPFFID4ALL'];
                $FILIDADE = '4ALL';
                $fidelityDetails = $this->extratocons->consultaFidelidade($CPFFIDELIDADE, $FILIDADE);
            }

            if (!empty($result['CDPROMOFID4ALL'])){
                // Busca os dados do voucher.
                $dadosFilial = $this->entityManager->getConnection()->fetchAssoc("BUSCA_DADOS_FILIAL", array('CDFILIAL' => $CDFILIAL));
                $productString = array();
                foreach ($ITEMVENDA as $product){
                    if (!array_key_exists("TAXADESERVICO", $product)){
                        array_push($productString, $product['CDPRODUTO']);
                    }
                }
                $cupom = $this->extratocons->validaCupom($dadosFilial['NRINSJURFILI'], $result['NRCPFFID4ALL'], $result['CDPROMOFID4ALL'], $TOTALVENDABRT, $productString);
                if (!array_key_exists("data", $cupom)) throw new \Exception("Os dados do cupom informado não foram enviados após validação do mesmo.");

                $cupomId = $cupom['data']['_id'];
            } else {
                $cupomId = null;
            }

            return array(
                'fidelityDetails' => $fidelityDetails,
                'CODPROMO' => array(
                    'id' => $cupomId
                )
            );

        } catch (\Exception $e){
            throw new \Exception('Ocorreu um erro ao validar os dados da 4all: ' . $e->getMessage());
        }
    }

    private function calculaTotalProdutos($ITEMVENDA, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR){
        $total = 0;
        foreach ($ITEMVENDA as &$ITEM) {
            if (($ITEM['IDTIPOCOMPPROD'] === '3' && $ITEM['IDIMPPRODUTO'] === '2') || $ITEM['IDTIPOCOMPPROD'] === 'C') {
                // itens do combo
                foreach ($ITEM['itensCombo'] as &$itemComboAtual) {
                    $produtoDefinido = self::definePrecoProduto($itemComboAtual, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR);

                    if (!$produtoDefinido['error']){
                        $itemComboAtual = $produtoDefinido['produto'];
                        $itemComboAtual['VRACRITVEND'] = floatval(bcmul(str_replace(',','.',strval($itemComboAtual['VRACRITVEND'])), str_replace(',','.',strval($itemComboAtual['QTPRODCOMVEN'])), '2'));
                        $itemComboAtual['VRDESITVEND'] = floatval(bcmul(str_replace(',','.',strval($itemComboAtual['VRDESITVEND'])), str_replace(',','.',strval($itemComboAtual['QTPRODCOMVEN'])), '2'));

                        $total += self::getItemPrice($itemComboAtual, $ITEM['IDTIPOCOMPPROD']);
                    } else {
                        $retorno = $produtoDefinido;
                        break 2;
                    }
                }
            } else {
                // produto normal
                $produtoDefinido = self::definePrecoProduto($ITEM, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR);

                if (!$produtoDefinido['error']){
                    $ITEM = $produtoDefinido['produto'];
                    $ITEM['VRACRITVEND'] = floatval(bcmul(str_replace(',','.',strval($ITEM['VRACRITVEND'])), str_replace(',','.',strval($ITEM['QTPRODCOMVEN'])), '2'));
                    $ITEM['VRDESITVEND'] = floatval(bcmul(str_replace(',','.',strval($ITEM['VRDESITVEND'])), str_replace(',','.',strval($ITEM['QTPRODCOMVEN'])), '2'));
                    $total += self::getItemPrice($ITEM, $ITEM['IDTIPOCOMPPROD']);
                } else {
                    $retorno = $produtoDefinido;
                    break;
                }
            }
        }
        return $total;
    }
    
    public function setConsumidorVendaToNull($cdFilial, $cdCaixa, $nrSeqVenda){
        $params = [
            'CDFILIAL' => $cdFilial,
            'CDCAIXA' => $cdCaixa,
            'NRSEQVENDA' => $nrSeqVenda
        ];
        return $this->entityManager->getConnection()->executeQuery("SET_VENDA_CONSUMER_NULL", $params);
    }
    
    public function getTipoItemvenda($dadosProdutoLoja, $cdProduto){
        if(!($dadosProdutoLoja || $cdProduto)) return self::PRODUTO_TIPO_NORMAL;
        if(
            $dadosProdutoLoja['IDCOMISVENDA'] != 'N'
            && $dadosProdutoLoja['IDTRATTAXASERV'] == 'P'
        ){
            if($dadosProdutoLoja['CDPRODTAXASERV'] == $cdProduto ) return self::PRODUTO_TIPO_TAXA_SERVICO;
            if($dadosProdutoLoja['CDPRODTXEXCEDE'] == $cdProduto) return self::PRODUTO_TIPO_TAXA_EXCEDENTE;
        }
        if($dadosProdutoLoja['IDTRATTAXAENTR'] == 'P'){
            if($dadosProdutoLoja['CDPRODTAXAENTR'] == $cdProduto) return self::PRODUTO_TIPO_TAXA_ENTREGA;
        }
        return self::PRODUTO_TIPO_NORMAL;
    }
    
    public function getInfoTipoItemvenda($cdFilial, $cdLoja){
        return $this->entityManager->getConnection()->fetchAssociative(
        'GET_INFO_PRODUTO_LOJA', ['CDFILIAL'=> $cdFilial, 'CDLOJA'=> $cdLoja]
        );
    }

    private function validaIntegridadeTabelas($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $DTVENDA){
        $this->util->geraLog(' -Verificação de dados antes da transmissão.', $CDFILIAL, $CDCAIXA, $DTVENDA);
        $this->util->geraLog(' ---NRSEQVENDA REGISTRADO: ' . $NRSEQVENDA, $CDFILIAL, $CDCAIXA, $DTVENDA);

        $paramsVali = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRSEQVENDA' => $NRSEQVENDA
        );

        $this->util->geraLog(' -----Validando dados da venda:', $CDFILIAL, $CDCAIXA, $DTVENDA);
        $validaDadosVenda = $this->entityManager->getConnection()->fetchAssoc("GET_VENDA_NFCE", $paramsVali);
        if (empty($validaDadosVenda)) $this->util->geraLog(' -------Dados da venda não encontrados', $CDFILIAL, $CDCAIXA, $DTVENDA);
        else $this->util->geraLog(' -------Valor da venda: ' . $validaDadosVenda['VRTOTVENDA'], $CDFILIAL, $CDCAIXA, $DTVENDA);

        $this->util->geraLog(' -----Validando itens da venda:', $CDFILIAL, $CDCAIXA, $DTVENDA);
        $validaItensVenda = $this->entityManager->getConnection()->fetchAssoc("GET_DADOS_ITEMVENDA", $paramsVali);
        if (empty($validaItensVenda)) $this->util->geraLog(' -------Itens da venda não encontrados', $CDFILIAL, $CDCAIXA, $DTVENDA);
        else $this->util->geraLog(' -------Quantidade de itens: ' . $validaItensVenda['ITENS'], $CDFILIAL, $CDCAIXA, $DTVENDA);

        $this->util->geraLog(' -----Validando movimentações da venda:', $CDFILIAL, $CDCAIXA, $DTVENDA);
        $validaMoviVenda = $this->entityManager->getConnection()->fetchAssoc("GET_DADOS_MOVCAIXA", $paramsVali);
        if (empty($validaMoviVenda)) $this->util->geraLog(' -------Movimentações da venda não encontrados' . "\n", $CDFILIAL, $CDCAIXA, $DTVENDA);
        else $this->util->geraLog(' -------Movimentações: ' . $validaMoviVenda['MOVS'] . "\n", $CDFILIAL, $CDCAIXA, $DTVENDA);

        if (empty($validaDadosVenda) || empty($validaItensVenda) || empty($validaMoviVenda)){
            $this->util->geraLog(' -----Dados da venda incompletos - venda abortada.', $CDFILIAL, $CDCAIXA, $DTVENDA);
            throw new \Exception('ERRO: Venda não realizada. Ocorreu um erro na hora de gravar os dados no banco. Tente novamente.');
        }
    }

}
